

/*
This is a UI file (.ui.qml) that is intended to be edited in Qt Design Studio only.
It is supposed to be strictly declarative and only uses a subset of QML. If you edit
this file manually, you might introduce QML code that is not supported by Qt Design Studio.
Check out https://doc.qt.io/qtcreator/creator-quick-ui-forms.html for details on .ui.qml files.
*/
import QtQuick 2.9
import QtQuick.Dialogs 1.3
import QtQuick.Controls 2.15
import QtQuick.Window 2.2


Rectangle {
    id: rectangle
    color: "#d0f284"
    width: 1920
    height: 1080
    
    FileDialog {
        id: loadParamsDialog
        title: "Open params file"
        selectedNameFilter: "Json Files (*.json)"
        nameFilters: ["All files (*)",  "Json Files (*.json)"]
        onAccepted: {
            load_settings.read_file(loadParamsDialog.fileUrl)
        }
    }

    FileDialog {
        id: saveParamsDialog
        title: "Save Params file"
        selectedNameFilter: "Json Files (*.json)"
        nameFilters: ["All files (*)",  "Json Files (*.json)"]
        selectExisting: false
        onAccepted: {
            save_settings.read_file(saveParamsDialog.fileUrl)
        }
    }

    FileDialog {
        id: bgglassDialog
        title: "Search for integrated glass data file"
        selectedNameFilter: "Azimuthal integration files (*.chi)"
        nameFilters: ["All files (*)", "Azimuthal integration files (*.chi)", "Dat files (*.dat)", "Text files (*.txt)"]
        onAccepted: {
            load_bgglass.read_file(bgglassDialog.fileUrl)
        }
    }

    FileDialog {
        id: measglassDialog
        title: "Search for integrated glass data file"
        selectedNameFilter: "Azimuthal integration files (*.chi)"
        nameFilters: ["All files (*)", "Azimuthal integration files (*.chi)", "Dat files (*.dat)", "Text files (*.txt)"]
        onAccepted: {
            load_glass.read_file(measglassDialog.fileUrl)
        }
    }

    FileDialog {
        id: loadBGreloadDialog
        title: "Select directory path of integrated background files"
        selectedNameFilter: "Matlab (*.mat)"
        nameFilters: ["All files (*)", "Matlab (*.mat)"]
        onAccepted: {
            import_reload_bg.read_file(loadBGreloadDialog.fileUrl)
        }
    }

    FileDialog {
        selectFolder: true
        id: newProjectDialog
        folder: Qt.resolvedUrl("./../Projects/")
        title: "Set directory path for new project"
        onAccepted: {
            new_project.read_file(newProjectDialog.folder);
            tabButton.checked = true;
            tabButton.enabled = true;
            tabButton1.enabled = true;
            tabButton2.enabled = true;
            pane1.visible = true;
            toolButton2.enabled = true;
            toolButton3.enabled = true;
        }
    }

    FileDialog {
        selectFolder: true
        id: loadProjectDialog
        folder: Qt.resolvedUrl("./../Projects/")
        title: "Select directory path of project"
        onAccepted: {
            load_project.read_file(loadProjectDialog.folder);
            tabButton.checked = true;
            tabButton.enabled = true;
            tabButton1.enabled = true;
            tabButton2.enabled = true;
            pane1.visible = true;
            toolButton2.enabled = true;
            toolButton3.enabled = true;
        }
    }

    FileDialog {
        selectFolder: true
        id: loadBackgroundDialog
        title: "Select directory path of integrated background files"
        onAccepted: {
            load_background.read_file(loadBackgroundDialog.folder)
        }
    }

    FileDialog {
        selectFolder: true
        id: loadMEASDialog
        title: "Select directory path of integrated background files"
        onAccepted: {
            load_meas.read_file(loadMEASDialog.folder)
        }
    }

    FileDialog {
        selectFolder: true
        id: loadBGtempDialog
        title: "Select directory path of metadata background files"
        onAccepted: {
            load_bgtemp.read_file(loadBGtempDialog.folder)
        }
    }

    FileDialog {
        selectFolder: true
        id: loadMEAStempDialog
        title: "Select directory path of metadata files"
        onAccepted: {
            load_meastemp.read_file(loadMEAStempDialog.folder)
        }
    }

    Grid {
        id: grid
        x: 0
        y: 0
        width: 1920
        height: 1080
        rows: 2
        columns: 2
                    
        Rectangle {
            id: row
            x:0
            y:0
            width: 450
            height: 1080

            ToolBar {
                id: toolBar
                x: 0
                y: 0
                width: 450
                height: 30

                ToolButton {
                    font.pixelSize: 12
                    id: toolButton
                    x: 0
                    y: 0
                    width: 100
                    height: 30
                    text: qsTr("New Project")

                    contentItem: Text {

                            text: parent.text
                            font: parent.font
                            opacity: enabled ? 1.0 : 0.3
                            color: parent.down ? "#69bd18" : "#ffffff"
                            horizontalAlignment: Text.AlignHCenter
                            verticalAlignment: Text.AlignVCenter
                            elide: Text.ElideRight
                    }
                    
                    background: Rectangle {
                        id: rectaloadBackgroundDialogngle7
                        x:-6
                        width: 100
                        height: 30
                        color: parent.down ? "#222222" : "#000000"  
                    }

                    onClicked: {
                        newProjectDialog.visible = true
                    }
                }

                ToolButton {
                    font.pixelSize: 12
                    id: toolButton1
                    x: 100
                    y: 0
                    width: 100
                    height: 30
                    text: qsTr("Load Project")

                    contentItem: Text {
                            text: parent.text
                            font: parent.font
                            opacity: enabled ? 1.0 : 0.3
                            color: parent.down ? "#2d94b8" : "#ffffff"
                            horizontalAlignment: Text.AlignHCenter
                            verticalAlignment: Text.AlignVCenter
                            elide: Text.ElideRight
                    }
                    
                    background: Rectangle {
                        id: toolbarrectangle2
                        x:-6
                        width: 100
                        height: 30
                        color: parent.down ? "#222222" : "#000000"  
                    }

                    onClicked: {
                        loadProjectDialog.visible = true
                    }
                }

                ToolButton {
                    font.pixelSize: 12
                    id: toolButton2
                    objectName: "save_params"
                    enabled: false
                    x: 200
                    y: 0
                    width: 100
                    height: 30
                    text: qsTr("Save Params")
                    onClicked: {
                        saveParamsDialog.visible = true
                    }

                    contentItem: Text {
                            text: parent.text
                            font: parent.font
                            opacity: enabled ? 1.0 : 0.3
                            color: parent.down ? "#2d94b8" : "#ffffff"
                            horizontalAlignment: Text.AlignHCenter
                            verticalAlignment: Text.AlignVCenter
                            elide: Text.ElideRight
                    }
                    
                    background: Rectangle {
                        id: toolbarrectangle3
                        x:-6
                        width: 100
                        height: 30
                        color: parent.down ? "#222222" : "#000000"  
                    }
                }

                ToolButton {
                    font.pixelSize: 12
                    id: toolButton3
                    objectName: "load_params"
                    enabled: false
                    x: 300
                    y: 0
                    width: 100
                    height: 30
                    text: qsTr("Load Params")
                    onClicked: {
                        loadParamsDialog.visible = true
                    }

                    contentItem: Text {
                            text: parent.text
                            font: parent.font
                            opacity: enabled ? 1.0 : 0.3
                            color: parent.down ? "#2d94b8" : "#ffffff"
                            horizontalAlignment: Text.AlignHCenter
                            verticalAlignment: Text.AlignVCenter
                            elide: Text.ElideRight
                    }
                    
                    background: Rectangle {
                        x:-6
                        width: 100
                        height: 30
                        color: parent.down ? "#222222" : "#000000"  
                    }
                }

                ToolButton {
                    font.pixelSize: 12
                    id: toolButton4
                    objectName: "settings"
                    x: 400
                    y: 0
                    width: 50
                    height: 30
                    // onClicked: {
                    //     loadParamsDialog.visible = true
                    // }
                    hoverEnabled: false
                    opacity: 1
                    text: qsTr("Button")
                    highlighted: false
                    icon.color: "#ffffff"
                    icon.source: "../die-einstellungen.png"
                    icon.width: 14
                    flat: true
                    icon.height: 14

                    background: Rectangle {
                        x:-6
                        width: 56
                        height: 30
                        color: parent.down ? "#222222" : "#000000"  
                    }

                    onClicked: {
                        general_settings.visible = true
                    }
                }
            }

            Rectangle {
                id: rectangle1
                x: 0
                y: 30
                width: 450
                height: 1080
                

                TabBar {
                    id: tabBar
                    x: 0
                    y: 0
                    width: 450
                    height: 50
                    contentHeight: 50
                    contentWidth: 450
                    wheelEnabled: false
                    activeFocusOnTab: false
                    background: Rectangle {
                            color: tabBar.currentIndex == 0 ? "#111111" : "#333333"  
                    }

                    TabButton {
                        font.pixelSize: 12
                        id: tabButton
                        x: 0
                        y: 0
                        width: 150
                        height: 50
                        text: qsTr("Background")
                        enabled: false
                        contentItem: Text {
                            text: tabButton.text
                            font: tabButton.font
                            opacity: enabled ? 1.0 : 0.3
                            color: tabButton.down ? "#ffffff" : "#69bd18"
                            horizontalAlignment: Text.AlignHCenter
                            verticalAlignment: Text.AlignVCenter
                            elide: Text.ElideRight
                        }
                        background: Rectangle {
                            color: tabBar.currentIndex == 0 ? "#111111" : "#333333"
                            opacity: enabled ? 1 : 0.3
                            border.color: tabButton.down ? "#ffffff" : "#ffffff"  
                            border.width: 0
                        }

                        checked: false
                        checkable: true
                        display: AbstractButton.TextBesideIcon
                        icon.color: "#a1005a"
                    }

                    TabButton {
                        font.pixelSize: 12
                        id: tabButton1
                        y: 0
                        width: 150
                        height: 50
                        text: "Measurment"
                        enabled: false
                        contentItem: Text {
                            text: tabButton1.text
                            font: tabButton1.font
                            opacity: enabled ? 1.0 : 0.3
                            color: tabButton1.down ? "#ffffff" : "#69bd18"
                            horizontalAlignment: Text.AlignHCenter
                            verticalAlignment: Text.AlignVCenter
                            elide: Text.ElideRight
                        }
                        background: Rectangle {
                            color: tabBar.currentIndex == 1 ? "#111111" : "#333333"
                            opacity: enabled ? 1 : 0.3
                            border.color: tabButton1.down ? "#ffffff" : "#ffffff"
                            border.width: 0  
                        }

                        checked: false
                        checkable: true
                    }

                    TabButton {
                        font.pixelSize: 12
                        id: tabButton2
                        x: 2
                        y: 0
                        width: 150
                        height: 50
                        text: "Batch plot PDF"
                        enabled: false
                        contentItem: Text {
                            text: tabButton2.text
                            font: tabButton2.font
                            opacity: enabled ? 1.0 : 0.3
                            color: tabButton2.down ? "#ffffff" : "#69bd18"
                            horizontalAlignment: Text.AlignHCenter
                            verticalAlignment: Text.AlignVCenter
                            elide: Text.ElideRight
                        }
                        background: Rectangle {
                            color: tabBar.currentIndex == 2 ? "#111111" : "#333333"
                            opacity: enabled ? 1 : 0.3
                            border.color: tabButton2.down ? "#ffffff" : "#ffffff"  
                            border.width: 0
                        }
                        checked: false
                        checkable: true
                    }
                }

                Flickable {
                    id: scrollViewPane1
                    x: 0
                    y: 80
                    width:  450
                    height: Window.height
                    clip: true
                    contentHeight: pane1.height  
                    visible: tabButton.checked

                    Pane {
                        id: pane1
                        objectName: "pane1"
                        x: 0
                        y: 0
                        width: 450
                        height: 820
                        visible: tabButton.checked
                        background: Rectangle {
                                color: "#ffffff"
                        }


                        Rectangle{
                            id: container6
                            height: contentHeight
                            width: 450
                            x:0
                            y:10

                            Button {
                                font.pixelSize: 12
                                id: button8
                                objectName: "loadBG"
                                hoverEnabled: false
                                x: 63
                                y: 50
                                width: 300
                                height: 50
                                text: qsTr("Load Background")
                                font.styleName: "Fett"
                                checked: false
                                flat: false
                                enabled: true
                                contentItem: Text {
                                    text: button8.text
                                    font: button8.font
                                    opacity: enabled ? 1.0 : 0.3
                                    color: button8.down ? "#ffffff" : "#69bd18"
                                    horizontalAlignment: Text.AlignHCenter
                                    verticalAlignment: Text.AlignVCenter
                                    elide: Text.ElideRight
                                }
                                background: Rectangle {
                                    color: button8.down == true ? "#111111" : "#333333"
                                    radius: 2
                                }

                                onClicked: {
                                    loadBackgroundDialog.visible = true
                                }
                            }

                            
                            CheckBox {
                                font.pixelSize: 12
                                indicator.height: 12
                                indicator.width: 12
                                id: checkBox2
                                x: 0
                                y: 120
                                text: qsTr("Import Background")
                                checked: false
                            }

                            Button {
                                font.pixelSize: 12
                                id: importBG1
                                hoverEnabled: false
                                x: 177
                                y: 120
                                width: 220
                                height: 30
                                text: qsTr("Search in projects")
                                font.styleName: "Fett"
                                checked: false
                                flat: false
                                visible: checkBox2.checked
                                contentItem: Text {
                                    text: importBG1.text
                                    font: importBG1.font
                                    opacity: enabled ? 1.0 : 0.3
                                    color: importBG1.down ? "#ffffff" : "#69bd18"
                                    horizontalAlignment: Text.AlignHCenter
                                    verticalAlignment: Text.AlignVCenter
                                    elide: Text.ElideRight
                                }
                                background: Rectangle {
                                    color: importBG1.down == true ? "#111111" : "#333333"
                                    radius: 2
                                }

                                onClicked: {
                                    loadBGreloadDialog.visible = true
                                }
                            }

                            CheckBox {
                                font.pixelSize: 12
                                indicator.height: 12
                                indicator.width: 12
                                id: checkBox4
                                x: 0
                                y: 0
                                text: qsTr("Define time path for BG")
                                checked: false
                            }

                            Button {
                                font.pixelSize: 12
                                id: importBGtime
                                hoverEnabled: false
                                x: 177
                                y: 0
                                width: 220
                                height: 30
                                text: qsTr("Search time directory")
                                font.styleName: "Fett"
                                checked: false
                                flat: false
                                visible: checkBox4.checked
                                contentItem: Text {
                                    text: parent.text
                                    font: parent.font
                                    opacity: enabled ? 1.0 : 0.3
                                    color: parent.down ? "#ffffff" : "#69bd18"
                                    horizontalAlignment: Text.AlignHCenter
                                    verticalAlignment: Text.AlignVCenter
                                    elide: Text.ElideRight
                                }
                                background: Rectangle {
                                    color: importBGtime.down == true ? "#111111" : "#333333"
                                    radius: 2
                                }

                                onClicked: {
                                    loadBGtempDialog.visible = true
                                }
                            }
                        }
                        

                        Rectangle{
                            id: container5
                            height: contentHeight
                            width: 450
                            x:0
                            y:150

                            TextField {
                                font.pixelSize: 12
                                id: textField48
                                objectName: "textField48"
                                x: 87
                                y: 30
                                width: 60
                                height: 25
                                text: textField22.text
                                topPadding: 5
                                horizontalAlignment: Text.AlignHCenter
                                placeholderText: qsTr("1")
                            }

                            Text {
                                id: text38
                                x: 1
                                y: 30
                                width: 80
                                height: 25
                                text: qsTr("Averaging:")
                                font.pixelSize: 12
                                horizontalAlignment: Text.AlignHCenter
                                verticalAlignment: Text.AlignVCenter
                            }

                            TextField {
                                font.pixelSize: 12
                                id: textField49
                                objectName: "textField49"
                                x: 263
                                y: 30
                                width: 60
                                height: 25
                                topPadding: 5
                                text: textField23.text
                                horizontalAlignment: Text.AlignHCenter
                                placeholderText: qsTr("25")
                            }

                            Text {
                                id: text39
                                x: 177
                                y: 30
                                width: 80
                                height: 25
                                text: qsTr("Qmask:")
                                font.pixelSize: 12
                                horizontalAlignment: Text.AlignHCenter
                                verticalAlignment: Text.AlignVCenter
                            }

                            CheckBox {
                                font.pixelSize: 12
                                indicator.height: 12
                                indicator.width: 12
                                id: checkBox3
                                x: 0
                                y: 100
                                text: qsTr("Empty glass")
                                checked: false
                            }

                            Button {
                                font.pixelSize: 12
                                id: button12
                                hoverEnabled: false
                                x: 177
                                y: 105
                                width: 220
                                height: 30
                                visible: checkBox3.checked
                                text: qsTr("Load Glass File")
                                font.styleName: "Fett"
                                checked: false
                                flat: false
                                contentItem: Text {
                                    text: parent.text
                                    font: parent.font
                                    opacity: enabled ? 1.0 : 0.3
                                    color: parent.down ? "#ffffff" : "#69bd18"
                                    horizontalAlignment: Text.AlignHCenter
                                    verticalAlignment: Text.AlignVCenter
                                    elide: Text.ElideRight
                                }
                                background: Rectangle {
                                    color: parent.down == true ? "#111111" : "#333333"
                                    radius: 2
                                }

                                onClicked: {
                                    bgglassDialog.visible = true
                                }
                            }

                            Slider {
                                id: slider32
                                x: -73
                                y: 160
                                width: 480
                                height: 20
                                visible: checkBox3.checked
                                scale: 0.7
                                value: textField46.text
                                handle: Rectangle {
                                    color: parent.pressed ? "#aaf661" : "white" 
                                    border.color: "#69bd18"
                                    border.width: 2
                                    implicitWidth: 24
                                    implicitHeight: 24
                                    radius: 12
                                    x: ((parent.value-parent.from)*(parent.width-implicitWidth))/(parent.to-parent.from)
                                }
                            }

                            Text {
                                id: text21
                                x: 6
                                y: 136
                                width: 29
                                height: 13
                                visible: checkBox3.checked
                                text: "Glass Scale"
                                font.pixelSize: 12
                            }

                            TextField {
                                font.pixelSize: 12
                                id: textField46
                                objectName: "textField46"
                                x: 340
                                y: 160
                                width: 60
                                height: 25
                                topPadding: 5
                                visible: checkBox3.checked
                                text: slider32.value
                                onTextChanged: if (length > 8) remove(8, length);
                                placeholderText: qsTr("Text Field")
                                padding: 2
                                leftPadding: 10
                            }
                        }

                        TextField {
                            font.pixelSize: 12
                            id: textField43
                            objectName: "textField43"
                            x: 0
                            y: 520
                            width: 130
                            height: 30
                            topPadding: 7
                            leftPadding: 10
                            text: "XRD Scan No."
                            placeholderText: qsTr("Text Field")
                        }

                        Text {
                                id: text33
                                x: 6
                                y: 560
                                width: 29
                                height: 13
                                text: "Total scans: ----"
                                font.pixelSize: 12
                            }

                        ColorButton3 {
                            font.pixelSize: 14
                            id: colorButton9
                            objectName: '1D_vs_time'
                            x: 178
                            y: 600
                            width: 220
                            height: 30
                            text: "XRD 1D vs time"
                        }

                        ColorButton {
                            font.pixelSize: 14
                            id: colorButton10
                            objectName: '1D_slice'
                            x: 178
                            y: 520
                            width: 220
                            height: 30
                            text: "XRD Slice Plot"
                            highlighted: false
                            checkable: false
                            icon.color: "#ffffff"
                            autoExclusive: false
                            checked: false
                            flat: false
                            autoRepeat: false
                        }
                        

                        ColorButton2 {
                            font.pixelSize: 14
                            id: colorButton11
                            objectName: '2D_vs_time'
                            x: 178
                            y: 560
                            width: 220
                            height: 30
                            text: "XRD 2D vs time"
                            transformOrigin: Item.TopLeft
                        }

                        Button {
                            font.pixelSize: 12
                            id: button9
                            hoverEnabled: false
                            x: 148
                            y: 525
                            width: 20
                            height: 20
                            opacity: 1
                            text: qsTr("Button")
                            highlighted: false
                            icon.color: "#ffffff"
                            Image {
                                id: image2
                                width: 20
                                height: 20
                                source: "../die-einstellungen.png"
                                fillMode: Image.PreserveAspectFit
                            }
                            icon.source: "../die-einstellungen.png"
                            icon.width: 20
                            flat: true
                            icon.height: 20
                        }

                        Rectangle {
                            id: rectangle6
                            x: -7
                            y: 660
                            width: 440
                            height: 4
                            color: "#353637"
                            radius: 2
                        }

                        Text {
                            id: text1
                            x: 63
                            y: 465
                            width: 300
                            height: 40
                            text: qsTr("Plotting of the BG alone")
                            font.pixelSize: 24
                            horizontalAlignment: Text.AlignHCenter
                            font.bold: true
                            font.underline: false
                        }
                    }
                }

                Flickable {
                    id: scrollViewPane2
                    x: 0
                    y: 80
                    width: 450
                    height: Window.height
                    clip: true
                    contentHeight: pane2.height
                    visible: tabButton1.checked

                    Pane {
                        id: pane2
                        x: 0
                        y: 0
                        width: 450
                        height: 1600
                        visible: tabButton1.checked
                        background: Rectangle {
                                color: "#ffffff"
                        }

                        Button {
                            font.pixelSize: 12
                            id: button3
                            objectName: 'loadMEAS'
                            hoverEnabled: false
                            x: 63
                            y: 60
                            width: 300
                            height: 50
                            text: qsTr("Load Measurment")
                            font.styleName: "Fett"
                            flat: false
                            checked: false
                            contentItem: Text {
                                text: button3.text
                                font: button3.font
                                opacity: enabled ? 1.0 : 0.3
                                color: button3.down ? "#ffffff" : "#69bd18"
                                horizontalAlignment: Text.AlignHCenter
                                verticalAlignment: Text.AlignVCenter
                                elide: Text.ElideRight
                            }
                            background: Rectangle {
                                color: button3.down == true ? "#111111" : "#333333"
                                radius: 2
                            }

                            onClicked: {
                                loadMEASDialog.visible = true
                            }
                        }

                        CheckBox {
                            font.pixelSize: 12
                            indicator.height: 12
                            indicator.width: 12
                            id: checkBox5
                            x: 0
                            y: 10
                            text: qsTr("Define time path for Meas")
                            checked: false
                        }

                        Button {
                            font.pixelSize: 12
                            id: importMEAStime
                            hoverEnabled: false
                            x: 177
                            y: 10
                            width: 220
                            height: 30
                            text: qsTr("Search time directory")
                            font.styleName: "Fett"
                            checked: false
                            flat: false
                            visible: checkBox5.checked
                            contentItem: Text {
                                text: parent.text
                                font: parent.font
                                opacity: enabled ? 1.0 : 0.3
                                color: parent.down ? "#ffffff" : "#69bd18"
                                horizontalAlignment: Text.AlignHCenter
                                verticalAlignment: Text.AlignVCenter
                                elide: Text.ElideRight
                            }
                            background: Rectangle {
                                color: importBGtime.down == true ? "#111111" : "#333333"
                                radius: 2
                            }

                            onClicked: {
                                loadMEAStempDialog.visible = true
                            }
                        }


                        Rectangle{
                            id: container1
                            height: contentHeight
                            width: 450
                            x:0
                            y:320
                        

                            Slider {
                                id: slider9
                                x: -72
                                y: 630
                                width: 480
                                height: 20
                                stepSize: 0.01
                                to: 30
                                from: 0.01
                                scale: 0.7
                                value: textField11.text
                                handle: Rectangle {
                                    color: parent.pressed ? "#aaf661" : "white" 
                                    border.color: "#69bd18"
                                    border.width: 2
                                    implicitWidth: 24
                                    implicitHeight: 24
                                    radius: 12
                                    x: ((parent.value-parent.from)*(parent.width-implicitWidth))/(parent.to-parent.from)
                                }
                            }

                            Text {
                                id: text9
                                x: 1
                                y: 616
                                width: 29
                                height: 13
                                text: qsTr("Qmax_inst")
                                font.pixelSize: 12
                            }

                            TextField {
                                font.pixelSize: 12
                                id: textField11
                                objectName: 'textField11'
                                x: 340
                                y: 630
                                width: 60
                                height: 25
                                topPadding: 5
                                text: slider9.value
                                onTextChanged: if (length > 4) remove(4, length);
                                placeholderText: qsTr("Text Field")
                                padding: 2
                                leftPadding: 10
                                font.pointSize: 9
                            }

                            Slider {
                                id: slider10
                                objectName: "textField12"
                                x: -72
                                y: 670
                                width: 480
                                height: 20
                                stepSize: 0.01
                                to: 3
                                from: 0.01
                                scale: 0.7
                                value: textField12.text
                                handle: Rectangle {
                                    color: parent.pressed ? "#aaf661" : "white" 
                                    border.color: "#69bd18"
                                    border.width: 2
                                    implicitWidth: 24
                                    implicitHeight: 24
                                    radius: 12
                                    x: ((parent.value-parent.from)*(parent.width-implicitWidth))/(parent.to-parent.from)
                                }
                            }

                            Text {
                                id: text10
                                x: 0
                                y: 656
                                width: 29
                                height: 13
                                text: qsTr("Qmin")
                                font.pixelSize: 12
                            }

                            TextField {
                                font.pixelSize: 12
                                id: textField12
                                objectName: "textField12"
                                x: 340
                                y: 670
                                width: 60
                                height: 25
                                topPadding: 5
                                text: slider10.value
                                baselineOffset: 15
                                onTextChanged: if (length > 4) remove(4, length);
                                placeholderText: qsTr("Text Field")
                                padding: 2
                                leftPadding: 10
                            }

                            Slider {
                                id: slider11
                                x: -72
                                y: 710
                                width: 480
                                height: 20
                                stepSize: 0.01
                                to: slider9.value
                                scale: 0.7
                                value: textField13.text
                                handle: Rectangle {
                                    color: parent.pressed ? "#aaf661" : "white" 
                                    border.color: "#69bd18"
                                    border.width: 2
                                    implicitWidth: 24
                                    implicitHeight: 24
                                    radius: 12
                                    x: ((parent.value-parent.from)*(parent.width-implicitWidth))/(parent.to-parent.from)
                                }
                            }

                            Text {
                                id: text11
                                x: 0
                                y: 696
                                width: 29
                                height: 13
                                text: qsTr("Qmax")
                                font.pixelSize: 12
                            }

                            TextField {
                                font.pixelSize: 12
                                id: textField13
                                objectName: 'textField13'
                                x: 340
                                y: 710
                                width: 60
                                height: 25
                                topPadding: 5
                                text: slider11.value
                                onTextChanged: if (length > 4) remove(4, length);
                                placeholderText: qsTr("Text Field")
                                padding: 2
                                leftPadding: 10
                            }

                            Slider {
                                id: slider12
                                x: -72
                                y: 750
                                width: 480
                                height: 20
                                stepSize: 1
                                to: 100
                                from: 1
                                scale: 0.7
                                value: textField14.text
                                handle: Rectangle {
                                    color: parent.pressed ? "#aaf661" : "white" 
                                    border.color: "#69bd18"
                                    border.width: 2
                                    implicitWidth: 24
                                    implicitHeight: 24
                                    radius: 12
                                    x: ((parent.value-parent.from)*(parent.width-implicitWidth))/(parent.to-parent.from)
                                }
                            }

                            Text {
                                id: text12
                                x: 0
                                y: 736
                                width: 29
                                height: 13
                                text: qsTr("Rmax")
                                font.pixelSize: 12
                            }

                            TextField {
                                font.pixelSize: 12
                                id: textField14
                                objectName: 'textField14'
                                x: 340
                                y: 750
                                width: 60
                                height: 25
                                topPadding: 5
                                text: slider12.value
                                onTextChanged: if (length > 4) remove(4, length);
                                placeholderText: "30"
                                padding: 2
                                leftPadding: 10
                            }

                            Slider {
                                id: slider13
                                x: -72
                                y: 790
                                width: 480
                                height: 20
                                stepSize: 0.01
                                to: 3
                                scale: 0.7
                                value: textField15.text
                                handle: Rectangle {
                                    color: parent.pressed ? "#aaf661" : "white" 
                                    border.color: "#69bd18"
                                    border.width: 2
                                    implicitWidth: 24
                                    implicitHeight: 24
                                    radius: 12
                                    x: ((parent.value-parent.from)*(parent.width-implicitWidth))/(parent.to-parent.from)
                                }
                            }

                            Text {
                                id: text13
                                x: 0
                                y: 776
                                width: 29
                                height: 13
                                text: qsTr("Rpoly")
                                font.pixelSize: 12
                            }

                            TextField {
                                font.pixelSize: 12
                                id: textField15
                                objectName: 'textField15'
                                x: 340
                                y: 790
                                width: 60
                                height: 25
                                topPadding: 5
                                text: slider13.value
                                onTextChanged: if (length > 4) remove(4, length);
                                padding: 2
                                leftPadding: 10
                            }

                            TextField {
                                font.pixelSize: 12
                                id: textField16
                                objectName: "textField16"
                                x: 0
                                y: 570
                                width: 398
                                height: 30
                                topPadding: 7
                                leftPadding: 10
                                text: "Composition"
                                placeholderText: qsTr("Text Field")
                            }

                            CheckBox {
                                font.pixelSize: 12
                                indicator.height: 12
                                indicator.width: 12
                                id: checkBox17
                                objectName: "static_bg"
                                x: 0
                                y: 830
                                text: qsTr("Use static background")
                                checked: false
                            }
                            
                            TextField {
                                font.pixelSize: 12
                                id: textField27
                                objectName: "textField27"
                                x: 270
                                y: 845
                                width: 130
                                height: 30
                                topPadding: 7
                                leftPadding: 10
                                text: "BG data point"
                                visible: checkBox17.checked
                                placeholderText: qsTr("BG data point")
                            }
                            
                            
                            Slider {
                                id: slider17
                                x: -72
                                y: 890
                                width: 480
                                height: 20
                                stepSize: 0.0001
                                to: 2
                                scale: 0.7
                                value: textField25.text
                                visible: checkBox17.checked
                                handle: Rectangle {
                                    color: parent.pressed ? "#aaf661" : "white" 
                                    border.color: "#69bd18"
                                    border.width: 2
                                    implicitWidth: 24
                                    implicitHeight: 24
                                    radius: 12
                                    x: ((parent.value-parent.from)*(parent.width-implicitWidth))/(parent.to-parent.from)
                                }
                            }

                            Text {
                                id: text22
                                x: 0
                                y: 876
                                width: 29
                                height: 13
                                visible: checkBox17.checked
                                text: qsTr("background scale")
                                font.pixelSize: 12
                            }
                            

                            TextField {
                                font.pixelSize: 12
                                id: textField25
                                objectName: 'textField25'
                                x: 340
                                y: 890
                                width: 60
                                height: 25
                                topPadding: 5
                                text: slider17.value
                                visible: checkBox17.checked
                                onTextChanged: if (length > 6) remove(6, length);
                                padding: 2
                                leftPadding: 10
                                placeholderText: "0.99"
                            }
                            
                            TextField {
                                font.pixelSize: 12
                                id: textField26
                                objectName: "textField26"
                                x: 0
                                y: 930
                                width: 130
                                height: 30
                                topPadding: 7
                                leftPadding: 10
                                text: "PDF scan No."
                                placeholderText: qsTr("Text Field")
                            }

                            Text {
                                id: text36
                                x: 6
                                y: 970
                                width: 29
                                height: 13
                                text: "Total scans: ----"
                                font.pixelSize: 12
                            }

                            ColorButton {
                                font.pixelSize: 14
                                id: colorButton4
                                objectName: 'create_pdf'
                                x: 178
                                y: 930
                                width: 220
                                height: 30
                                text: "Create PDF"
                                icon.color: "#ffffff"
                            }
                            
                            ColorButton2 {
                                font.pixelSize: 14
                                id: colorButton8
                                objectName: "batch_PDF"
                                x: 178
                                y: 970
                                width: 220
                                height: 30
                                text: "Batch process"
                                transformOrigin: Item.TopLeft
                            }


                            Rectangle {
                                id: rectangle4
                                x: -7
                                y: 540
                                width: 440
                                height: 4
                                color: "#353637"
                                radius: 2
                            }
                        }


                        Rectangle{
                            id: container2
                            height: contentHeight
                            width: 450
                            x:0
                            y:320

                            TextField {
                                font.pixelSize: 12
                                id: textField17
                                objectName: "textField17"
                                x: 0
                                y: 400
                                width: 130
                                height: 30
                                topPadding: 7
                                leftPadding: 10
                                text: "XRD scan No."
                                placeholderText: qsTr("Text Field")
                            }
                            
                            Text {
                                id: text35
                                x: 6
                                y: 440
                                width: 29
                                height: 13
                                text: "Total scans: ----"
                                font.pixelSize: 12
                            }

                            ColorButton3 {
                                font.pixelSize: 14
                                id: colorButton3
                                objectName: "XRD_1D_vs_time_meas"
                                x: 178
                                y: 480
                                width: 220
                                height: 30
                                text: "XRD 1D vs time"
                            }

                            ColorButton {
                                font.pixelSize: 14
                                id: colorButton1
                                objectName: "XRD_1D_vs_ref_meas"
                                x: 178
                                y: 400
                                width: 220
                                height: 30
                                text: "XRD vs Reference"
                                autoRepeat: false
                                autoExclusive: false
                                checkable: false
                                checked: false
                                flat: false
                                highlighted: false
                                icon.color: "#ffffff"
                            }

                            ColorButton2 {
                                font.pixelSize: 14
                                id: colorButton2
                                objectName: "XRD_2D_vs_time_meas"
                                x: 178
                                y: 440
                                width: 220
                                height: 30
                                text: "XRD 2D vs time"
                                transformOrigin: Item.TopLeft
                            }

                            Button {
                                font.pixelSize: 12
                                id: button1
                                hoverEnabled: false
                                x: 148
                                y: 405
                                width: 20
                                height: 20
                                opacity: 1
                                text: qsTr("Button")
                                icon.height: 20
                                icon.width: 20
                                icon.color: "#ffffff"
                                icon.source: "../die-einstellungen.png"
                                highlighted: false
                                flat: true

                                Image {
                                    id: image
                                    width: 20
                                    height: 20
                                    source: "../die-einstellungen.png"
                                    fillMode: Image.PreserveAspectFit
                                }
                            }
                        }

                        
                        Rectangle {
                            id: container4
                            height: contentHeight
                            width: 450
                            x:0
                            y:0

                            CheckBox {
                                font.pixelSize: 12
                                indicator.height: 12
                                indicator.width: 12
                            id: checkBox6
                            objectName: "XRD_Analyses"
                            x: 0
                            y: 420
                            text: qsTr("XRD Analyses")
                            checked: false
                            }

                            Text {
                                id: text31
                                x: 6
                                y: 446
                                width: 29
                                height: 13
                                visible: checkBox6.checked
                                text: "Filter Type"
                                font.pixelSize: 12
                            }

                            

                            CheckBox {
                                font.pixelSize: 12
                                indicator.height: 12
                                indicator.width: 12
                                id: checkBox7
                                objectName: "filter_type1"
                                x: 10
                                y: 470
                                visible: checkBox6.checked
                                text: qsTr("None")
                                checked: true
                                function checkboxes1() {
                                    if (checkBox7.checked == true){
                                        checkBox8.checked = false
                                        checkBox9.checked = false
                                    }
                                }
                                onClicked: checkboxes1()
                            }

                            CheckBox {
                                font.pixelSize: 12
                                indicator.height: 12
                                indicator.width: 12
                                id: checkBox8
                                objectName: "filter_type2"
                                x: 130
                                y: 470
                                visible: checkBox6.checked
                                text: qsTr("Whittaker")
                                checked: false
                                function checkboxes1() {
                                    if (checkBox8.checked == true){
                                        checkBox7.checked = false
                                        checkBox9.checked = false
                                        slider33.to = 100
                                    }
                                }
                                onClicked: checkboxes1()
                            }

                            CheckBox {
                                font.pixelSize: 12
                                indicator.height: 12
                                indicator.width: 12
                                id: checkBox9
                                objectName: "filter_type3"
                                x: 250
                                y: 470
                                visible: checkBox6.checked
                                text: qsTr("Sav-Gol")
                                checked: false
                                function checkboxes1() {
                                    if (checkBox9.checked == true){
                                        checkBox8.checked = false
                                        checkBox7.checked = false
                                        slider33.to = 30
                                    }
                                }
                                onClicked: checkboxes1()
                            }

                            Slider {
                                id: slider33
                                x: -73
                                y: 520
                                width: 480
                                height: 20
                                visible: checkBox6.checked
                                scale: 0.7
                                to: 100
                                from: 0
                                stepSize: 1
                                value: textField50.text
                                handle: Rectangle {
                                    color: parent.pressed ? "#aaf661" : "white" 
                                    border.color: "#69bd18"
                                    border.width: 2
                                    implicitWidth: 24
                                    implicitHeight: 24
                                    radius: 12
                                    x: ((parent.value-parent.from)*(parent.width-implicitWidth))/(parent.to-parent.from)
                                }
                            }

                            Text {
                                id: text30
                                x: 6
                                y: 506
                                width: 29
                                height: 13
                                visible: checkBox6.checked
                                text: "Filter strenght"
                                font.pixelSize: 12
                            }

                            TextField {
                                font.pixelSize: 12
                                id: textField50
                                objectName: "textField50"
                                x: 340
                                y: 520
                                width: 60
                                height: 25
                                topPadding: 5
                                visible: checkBox6.checked
                                text: slider33.value
                                onTextChanged: if (length > 6) remove(6, length);
                                placeholderText: qsTr("Text Field")
                                padding: 2
                                leftPadding: 10
                            }

                            Text {
                                id: text32
                                x: 6
                                y: 556
                                width: 29
                                height: 13
                                visible: checkBox6.checked
                                text: "Polynomal degree"
                                font.pixelSize: 12
                            }

                            CheckBox {
                                font.pixelSize: 12
                                indicator.height: 12
                                indicator.width: 12
                                id: checkBox10
                                objectName: "filter_degree2"
                                x: 130
                                y: 556
                                visible: checkBox6.checked
                                text: qsTr("2")
                                checked: false
                                function checkboxes1() {
                                    if (checkBox10.checked == true){
                                        checkBox11.checked = false
                                    }
                                }
                                onClicked: checkboxes1()
                            }

                            CheckBox {
                                font.pixelSize: 12
                                indicator.height: 12
                                indicator.width: 12
                                id: checkBox11
                                objectName: "filter_degree3"
                                x: 250
                                y: 556
                                visible: checkBox6.checked
                                text: qsTr("3")
                                checked: true
                                function checkboxes1() {
                                    if (checkBox11.checked == true){
                                        checkBox10.checked = false
                                    }
                                }
                                onClicked: checkboxes1()
                            }
                            
                            Text {
                                id: text34
                                x: 6
                                y: 606
                                width: 29
                                height: 13
                                visible: checkBox6.checked
                                text: "Baseline Correction Type"
                                font.pixelSize: 12
                            }

                            
                            CheckBox {
                                font.pixelSize: 12
                                indicator.height: 12
                                indicator.width: 12
                                id: checkBox12
                                objectName: "baseline_type1"
                                x: 10
                                y: 626
                                visible: checkBox6.checked
                                text: qsTr("None")
                                checked: true
                                function checkboxes1() {
                                    if (checkBox12.checked == true){
                                        checkBox13.checked = false
                                    }
                                }
                                onClicked: checkboxes1()
                            }

                            CheckBox {
                                font.pixelSize: 12
                                indicator.height: 12
                                indicator.width: 12
                                id: checkBox13
                                objectName: "baseline_type2"
                                x: 130
                                y: 626
                                visible: checkBox6.checked
                                text: qsTr("SNIP")
                                checked: false
                                function checkboxes1() {
                                    if (checkBox13.checked == true){
                                        checkBox12.checked = false
                                    }
                                }
                                onClicked: checkboxes1()
                            }
                            
                            Slider {
                                id: slider34
                                x: -73
                                y: 670
                                width: 480
                                height: 20
                                visible: checkBox6.checked
                                scale: 0.7
                                to: 50
                                from: 0
                                stepSize: 1
                                value: textField51.text
                                handle: Rectangle {
                                    color: parent.pressed ? "#aaf661" : "white" 
                                    border.color: "#69bd18"
                                    border.width: 2
                                    implicitWidth: 24
                                    implicitHeight: 24
                                    radius: 12
                                    x: ((parent.value-parent.from)*(parent.width-implicitWidth))/(parent.to-parent.from)
                                }
                            }

                            Text {
                                id: text37
                                x: 6
                                y: 656
                                width: 29
                                height: 13
                                visible: checkBox6.checked
                                text: "Correction Degree"
                                font.pixelSize: 12
                            }

                            TextField {
                                font.pixelSize: 12
                                id: textField51
                                objectName: "textField51"
                                x: 340
                                y: 670
                                width: 60
                                height: 25
                                topPadding: 5
                                visible: checkBox6.checked
                                text: slider34.value
                                onTextChanged: if (length > 6) remove(6, length);
                                placeholderText: qsTr("Text Field")
                                padding: 2
                                leftPadding: 10
                            }
                        }

                        
                        Rectangle {
                            id: container3
                            height: contentHeight
                            width: 450
                            x:0
                            y:130

                            CheckBox {
                                font.pixelSize: 12
                                indicator.height: 12
                                indicator.width: 12
                            id: checkBox1
                            x: 0
                            y: 200
                            text: qsTr("Empty glass")
                            checked: false
                            }

                            Slider {
                                id: slider14
                                x: -72
                                y: 120
                                width: 480
                                height: 20
                                stepSize: 1
                                to: 75
                                scale: 0.7
                                value: textField18.text
                                handle: Rectangle {
                                    color: parent.pressed ? "#aaf661" : "white" 
                                    border.color: "#69bd18"
                                    border.width: 2
                                    implicitWidth: 24
                                    implicitHeight: 24
                                    radius: 12
                                    x: ((parent.value-parent.from)*(parent.width-implicitWidth))/(parent.to-parent.from)
                                }
                            }

                            Text {
                                id: text14
                                x: 0
                                y: 106
                                width: 29
                                height: 13
                                text: qsTr("Start Temperature")
                                font.pixelSize: 12
                            }

                            TextField {
                                font.pixelSize: 12
                                id: textField18
                                objectName: 'textField18'
                                x: 340
                                y: 120
                                width: 60
                                height: 25
                                text: slider14.value
                                topPadding: 5
                                padding: 2
                                leftPadding: 10
                            }

                            Slider {
                                id: slider15
                                x: -73
                                y: 160
                                width: 480
                                height: 20
                                stepSize: 1
                                to: 250
                                scale: 0.7
                                value: textField19.text
                                handle: Rectangle {
                                    color: parent.pressed ? "#aaf661" : "white" 
                                    border.color: "#69bd18"
                                    border.width: 2
                                    implicitWidth: 24
                                    implicitHeight: 24
                                    radius: 12
                                    x: ((parent.value-parent.from)*(parent.width-implicitWidth))/(parent.to-parent.from)
                                }
                            }

                            Text {
                                id: text15
                                x: -1
                                y: 146
                                width: 29
                                height: 13
                                text: qsTr("Reaction Tempertaure")
                                font.pixelSize: 12
                            }

                            TextField {
                                font.pixelSize: 12
                                id: textField19
                                objectName: 'textField19'
                                x: 340
                                y: 160
                                width: 60
                                height: 25
                                text: slider15.value
                                topPadding: 5
                                padding: 2
                                placeholderText: qsTr("150")
                                leftPadding: 10
                            }

                            TextField {
                                font.pixelSize: 12
                                id: textField20
                                objectName: "textField20"
                                x: 0
                                y: 50
                                width: 398
                                height: 30
                                topPadding: 7
                                leftPadding: 10
                                text: "Reference Pack"
                                placeholderText: qsTr("Text Field")
                            }

                            Slider {
                                id: slider16
                                x: -73
                                y: 260
                                width: 480
                                height: 20
                                visible: checkBox1.checked
                                scale: 0.7
                                from: -1
                                to: 1
                                value: textField21.text
                                handle: Rectangle {
                                    color: parent.pressed ? "#aaf661" : "white" 
                                    border.color: "#69bd18"
                                    border.width: 2
                                    implicitWidth: 24
                                    implicitHeight: 24
                                    radius: 12
                                    x: ((parent.value-parent.from)*(parent.width-implicitWidth))/(parent.to-parent.from)
                                }
                            }

                            Text {
                                id: text16
                                x: 6
                                y: 246
                                width: 29
                                height: 13
                                visible: checkBox1.checked
                                text: "Glass Scale"
                                font.pixelSize: 12
                            }

                            TextField {
                                font.pixelSize: 12
                                id: textField21
                                objectName: "textField21"
                                x: 340
                                y: 260
                                width: 60
                                height: 25
                                topPadding: 5
                                visible: checkBox1.checked
                                text: slider16.value
                                onTextChanged: if (length > 9) remove(9, length);
                                placeholderText: qsTr("Text Field")
                                padding: 2
                                leftPadding: 10
                            }

                            TextField {
                                font.pixelSize: 12
                                id: textField22
                                objectName: "textField22"
                                x: 65
                                y: 10
                                width: 60
                                height: 25
                                topPadding: 5
                                text: textField48.text
                                horizontalAlignment: Text.AlignHCenter
                                placeholderText: qsTr("1")
                            }

                            Text {
                                id: text18
                                x: -10
                                y: 10
                                width: 80
                                height: 25
                                text: qsTr("Averaging:")
                                font.pixelSize: 12
                                horizontalAlignment: Text.AlignHCenter
                                verticalAlignment: Text.AlignVCenter
                            }

                            TextField {
                                font.pixelSize: 12
                                id: textField23
                                objectName: "textField23"
                                x: 190
                                y: 10
                                width: 60
                                height: 25
                                topPadding: 5
                                text: textField49.text
                                horizontalAlignment: Text.AlignHCenter
                                placeholderText: qsTr("25")
                            }

                            Text {
                                id: text19
                                x: 125
                                y: 10
                                width: 80
                                height: 25
                                text: qsTr("Qmask:")
                                font.pixelSize: 12
                                horizontalAlignment: Text.AlignHCenter
                                verticalAlignment: Text.AlignVCenter
                            }


                            TextField {
                                font.pixelSize: 12
                                id: textField24
                                objectName: "textField24"
                                x: 338
                                y: 10
                                width: 60
                                height: 25
                                topPadding: 5
                                text: "1.3"
                                horizontalAlignment: Text.AlignHCenter
                                placeholderText: qsTr("1.3")
                            }

                            Text {
                                id: text20
                                x: 263
                                y: 10
                                width: 80
                                height: 25
                                text: qsTr("Factormax:")
                                font.pixelSize: 12
                                horizontalAlignment: Text.AlignHCenter
                                verticalAlignment: Text.AlignVCenter
                            }

                            Button {
                                font.pixelSize: 12
                                id: button4
                                hoverEnabled: false
                                x: 177
                                y: 205
                                width: 220
                                height: 30
                                visible: checkBox1.checked
                                text: qsTr("Load Glass File")
                                font.styleName: "Fett"
                                checked: false
                                flat: false
                                contentItem: Text {
                                    text: button4.text
                                    font: button4.font
                                    opacity: enabled ? 1.0 : 0.3
                                    color: button4.down ? "#ffffff" : "#69bd18"
                                    horizontalAlignment: Text.AlignHCenter
                                    verticalAlignment: Text.AlignVCenter
                                    elide: Text.ElideRight
                                }
                                background: Rectangle {
                                    color: button4.down == true ? "#111111" : "#333333"
                                    radius: 2
                                }

                                onClicked: {
                                    measglassDialog.visible = true
                                }
                            }
                        }     
                    }
                }

                Flickable {
                    id: scrollViewPane3
                    x: 0
                    y: 80
                    width: 450
                    height: Window.height
                    clip: true
                    contentHeight: pane3.height
                    visible: tabButton2.checked

                    Pane {
                        id: pane3
                        x: 0
                        y: 50
                        width: 450
                        height: 550
                        visible: tabButton2.checked
                        background: Rectangle {
                                color: "#ffffff"
                        }

                        Button {
                            font.pixelSize: 12
                            id: button5
                            objectName: 'PDF_load'
                            hoverEnabled: false
                            x: 63
                            y: 200
                            width: 300
                            height: 50
                            text: qsTr("Load PDF")
                            font.styleName: "Fett"
                            checked: false
                            flat: false
                            contentItem: Text {
                                text: button5.text
                                font: button5.font
                                opacity: enabled ? 1.0 : 0.3
                                color: button5.down ? "#ffffff" : "#69bd18"
                                horizontalAlignment: Text.AlignHCenter
                                verticalAlignment: Text.AlignVCenter
                                elide: Text.ElideRight
                            }
                            background: Rectangle {
                                color: button5.down == true ? "#111111" : "#333333"
                                radius: 2
                            }
                        }

                        Slider {
                            id: slider22
                            x: -72
                            y: 70
                            width: 480
                            height: 20
                            stepSize: 1
                            scale: 0.7
                            value: textField31.text
                            to: 60
                            from: 1
                            handle: Rectangle {
                                color: parent.pressed ? "#aaf661" : "white" 
                                border.color: "#69bd18"
                                border.width: 2
                                implicitWidth: 24
                                implicitHeight: 24
                                radius: 12
                                x: ((parent.value-parent.from)*(parent.width-implicitWidth))/(parent.to-parent.from)
                            }
                        }

                        Text {
                            id: text25
                            x: 0
                            y: 56
                            width: 29
                            height: 13
                            text: qsTr("Scan Steps")
                            font.pixelSize: 12
                        }

                        TextField {
                            font.pixelSize: 12
                            id: textField31
                            objectName: "textField31"
                            x: 340
                            y: 70
                            width: 60
                            height: 25
                            text: slider22.value
                            leftPadding: 10
                            topPadding: 5
                            padding: 2
                            placeholderText: qsTr("Text Field")
                        }

                        TextField {
                            font.pixelSize: 12
                            id: textField35
                            objectName: "textField35"
                            x: 87
                            y: 10
                            width: 60
                            height: 30
                            text: "1"
                            topPadding: 5
                            horizontalAlignment: Text.AlignHCenter
                            placeholderText: qsTr("Text Field")
                        }

                        Text {
                            id: text28
                            x: 1
                            y: 10
                            width: 80
                            height: 30
                            text: qsTr("Start San:")
                            font.pixelSize: 12
                            horizontalAlignment: Text.AlignHCenter
                            verticalAlignment: Text.AlignVCenter
                        }

                        TextField {
                            font.pixelSize: 12
                            id: textField36
                            objectName: "textField36"
                            x: 263
                            y: 10
                            width: 60
                            height: 30
                            topPadding: 5
                            text: "100000"
                            horizontalAlignment: Text.AlignHCenter
                            placeholderText: qsTr("25")
                        }

                        Text {
                            id: text29
                            x: 177
                            y: 10
                            width: 80
                            height: 30
                            text: qsTr("End Scan:")
                            font.pixelSize: 12
                            horizontalAlignment: Text.AlignHCenter
                            verticalAlignment: Text.AlignVCenter
                        }

                        ColorButton3 {
                            font.pixelSize: 14
                            id: colorButton5
                            objectName: 'PDF_2D_map'
                            x: 178
                            y: 480
                            width: 220
                            height: 30
                            text: "2D time map"
                        }

                        ColorButton {
                            font.pixelSize: 14
                            id: colorButton6
                            objectName: 'PDF_1D_projected'
                            x: 178
                            y: 400
                            width: 220
                            height: 30
                            text: "1D time projected"
                            highlighted: false
                            checkable: false
                            icon.color: "#ffffff"
                            autoExclusive: false
                            checked: false
                            flat: false
                            autoRepeat: false
                        }

                        ColorButton2 {
                            font.pixelSize: 14
                            id: colorButton7
                            objectName: 'PDF_1D_overlay'
                            x: 178
                            y: 440
                            width: 220
                            height: 30
                            text: "1D time overlay"
                            transformOrigin: Item.TopLeft
                        }

                        Button {
                            font.pixelSize: 12
                            id: button6
                            hoverEnabled: false
                            x: 100
                            y: 405
                            width: 20
                            height: 20
                            opacity: 1
                            text: qsTr("Button")
                            highlighted: false
                            icon.color: "#ffffff"
                            Image {
                                id: image1
                                width: 20
                                height: 20
                                source: "../die-einstellungen.png"
                                fillMode: Image.PreserveAspectFit
                            }
                            icon.source: "../die-einstellungen.png"
                            icon.width: 20
                            flat: true
                            icon.height: 20
                        }

                        Rectangle {
                            id: rectangle5
                            x: -7
                            y: 540
                            width: 440
                            height: 4
                            color: "#353637"
                            radius: 2
                        }


                        Text {
                            id: text2
                            x: 63
                            y: 345
                            width: 300
                            height: 40
                            text: qsTr("Plotting options for the PDF")
                            font.pixelSize: 24
                            horizontalAlignment: Text.AlignHCenter
                            font.bold: true
                            font.underline: false
                        }

                        Text {
                            id: text3
                            x: 1
                            y: 400
                            width: 130
                            height: 30
                            font.pixelSize: 12
                            text: qsTr("Figure options:")
                            horizontalAlignment: Text.AlignLeft
                            verticalAlignment: Text.AlignVCenter
                            leftPadding: 10
                            topPadding: 0
                        }

                        CheckBox {
                            font.pixelSize: 12
                            indicator.height: 12
                            indicator.width: 12
                            id: checkBox14
                            objectName: "pdf_set1"
                            x: 45
                            y: 120
                            text: qsTr("Uncorrected")
                            checked: true
                            function checkboxes1() {
                                if (checkBox14.checked == true){
                                    checkBox15.checked = false
                                    checkBox16.checked = false
                                }
                            }
                            onClicked: checkboxes1()
                        }

                        CheckBox {
                            font.pixelSize: 12
                            indicator.height: 12
                            indicator.width: 12
                            id: checkBox15
                            objectName: "pdf_set2"
                            x: 165
                            y: 120
                            text: qsTr("Pre-corrected")
                            checked: false
                            function checkboxes1() {
                                if (checkBox15.checked == true){
                                    checkBox14.checked = false
                                    checkBox16.checked = false
                                }
                            }
                            onClicked: checkboxes1()
                        }

                        CheckBox {
                            font.pixelSize: 12
                            indicator.height: 12
                            indicator.width: 12
                            id: checkBox16
                            objectName: "pdf_set3"
                            x: 285
                            y: 120
                            text: qsTr("Post-corrected")
                            checked: false
                            function checkboxes1() {
                                if (checkBox16.checked == true){
                                    checkBox15.checked = false
                                    checkBox14.checked = false
                                }
                            }
                            onClicked: checkboxes1()
                        }
                    }
                }
            }
        }
        
        Flow {
            id: row1
            width: Window.width - 450
            height: Window.height
            
            Flickable {
                id: scrollView
                anchors.fill: parent
                x: 0
                y: 0
                width: parent.width
                height: parent.height-200
                clip: true
                contentHeight: 1.5*column2.height+column1.height

                Flow {
                    id: column1
                    x: 0
                    y: 0
                    width: parent.width
                    height: parent.height
                    

                    Rectangle {
                        id: rectangle3
                        x: 0
                        y: 0
                        width: parent.width
                        height: parent.height
                        color: "#d0f284"

                        Rectangle {
                            id: general_settings
                            x: Window.width/2 - 225 - width/2 
                            y: Window.height/2 - 100 - height/2
                            width: 500
                            height: 250
                            color: "#ffffff"  
                            visible: false

                            Rectangle {
                                width: 500
                                height: 30
                                color: "#000000" 

                                Button {
                                    font.pixelSize: 12
                                    id: button13
                                    objectName: "close_generals"
                                    hoverEnabled: false
                                    x: 477
                                    y: 7
                                    width: 16
                                    height: 16
                                    checked: false
                                    flat: false
                                    enabled: true
                                    contentItem: Text {
                                        text: parent.text
                                        font: parent.font
                                        opacity: enabled ? 1.0 : 0.3
                                        color: parent.down ? "#8f170e" : "#ff1100" 
                                        horizontalAlignment: Text.AlignHCenter
                                        verticalAlignment: Text.AlignVCenter
                                        elide: Text.ElideRight
                                    }
                                    background: Rectangle {
                                        color: parent.down == true ? "#ff1100" : "#b81a0f" 
                                        radius: 10
                                    }

                                    onClicked: {
                                        general_settings.visible = false
                                    }
                                }
                            }


                            Text {
                                id: gentext1
                                x: 20
                                y: 55
                                width: 150
                                height: 12
                                text: qsTr("Temperature signature:")
                                font.pixelSize: 12
                            }

                            TextField {
                                font.pixelSize: 12
                                id: gentextField1
                                objectName: "temp"
                                x: 160
                                y: 50
                                width: 310
                                height: 30
                                topPadding: 7
                                leftPadding: 10
                                text: 'userComment2="Temperature='
                                placeholderText: qsTr("Text Field")
                            }
                            
                            Text {
                                id: gentext2
                                x: 20
                                y: 105
                                width: 150
                                height: 12
                                text: qsTr("Time signature:")
                                font.pixelSize: 12
                            }

                            TextField {
                                font.pixelSize: 12
                                id: gentextField2
                                objectName: "time"
                                x: 160
                                y: 100
                                width: 310
                                height: 30
                                topPadding: 7
                                leftPadding: 10
                                text: 'dateString='
                                placeholderText: qsTr("Text Field")
                            }

                            Text {
                                id: gentext3
                                x: 20
                                y: 155
                                width: 150
                                height: 12
                                text: qsTr("Integraton ends with:")
                                font.pixelSize: 12
                            }

                            TextField {
                                font.pixelSize: 12
                                id: gentextField3
                                objectName: "endswith"
                                x: 160
                                y: 150
                                width: 310
                                height: 30
                                topPadding: 7
                                leftPadding: 10
                                text: '.dat'
                                placeholderText: qsTr("Text Field")
                            }

                            Text {
                                id: gentext4
                                x: 20
                                y: 205
                                width: 150
                                height: 12
                                text: qsTr("Metadata dir path:")
                                font.pixelSize: 12
                            }

                            TextField {
                                font.pixelSize: 12
                                id: gentextField4
                                objectName: "metadata_dir"
                                x: 160
                                y: 205
                                width: 310
                                height: 30
                                topPadding: 7
                                leftPadding: 10
                                text: '/asap3/petra3/gpfs/p21.1/2023/data/11017687/raw/pe/insitu/insitu_pdf/'
                                placeholderText: qsTr("Text Field")
                            }
                        }

                        Rectangle {
                            id: rectangle2
                            x: 0
                            y: 0
                            width: parent.width
                            height: parent.height
                            opacity: 1
                            color: "#00ffffff"
                            radius: 30

                            BusyIndicator {
                                id: busyIndicator1
                                objectName: "BGwaitscreen"
                                x: Window.width/2 - 225 - width/2 
                                y: Window.height/2 - 100 - height/2
                                width: 136
                                height: 118
                                visible: false
                                TextEdit {
                                    id: textEdit1
                                    x: -width/2 + parent.width/2
                                    y: 118
                                    width: 586
                                    height: 150
                                    visible: parent.visible
                                    text: qsTr("Please wait \nwhile the Background scans are loaded\n\n")
                                    font.pixelSize: 32
                                    horizontalAlignment: Text.AlignHCenter
                                }
                            }
                            
                            BusyIndicator {
                                id: busyIndicator2
                                objectName: "MEASwaitscreen"
                                x: Window.width/2 - 225 - width/2 
                                y: Window.height/2 - 100 - height/2
                                width: 136
                                height: 118
                                visible: false
                                TextEdit {
                                    id: textEdit2
                                    x: -width/2 + parent.width/2
                                    y: 118
                                    width: 586
                                    height: 150
                                    visible: parent.visible
                                    text: qsTr("Please wait \nwhile the Measurment scans are loaded\n\n")
                                    font.pixelSize: 32
                                    horizontalAlignment: Text.AlignHCenter
                                }
                            }
                            
                            BusyIndicator {
                                id: busyIndicator3
                                objectName: "PDFwaitscreen"
                                x: Window.width/2 - 225 - width/2 
                                y: Window.height/2 - 100 - height/2
                                width: 136
                                height: 118
                                visible: false
                                TextEdit {
                                    id: textEdit3
                                    x: -width/2 + parent.width/2
                                    y: 118
                                    width: 586
                                    height: 150
                                    visible: parent.visible
                                    text: qsTr("Please wait \nwhile the PDFs are loaded\n\n")
                                    font.pixelSize: 32
                                    horizontalAlignment: Text.AlignHCenter
                                }
                            }
                        }
                    }
                }
            }
            Flow {
                id: column2
                x: 0
                y: Window.height -200
                width: Window.width -450
                height: 200

                Rectangle {
                    id: rectangle8
                    x: 0
                    y: 880
                    width: Window.width -450
                    height: 200
                    color: "#ffffff"
                    border.width: 4

                    Rectangle {
                        id: rectangle9
                        x: 4
                        y: 4
                        width: Window.width -458
                        height: 25
                        color: "#0d2438"

                        Text {
                            id: text5
                            x: 35
                            y: -9
                            width: 400
                            height: 40
                            color: "#ffffff"
                            text: qsTr("Consol Log")
                            font.pixelSize: 12
                            horizontalAlignment: Text.AlignLeft
                            verticalAlignment: Text.AlignVCenter
                        }
                    }

                    Rectangle {
                        id: rectangle10
                        x: 4
                        y: 29
                        width: Window.width -458
                        height: 175
                        color: "#143654"
                        
                        Text {
                            id: text40
                            objectName: "consoleLog"
                            x: 10
                            y: 5
                            width: Window.width -463
                            height: 165
                            color: "#ffffff"
                            text: qsTr("--- FlyPDF started, version 0.5.0 ---")
                            font.pixelSize: 12
                            horizontalAlignment: Text.AlignLeft
                        }
                    }
                }
            }
        }
    }
}

/*##^##
Designer {
    D{i:0}D{i:5}D{i:6}D{i:65}D{i:67}D{i:26;invisible:true}D{i:87}D{i:68}
}
##^##*/

