"""

:platform: Unix, Windows 
:last changed: 2023-01-19

.. moduleauthor:: Tjark Leon Raphael Gröne <tgroene@physnet.uni-hamburg.de>


This Code is used to import integrated .chi data and processes it to to the PDF .gr 
"""
# import sys, getopt
# import threading
#try:
from python_scripts.backbone.PDF_generation_class import PDF_generation_class
#except:
#    from backbone.PDF_generation_class import PDF_generation_class
    
import pathlib
rootdir = str(pathlib.Path(__file__).parent.resolve())
# from backbone import PDF_generation_class


class LOADBG():
    def __init__(self,projectPath: str,path: str, reload: str = None, average: int=1,tempPath: str=None,sampleName: str=None, bg_glass: str=None, empty_glass_int: float=0.0, Qmaxinst_XRD: float=30, counter_PDF: dict=None, mode: str=None,
                 temp_signatur: str='userComment1="Temperature=',time_signatur: str="dateString=", endswith: str=".dat",metadata_dir: str='/asap3/petra3/gpfs/p21.1/2023/data/11017687/raw/pe/insitu/insitu_pdf/'): #slice, bg_glass
        # self.stdout = None
        # self.stderr = None
        # threading.Thread.__init__(self)
        self.projectPath = projectPath
        if sampleName == None:
            self.sampleName = path.split("/")[-1] 
        if tempPath == None:
            self.tempPath = metadata_dir + sampleName + "/"  #metadat path
        else:
            self.tempPath = tempPath + "/"
        if path != None:
            self.path = path + "/"
        else: 
            self.path = path
        self.sampleName = sampleName
        self.reload = reload
        self.average = average
        self.Qmaxinst_XRD = Qmaxinst_XRD
        self.bg_glass = bg_glass
        self.mode = mode
        self.empty_glass_int = empty_glass_int 
        self.endswith = endswith
        
        if counter_PDF == None:
            self.counter_PDF = {
                "time": time_signatur,
                "temperature": temp_signatur,
            }
        
        self.importBG = False
        if reload == None:
            reload = pathlib.Path(self.projectPath + '/processed/Reload_files/bg_Reload/' + self.sampleName + '.mat')
            if reload.is_file():
                print('Starting from Reload')
                self.reload = reload
            else: 
                self.reload = None
        else:
            reload = pathlib.Path(self.reload)
            if reload.is_file():
                print('Starting from Reload')
                self.reload = reload
            else: 
                self.reload = None
        
        self.load_background()



    def load_background(self):
        #Loading the background data
        startScan = 0
        endScan = 1e10
        try:
            try:
                if self.reload != None:
                    print('Reload accepted')
                
                self.bg = PDF_generation_class(projectPath=self.projectPath,path=self.path, tempPath=self.tempPath, sampleName=self.sampleName, startScan=startScan, endScan=endScan, counter_PDF=self.counter_PDF, reload=self.reload, average=self.average, ends_with=self.endswith)
                self.bg.start()
                self.bg.join()
            except:
                print('No valid reload present')
                print(self.reload)
                self.bg = PDF_generation_class(projectPath=self.projectPath,path=self.path, tempPath=self.tempPath, sampleName=self.sampleName, startScan=startScan, endScan=endScan, counter_PDF=self.counter_PDF, average=self.average, ends_with=self.endswith)
                self.bg.start()
                self.bg.join()
            
            self.bg.join()
            if (self.mode==None and self.reload == None) or self.importBG == True:    
                print(self.mode,self.importBG)
                self.bg.saveReload(self.projectPath + '/processed/Reload_files/bg_Reload/')
            self.bg.PrintLogFile(self.projectPath + '/processed/Log_files/bg_Log/')
            
            if (self.mode==None and self.reload == None) or self.importBG == True:
                self.bg.saveReload(self.projectPath + '/processed/Reload_files/bg_Reload/')
                self.bg.PrintLogFile(self.projectPath + '/processed/Log_files/bg_Log/')
                print(self.mode,self.importBG)

            self.bg.norm(Qmaxinst=self.Qmaxinst_XRD)
            
            if self.bg_glass != None:
                self.bg.bg_glassSubtraction(self.sampleName, empty_glass=self.bg_glass, empty_glass_int=self.empty_glass_int)

            self.bg.PrintLogFile(self.projectPath + '/processed/Log_files/bg_Log/')
        except:
            self.bg = None

        return self.bg

    def plot_1D_XRD_Slice(self,slice):
        references = {  'Dummy': {'path': rootdir + "/References/Dummy.int",
                                    'skiprows': 2,
                                    'energy': 100} ,}
        check_phases_at_scan = slice #choose last scan processed by the live integration devided by the number of averged scans
        # self.bg.baselineCorrection(40)
        # self.bg.Filter('whittaker',20)
        self.bg.checkPhases(check_phases_at_scan,references=references,savefig='png',outputPath=self.projectPath + '/processed/XRD_Plots/bg_phase_check/', maxQ=9)
        self.bg.PrintLogFile(self.projectPath + '/processed/Log_files/bg_Log/')
        
    def plot_1D_XRD_vs_time(self):
        self.bg.plot1dvsTime(step=2,maxQ=15, plot_temp=True,savefig='svg',outputPath=self.projectPath + '/processed/XRD_Plots/bg_1D_plot/')
        self.bg.PrintLogFile(self.projectPath + '/processed/Log_files/bg_Log/')

    def plot_2D_XRD_vs_time(self):
        self.bg.plot2dTemp(savefig='svg',outputPath=self.projectPath + '/processed/XRD_Plots/bg_2D_plot/')
        self.bg.PrintLogFile(self.projectPath + '/processed/Log_files/bg_Log/')
        
                        
       
                
            
            






