"""

:platform: Unix, Windows 
:last changed: 2022-05-14

.. moduleauthor:: Tjark Leon Raphael Gröne <tgroene@physnet.uni-hamburg.de>

This code is used to load and plot pdf data
-- There for exitu data we will load the class called exituPDF
-- To display insitu data the insituPDF class is suitable
-- Last there is the insituPDFfitted class you can use to display 
    the different properties that the diffpy-cmi will print in the result files

"""

import backbone.PDF_class_insitu as PDF_class
try:
    from python_scripts.backbone.settings.reference_sets import *
except:
    from backbone.settings.reference_sets import *
import pathlib
rootdir = str(pathlib.Path(__file__).parent.resolve())


class LOADPDF():
    def __init__(self,projectPath: str, startScan: int=1, endScan: int=10000, scanStep: int=1, reload: str = None, average: int=1, sampleName: str=None,  mode: str=None):
        self.projectPath = projectPath
        self.path = self.projectPath + "/processed/PDF_files/" + mode + "/"
        self.path_to_Temp = self.projectPath + "/processed/Reload_files/meas_Reload/" + sampleName + ".mat"
        self.sampleName = sampleName
        self.reload = reload
        self.average = average
        self.startScan = startScan
        self.endScan = endScan
        self.scanStep = scanStep
        self.mode = mode
        
        if mode != "created_single_pdf":
            self.load_batch_pdf()
        else:
            self.load_pdf()
        
    def load_batch_pdf(self):
        self.meas = PDF_class.InsituPDF(path=self.path, sampleName=self.sampleName, startScan=self.startScan, endScan=self.endScan,
                                    timeOfset=0, tempPath=self.path_to_Temp, average=self.average,
                                    figratio1D=[4,3],figratio2D=[7,4])
        
    def load_pdf(self):
        self.meas = PDF_class.singlePDF(path=self.path, sampleName=self.sampleName, timeOfset=0, average=self.average,
                                    figratio1D=[4,3],figratio2D=[7,4])
                                    
    def plot_single_PDF(self, references, Qmax, Qmin, ref_type: str="debye", qdamp: float= None, qbroad: float = None):
        
        self.meas.checkPDF(references=references,savefig="svg",outputPath=self.projectPath + "/processed/PDF_Plots/PDF_1D_projected_plot/", maxQ = Qmax, minQ = Qmin, ref_type=ref_type, qdamp=qdamp, qbroad= qbroad)
                               
    def plot_1D_projected(self):        
        # Waterfall like 1D plot
        self.meas.plot1dvsTimeProjectedOntop(offsetTime=0,savefig="svg",outputPath=self.projectPath + "/processed/PDF_Plots/PDF_1D_projected_plot/",jump=self.scanStep, minR=1, maxR=10, intercept=0.2) #plot in 1D, insitu=True will define color change and hides ledgend
        
    def plot_1D_vs_time(self):
        # 1D plot over time with temperature label
        self.meas.plot1dvsTime(offsetTime=0,savefig="svg",outputPath=rootdir + "/processed/" + self.sampleName + "/PDF_Plots/PDF_1D_plot/",jump=self.scanStep, minR=1, maxR=10) #plot in 1D, insitu=True will define color change and hides ledgend

    def plot_2D_vs_time(self):
        # 2D plot over time with temperature curve
        self.meas.plot2dTemp(vmin=-0.5,vmax=1.5,savefig="svg",outputPath= rootdir + '/processed/' + self.sampleName + '/PDF_Plots/PDF_2D_plot/', maxR=10) #rect=rectangle)
        

