
import pathlib
rootdir = str(pathlib.Path(__file__).parent.resolve())

from time import time
from timeit import default_timer as timer
import threading
import subprocess
from multiprocessing import Process, Pipe 
from scipy import io as sp_io

try:
    from backbone.functions.plotting import Plotting_XRD
    from backbone.functions.data_processing import Processing_Data
    from backbone.functions.export import Export_PDF_Gen
    from backbone.functions.import_data import Import_PDF_Gen
    from backbone.functions.utils import averageArray
except:
    from python_scripts.backbone.functions.plotting import Plotting_XRD
    from python_scripts.backbone.functions.data_processing import Processing_Data
    from python_scripts.backbone.functions.export import Export_PDF_Gen
    from python_scripts.backbone.functions.import_data import Import_PDF_Gen
    from python_scripts.backbone.functions.utils import averageArray

class PDF_generation_class(threading.Thread,Plotting_XRD,Processing_Data,Export_PDF_Gen,Import_PDF_Gen):
    def __init__(self, projectPath: str, path: str,tempPath: str,sampleName: str,startScan: int,endScan: int, counter_PDF: dict, timeOfset=0, ends_with = '.chi', without_dark=True, 
                figratio1D=[16,9], figratio2D=[7, 4], reload: str = None, number_of_steps: int=1, skiprows_of_files: int = 23, average: int=1):
        """
        Arguments
        ----------
        path:               Path to file
        tempPath:            Path to the .metadata
        ends_with:          file extension of the integrated data (.chi/.dat)
        skiprows_of_file:   header lines in integrated data
        number_of_steps:    load faster by introducing steps, that reduce data quality
        sampleName:         Name of your sample (for Ledgend)
        startScan:          First scan to include 
        endScan:            Last scan to include
        timeOfset:          Ofset of the timeaxes for insitu measurements in s (-10 time axis starts with -10 s)       
        figratio1D:         Ratio of the plot window for 1D plots
        figratio2D:         Ratio of the plot window for 2D plots
        reload:             Path to a saved MAT file, that was saved with the saveReload() function before
        """
        self.stdout = None
        self.stderr = None
        threading.Thread.__init__(self)
        self.t0 = time() #start time 

        self.path = path
        self.sampleName = sampleName
        self.startScan = int(startScan)
        self.endScan = int(endScan)
        self.counter_PDF = counter_PDF
        self.timeOfset= timeOfset
        self.without_dark = without_dark
        self.reload = reload
        self.number_of_steps = number_of_steps
        self.ends_with = ends_with
        self.skiprows_of_files = skiprows_of_files
        self.average = average
        self.glassGr = False
        self.tempPath = tempPath
        self.projectPath = projectPath
        
        self.max_all_int = 0
        self.min_all_int = 0

        ratio1D = figratio1D[0]/figratio1D[1] 
        figratio1D = [ratio1D*9,9]
        self.figratio1D = figratio1D
        ratio2D = figratio2D[0]/figratio2D[1] 
        figratio2D = [ratio2D*4,4]
        self.figratio2D = figratio2D

        self.logger = {"Initial parameters":{   "Path to .chi/.dat": path,
                                                "Path to .metadata": tempPath,
                                                "File Extension of integrated data": ends_with,
                                                "Header lines skipped": skiprows_of_files,
                                                "Data was loop over in number of steps": number_of_steps,
                                                "Sample name": sampleName,
                                                "Start scan": startScan,
                                                "End scan": endScan,
                                                "time Ofset": timeOfset,
                                                "Data was reloaded": self.reload,},}
    def run(self):
        if self.reload != None:
            loadedmat = sp_io.loadmat(self.reload)
            self.I = loadedmat['I_matrix']
            self.q = loadedmat['q_axis'][0]
            self.time_array = loadedmat['time'][0] + self.timeOfset
            self.time_averaged_array = loadedmat['time_averaged'][0][self.startScan:self.endScan] + self.timeOfset
            self.temp_array = loadedmat['temp'][0]
            self.temp_averaged_array = loadedmat['temp_averaged'][0][self.startScan:self.endScan]
            self.number_of_files = len(self.I)
            self.time_average_array = averageArray(self.time_array,self.average)
            self.temp_average_array = averageArray(self.temp_array,self.average)
            self.useAverage()
        else:
            # p1_conn1, p1_conn2 = Pipe()
            # p2_conn1, p2_conn2 = Pipe()
            # p1 = Process(target=self.import_data,args=(p1_conn2,))
            # p2 = Process(target=self.getTempTime,args=(tempPath,p2_conn2))
            # p1.start()
            # p2.start()
            # self.q, self.I = p1_conn1.recv()
            # data = p1_conn1.recv()
            # self.time_array, self.temp_array, self.time_average_array, self.temp_average_array, self.logger = data
            # print(self.temp_array)
            # start = timer()
            # p1.join()
            # p2.join()
            # print("without GPU:", timer()-start) 
            self.getTempTime(self.tempPath)
            self.import_data()
            self.useAverage()

        print("\n Time for processing")
        print("------------------------------------------------------------------")
        print("Time: " + str((time() - self.t0)/60) + "\n")
        self.logger['Time it took to load'] = {'Time in min': str((time() - self.t0)/60)}

