import numpy as np    
import pathlib, os
import subprocess
from scipy.signal import savgol_filter
try:
    from backbone.functions.utils import find_nearest, averageArray, averageArrayError, round_to_multiple, ensure_dir, getIfile, del_folder_content
    from backbone.functions.export import outputDatFile
    from backbone.functions.whittaker_smooth import whittaker_smooth
except:
    from python_scripts.backbone.functions.utils import find_nearest, averageArray, averageArrayError, round_to_multiple, ensure_dir, getIfile, del_folder_content
    from python_scripts.backbone.functions.export import outputDatFile
    from python_scripts.backbone.functions.whittaker_smooth import whittaker_smooth
from pybaselines import Baseline
import pathlib
rootdir = str(pathlib.Path(__file__).parent.resolve())

class Processing_Data:
    
    #----------------------------------------------------------------------------------------#
    #                                 Get Start and End of ramp                              #
    #----------------------------------------------------------------------------------------#
    def getTempPoints(self,rampTemp,holdTemp):
        """
        Arguments
        ----------
        rampTemp:   temperature the ramp is defined to start 
        holdTemp:   temperature the plato is defined to start
        """
        # Log current event
        self.logger['Scan idizes of the ramp satrt and end collected'] = {  'Temperature the ramp is set to start': rampTemp,
                                                                            'Temperatrue the plato is set to start': holdTemp}
        
        # Create initial values for the linear scanning throught the data set
        ramp_started = False
        plato_started = False
        plato_ended = False

        # Exclude skipped scans
        if self.startScan > 0:
            
            # Evaluate the number of the scan after category (therefore multipliing by number of scans)
            if self.endScan < self.number_of_files*self.number_of_steps:
                file_index = np.linspace(self.startScan,self.endScan,self.number_of_files)
            else:
                file_index = np.linspace(self.startScan,self.number_of_files*self.number_of_steps,self.number_of_files)
        else: 
            
            # Defining for start scans smaller 1 
            if self.endScan < self.number_of_files*self.number_of_steps:
                file_index = np.linspace(1,self.endScan,self.number_of_files)
            else:
                file_index = np.linspace(1,self.number_of_files*self.number_of_steps,self.number_of_files)

        # Give feedback to live integration speed
        if len(file_index) < len(self.temp_array):   
            print('\n\n ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~')
            print('          Live integration lags behind by ' + str(len(self.temp_array) - len(file_index)) + ' scans')
            print(' ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n')
            
        # Looping over the temperature array to define ramp and plato zones
        for i,j in enumerate(self.temp_array):
            
            # Searching for the start of the temp. ramp
            if j > rampTemp and ramp_started==False:
                
                # Give user feedback
                print('Ramp start: ', file_index[i],'Temp: ', j)
                
                # save in self varible 
                self.rampstart_index = int(file_index[i])
                ramp_started = True 
                
                # Log step
                self.logger['Ramp Start'] = {   'Index': file_index[i],
                                                'Temperature': j}
            
            # Searching for the start of the temp. plato at reaction temp.
            if j > holdTemp and plato_started==False:
                
                # Give user feedback
                print('Plato start: ',file_index[i],'Temp: ', j)
                
                # save in self varible 
                self.platostart_index = int(file_index[i])
                plato_started = True 
                
                # Log step
                self.logger['Plato Start'] = {  'Index': file_index[i],
                                                'Temperature': j}
            
            # Searching for the end of the temp. plato at reaction temp.
            if j < holdTemp-5 and plato_ended==False and plato_started==True:
                
                # Give user feedback
                print('Plato endet: ', file_index[i],'Temp: ', j)
                
                # save in self varible 
                plato_ended = True
                self.platoend_index = int(file_index[i]) 
                
                # Log step
                self.logger['Plato End'] = {   'Index': file_index[i],
                                                'Temperature': j}
        
        # Raise exeption if user chooses to small plato temp.
        if 'platostart_index' not in dir(self):
            raise Exception("Temperature always below the set tempertaure defined") 
        
        # When dataset does not include cooldown, end is set to last scan
        if 'platoend_index' not in dir(self):
            self.platoend_index = len(self.temp_array)
            
        # Return self varibles to main class
        return self.logger, self.rampstart_index, self.platoend_index, self.platoend_index

    
    
    
    #----------------------------------------------------------------------------------------#
    #                              Shorten in Q and Normalize Data                           #
    #----------------------------------------------------------------------------------------#
    def norm(self, normalize: str=None,  Qmaxinst: float=30):
        """
        Arguments
        ----------
        normalize:  normalisation mode None, max or minmax
        Qmaxinst:   cut q to better fit the background subtration late
        """
        # Log current event
        self.logger['Data was normalized and or cutted in q'] = {   'Normalization mode': normalize,
                                                                    'Q cutted at Qmaxinst': Qmaxinst,}
        
        # Set initial varibles
        q = None
        new_I = []
        
        # Loop over all intensities
        for i,j in enumerate(self.I):
            
            # set 
            if q is None:
                q = self.q
                intensity = j
                clostest_Qmaxinst = find_nearest(q,Qmaxinst)
                clostest_Qmaxinst = int(np.where(q==clostest_Qmaxinst)[0])
                self.q = q[:clostest_Qmaxinst]
            
            # resize intensity array to Qmaxinst
            I_norm = j[:clostest_Qmaxinst]
            
            # normalize just to the maximum of I
            if normalize == 'max':
                I_norm = I_norm/max(I_norm)
                
            # normalize to the difference betwwen the minimum of I and the maximum of I
            if normalize == 'minmax':       
                I_norm = (I_norm - min(I_norm)) / (max(I_norm)-min(I_norm))

            # Build new matrix of I arrays
            new_I.append(I_norm)
            
        # Update self varible 
        self.I = new_I
        
        # Return self varibles to main class
        return self.logger, self.I, self.q


    #----------------------------------------------------------------------------------------#
    #                                          Average                                       #
    #----------------------------------------------------------------------------------------#
    def useAverage(self):
        """
        Averages the I(q) data over a set of scans
        
        Arguments
        ----------
        """
        #Storing Event in logger
        self.logger['Data was averaged'] = {   'Averaged over': self.average,}
        
        #Create copie of I(q), thus it can be accesed later on
        self.I_original = self.I
        # self.dI_original = self.dI_matrix

        
        #Update I(q) I to the averaged data
        self.I = averageArray(self.I,self.average,2)
        # self.dI = averageArrayError(self.dI,self.average,2)
                
        # Return self varibles to main class
        return self.I, self.I_original  #, self.dI ,self.dI_original 

    
    #----------------------------------------------------------------------------------------#
    #                               Get parts of ramps and platos                            #
    #----------------------------------------------------------------------------------------#
    def getRampPlatoLists(self,heat_ranges):
        """
        Arguments
        ----------
        heat ranges:    heat ranges dictionary including the start and end of the temp ramp
        """
        # Log current event
        self.logger['I(q) for Ramp and Plato where collected in lists'] = {   'Heatrages input': heat_ranges,}
        
        
        plato_file_list = []
        self.plato_list = [[],[]]
        ramp_file_list = []
        self.ramp_list = [[],[]]

        step = 1
        print(heat_ranges)
        for i, j in enumerate(self.I):
            
            if step % 2 != 0:
                plato_file_list.append(j)
                    
            if step % 2 == 0:
                ramp_file_list.append(j)

            if  i == round_to_multiple(heat_ranges["ramp1_start"],self.average) /self.average:
                self.plato_list[0].append(plato_file_list)
                plato_file_list = []
                step = 2

            if i == round_to_multiple(heat_ranges["ramp1_end"],self.average) /self.average:
                self.ramp_list[0].append(ramp_file_list)
                ramp_file_list = []
                step = 3

            if i == round_to_multiple(heat_ranges["ramp2_start"],self.average) /self.average:
                self.plato_list[0].append(plato_file_list)
                plato_file_list = []
                step = 4

            if i == round_to_multiple(heat_ranges["ramp2_end"],self.average) /self.average:
                self.ramp_list[0].append(ramp_file_list)
                ramp_file_list = []
                step = 5

        self.plato_list[0].append(plato_file_list)
        plato_file_list = []
        
        # Return self varibles to main class
        return self.logger, self.plato_list, self.ramp_list


    #----------------------------------------------------------------------------------------#
    #                               Get parts of ramps and platos                            #
    #----------------------------------------------------------------------------------------#
    def getGlassRampPlatoLists(self,heat_ranges):
        """
        Arguments
        ----------
        heat ranges:    heat ranges dictionary including the start and end of the temp ramp
        """
        # Log current event
        self.logger['I(q) for Ramp and Plato where collected in lists'] = {   'Heatrages input': heat_ranges,}
        
        
        plato_file_list = []
        ramp_file_list = []

        step = 1
        print(heat_ranges)
        for i, j in enumerate(self.Gr):
            
            if step % 2 != 0:
                plato_file_list.append(j)
                    
            if step % 2 == 0:
                ramp_file_list.append(j)


            if  i == round_to_multiple(heat_ranges["ramp1_start"],self.average) /self.average:
                self.plato_list[1].append(plato_file_list)
                plato_file_list = []
                step = 2

            if i == round_to_multiple(heat_ranges["ramp1_end"],self.average) /self.average:
                self.ramp_list[1].append(ramp_file_list)
                ramp_file_list = []
                step = 3

            if i == round_to_multiple(heat_ranges["ramp2_start"],self.average) /self.average:
                self.plato_list[1].append(plato_file_list)
                plato_file_list = []
                step = 4

            if i == round_to_multiple(heat_ranges["ramp2_end"],self.average) /self.average:
                self.ramp_list[1].append(ramp_file_list)
                ramp_file_list = []
                step = 5

        self.plato_list[1].append(plato_file_list)
        plato_file_list = []
        
        # Return self varibles to main class
        return self.logger, self.plato_list, self.ramp_list
           
            
    
    #----------------------------------------------------------------------------------------#
    #                                  Background Subtraction                                #
    #----------------------------------------------------------------------------------------#
        #----------------------------------------------------------------------------------------#
    #                                  Background Subtraction                                #
    #----------------------------------------------------------------------------------------#
    def bgSubtraction(self,plato_bg_list, ramp_bg_list, heat_ranges, bg_sampleName, mode: str='last', FactorMax: float=0.99,single_BG_file: str=None, deviation: float=0, empty_glass_int: float=0, empty_glass: str=None, Qmin: float=None, Qmax: float=None, my_grain_of_dirt: float=0):
        """
        Arguments
        ----------
        plato_bg_list:  list of array from the plato related background scans     
        ramp_bg_list:   list of array from the ramp related background scans
        heat_ranges:    heat ranges dictionary including the start and end of the temp ramp
        bg_sampleName:  Name of the BG sample for .dat output
        mode:           choose "last" to repeat the last background scan or "average" to use an everage of the plato 
        FactorMax:      maximum factor allowed for the background intesity subtracted from the data
        single_bg_file: path to a single background if only one background file should be subtracted from all data
        """
        self.logger['Background was subtracted'] = {'heat ranges used': heat_ranges,
                                                    'Name of the background': bg_sampleName,
                                                    'Mode used for shorter background array': mode,
                                                    'Maximum factor allowed': FactorMax,
                                                    'Path to single background file (if used)': single_BG_file}
        print('\n\n Background factors and ')
        
        if Qmax != None:
            Qmax = find_nearest(self.q,Qmax)
            Qmax = int(np.where(self.q==Qmax)[0][0])
        else:
            Qmax =-1
        if Qmin != None:
            Qmin = find_nearest(self.q,Qmin)
            Qmin = int(np.where(self.q==Qmin)[0][0])
        else:
            Qmin = 0
            
        if empty_glass != None and empty_glass != 0.0: 
            print("Empty Glass used")
            with open(empty_glass) as f:
                lines = [line for line in f if line.startswith('#')]
            skiprows_of_files = len(lines)
            try:
                self.q_empty_glass, self.I_empty_glass = np.loadtxt(empty_glass,skiprows=skiprows_of_files).T
            except:
                self.q_empty_glass, self.I_empty_glass, dI = np.loadtxt(empty_glass,skiprows=skiprows_of_files).T
        else:
            self.q_empty_glass = 0
            self.I_empty_glass = 0
        self.plato_bg_list = plato_bg_list
        self.ramp_bg_list = ramp_bg_list
        self.factor_list = []
        self.raw_factor_list = []
        self.subtracted_I = []
        if single_BG_file == None:
            if mode == 'last':
                for i,j in enumerate(self.plato_list[0]):
                    if len(j) > len(self.plato_bg_list[0][i]):
                        for k in range(int(len(j)-len(self.plato_bg_list[0][i]))):
                            self.plato_bg_list[0][i].append(self.plato_bg_list[0][i][-1])
            if mode == 'average':
                average = 0
                average_list = []
                for i,j in enumerate(self.plato_bg_list[0]): 
                    for a,b in enumerate(j): 
                        average = average + getIfile(b)
                    average = average/len(j)
                    average_list.append(average)
                    average = 0

        print('Lenght of 1st Plato BG:  ' + str(len(self.plato_bg_list[0][0])),'    Lenght of 2nd Plato BG:  ' + str(len(self.plato_bg_list[0][1])), '    Lenght of 1st Ramping BG:  ' + str(len(self.ramp_bg_list[0][0])))
        print('Lenght of 1st Plato Measurement:  ' + str(len(self.plato_list[0][0])),'    Lenght of 2nd Plato Measurement:  ' + str(len(self.plato_list[0][1])), '    Lenght of 1st Ramping Measurement:  ' + str(len(self.ramp_list[0][0])))
        
        self.bg_matrix = self.plato_bg_list[0][0] + self.ramp_bg_list[0][0] + self.plato_bg_list[0][1]
        if self.glassGr == True:
            self.bg_matrix_Gr = self.plato_bg_list[1][0] + self.ramp_bg_list[1][0] + self.plato_bg_list[1][1]
        #glass_q, glass_I = np.loadtxt('C:/Users/admin/Nextcloud/Beamtimes/bt_esrf_0423/empty_accell_insitu_avg01343.txt',skiprows=1).T
        outputDatFile(self.projectPath + '/processed/Dat_files/bg_Dat/',self.q,self.bg_matrix,self.average,bg_sampleName)

        if single_BG_file != None:
            self.I_bg = getIfile(single_BG_file)
            for i,j in enumerate(self.I):  
                I_bg_frac = j[:int(len(j)*0.9)] / self.I_bg[:int(len(self.I_bg)*0.9)]
                I_bg_frac = savgol_filter(I_bg_frac,1000,2) 
                self.factor = min(I_bg_frac)
                if FactorMax != None:
                    if self.factor > FactorMax:
                        self.factor = FactorMax
                # self.factor = 1
                print(self.factor)

                self.I[i] = j - self.factor*self.I_bg
                if min(self.I[i]) < 0:
                    self.I[i] = j - self.factor*self.I_bg - min(self.I[i])
        else:
            I = 0
            count = 0
            self.bg_offset = 0
            if len(self.plato_bg_list[0][0]) > (round_to_multiple(heat_ranges["ramp1_start"],self.average)/self.average):
                self.bg_offset = int(len(self.plato_bg_list[0][0]) - (round_to_multiple(heat_ranges["ramp1_start"],self.average)/self.average) - 1)
                print('BG is offset by:  ' + str(self.bg_offset) + '  number of averaged scans')
                count = self.bg_offset
            step = 1
            for i,j in enumerate(self.I):  
                
                if i > round_to_multiple(heat_ranges["ramp1_start"],self.average)/self.average and step < 2:
                    step = 2
                    count = 0

                if i-1 == round_to_multiple(heat_ranges["ramp1_end"],self.average)/self.average:
                    step = 3
                    count = 0

                if i-1 == round_to_multiple(heat_ranges["ramp2_start"],self.average)/self.average:
                    step = 4
                    count = 0

                if i-1 == round_to_multiple(heat_ranges["ramp2_end"],self.average)/self.average:
                    step = 5
                    count = 0


                if step == 2:
                    try:
                        self.I_bg =  self.ramp_bg_list[0][0][count]
                        if self.glassGr == True:
                            self.Gr_bg = self.ramp_bg_list[1][0][count]
                        self.I_bg_name = self.ramp_bg_list[0][0][count]
                        print('Scan:  ' + str(count),'    in Ramping with lenght: ' + str(len(self.ramp_bg_list[0][0])-1), '   step in procedure:  ' + str(step))
                    except:
                        print('optimize step 2')

                if step == 4:
                    try:
                        self.I_bg = self.ramp_bg_list[0][1][count]
                        if self.glassGr == True:
                            self.Gr_bg = self.ramp_bg_list[1][1][count]
                        self.I_bg_name = self.ramp_bg_list[0][1][count]  
                        print('Scan:  ' + str(count),'    in Ramping with lenght: ' + str(len(self.ramp_bg_list[0][1])-1), '   step in procedure:  ' + str(step))
                    except:
                        print('optimize step 4')

                if mode == 'last':
                    if step == 1:
                        self.I_bg =  self.plato_bg_list[0][0][count]
                        if self.glassGr == True:
                            self.Gr_bg =  self.plato_bg_list[1][0][count]
                        self.I_bg_name = self.plato_bg_list[0][0][count]
                        print('Scan:  ' + str(count),'    in Plato with lenght: ' + str(len(self.plato_bg_list[0][0])-1), '   step in procedure:  ' + str(step))

                    if step == 3:
                        try:
                            self.I_bg = self.plato_bg_list[0][1][count]
                            if self.glassGr == True:
                                self.Gr_bg = self.plato_bg_list[1][1][count]
                            self.I_bg_name = self.plato_bg_list[0][1][count]
                            print('Scan:  ' + str(count),'    in Plato with lenght: ' + str(len(self.plato_bg_list[0][1])-1), '   step in procedure:  ' + str(step))
                        except:
                            print('optimize step 3')

                    if step == 5:
                        try:
                            self.I_bg = self.plato_bg_list[0][2][count]
                            if self.glassGr == True:
                                self.Gr_bg = self.plato_bg_list[1][2][count]
                            self.I_bg_name = self.plato_bg_list[0][2][count]
                            print('Scan:  ' + str(count),'    in Plato with lenght: ' + str(len(self.plato_bg_list[0][2])-1), '   step in procedure:  ' + str(step))
                        except:
                            print('optimize step 5')


                if mode == 'average':
                    if step == 1:
                        self.I_bg = average_list[0]
                        

                    if step == 3:
                        self.I_bg = average_list[1]
                        

                    if step == 5:
                        self.I_bg = average_list[2]
                
                if empty_glass_int != 0:  
                    print("empty subtracted")
                    print(empty_glass_int/1e6)
                    self.I_empty_glass = self.I_empty_glass[:len(j)]  
                    j = j - self.I_empty_glass*max(j)*(empty_glass_int/1e6)
                if self.glassGr == True:
                    self.factor = max(self.Gr[i]) / max(self.Gr_bg)
                    print(self.factor)
                    print('max Gr:  ' + str(max(self.Gr[i])))
                    print('max Gr bg:  ' + str(max(self.Gr_bg)))                    
                else:
                    I_bg_frac = j[Qmin:Qmax] / self.I_bg[Qmin:Qmax] 
                    self.factor = min(I_bg_frac-deviation)
                if FactorMax != None:# and step < 3:
                    if self.factor > FactorMax:
                        self.factor = FactorMax
                
                subtracted_I = j - self.factor*self.I_bg
                turns = 0
                while min(subtracted_I[Qmin:Qmax]) < 0 and turns < 1000:
                    min_index = np.where(min(subtracted_I[Qmin:Qmax])==subtracted_I[Qmin:Qmax])[0][0]
                    self.factor =  j[int(Qmin+min_index)] / self.I_bg[int(Qmin+min_index)]
                    #subtracted_I[Qmin:Qmax] = subtracted_I[Qmin:Qmax] - min(subtracted_I[Qmin:Qmax])
                    subtracted_I = j - self.factor*self.I_bg 
                    turns = turns + 1
                
                self.raw_factor_list.append(self.factor)
                
                sub_baseline = 0 #self.baselineCorrection2(subtracted_I,int(len(subtracted_I)/75))
                subtracted_I = subtracted_I - sub_baseline
                
                turns = 0
                while min(subtracted_I[Qmin:Qmax]) < 0 and turns < 1000:
                    min_index = np.where(min(subtracted_I[Qmin:Qmax])==subtracted_I[Qmin:Qmax])[0][0]
                    self.factor =  j[int(Qmin+min_index)] / self.I_bg[int(Qmin+min_index)]
                    #subtracted_I[Qmin:Qmax] = subtracted_I[Qmin:Qmax] - min(subtracted_I[Qmin:Qmax])
                    subtracted_I = j - self.factor*self.I_bg - sub_baseline
                    turns = turns + 1

                j = j -sub_baseline 
                subtracted_I = np.clip(subtracted_I,0.001,None)
                
                
                
                min_list = [l for l,v in enumerate(subtracted_I) if v == 0.001]
                for l in min_list:
                    j[l] = self.I_bg[l]*self.factor
                j[:Qmin] = self.I_bg[:Qmin]*self.factor
                j =j+my_grain_of_dirt
                # import matplotlib.pyplot as plt
                # # plt.plot(self.q,j)
                # plt.plot(self.q,j-self.factor*self.I_bg)
                # # plt.plot(self.q,subtracted_I)
                # plt.xlim(left=1.5)
                # plt.ylim(bottom=0)
                # plt.show()
                
                if i ==0:
                    outputDatFile(self.projectPath + '/processed/Dat_files/meas_type3_corrected_after_Dat/',self.q,[j],(i+1)*self.average,self.sampleName, clear_folder=True)
                    outputDatFile(self.projectPath + '/processed/Dat_files/bg_type3_corrected_after_Dat/',self.q,[self.I_bg],(i+1)*self.average,'background_'+self.sampleName, clear_folder=True)
                else:
                    outputDatFile(self.projectPath + '/processed/Dat_files/meas_type3_corrected_after_Dat/',self.q,[j],(i+1)*self.average,self.sampleName)
                    outputDatFile(self.projectPath + '/processed/Dat_files/bg_type3_corrected_after_Dat/',self.q,[self.I_bg],(i+1)*self.average,'background_'+self.sampleName)
                # self.outputMatFile(rootdir + '/processed/' + self.sampleName + '/Mat_files/meas_type3_corrected_after_Mat/')
                
                self.subtracted_I.append(subtracted_I)
                print('BG_scale:  ' + str(self.factor))
                self.factor_list.append(self.factor)
                count = count + 1

        return self.logger, self.factor, self.raw_factor_list, self.factor_list, self.I_bg, self.I_bg_name, self.bg_matrix, self.subtracted_I, self.plato_bg_list[0], self.ramp_bg_list[0], self.q_empty_glass, self.I_empty_glass, self.bg_offset
    
    
    # def bgSubtraction(self,plato_bg_list, ramp_bg_list, heat_ranges, bg_sampleName, mode: str='last', FactorMax: float=0.99,single_BG_file: str=None, deviation: float=0, empty_glass_int: float=0, empty_glass: str=None):
    #     """
    #     Arguments
    #     ----------
    #     plato_bg_list:  list of array from the plato related background scans     
    #     ramp_bg_list:   list of array from the ramp related background scans
    #     heat_ranges:    heat ranges dictionary including the start and end of the temp ramp
    #     bg_sampleName:  Name of the BG sample for .dat output
    #     mode:           choose "last" to repeat the last background scan or "average" to use an everage of the plato 
    #     FactorMax:      maximum factor allowed for the background intesity subtracted from the data
    #     single_bg_file: path to a single background if only one background file should be subtracted from all data
    #     """
    #     # Log current event
    #     self.logger['Background was subtracted'] = {'heat ranges used': heat_ranges,
    #                                                 'Name of the background': bg_sampleName,
    #                                                 'Mode used for shorter background array': mode,
    #                                                 'Maximum factor allowed': FactorMax,
    #                                                 'Path to single background file (if used)': single_BG_file}
        
    #     print('\n\n Background factors and ')
        
    #     if empty_glass != None and empty_glass != 0.0: 
    #         print("Empty Glass used")
    #         with open(empty_glass) as f:
    #             lines = [line for line in f if line.startswith('#')]
    #         skiprows_of_files = len(lines)
    #         self.q_empty_glass, self.I_empty_glass = np.loadtxt(empty_glass,skiprows=skiprows_of_files).T
    #     else:
    #         self.q_empty_glass = 0
    #         self.I_empty_glass = 0
    #     self.plato_bg_list = plato_bg_list
    #     self.ramp_bg_list = ramp_bg_list
    #     self.factor_list = []
    #     self.subtracted_I = []
    #     if single_BG_file == None:
    #         if mode == 'last':
    #             for i,j in enumerate(self.plato_list[0]):
    #                 if len(j) > len(self.plato_bg_list[0][i]):
    #                     for k in range(int(len(j)-len(self.plato_bg_list[0][i]))):
    #                         self.plato_bg_list[0][i].append(self.plato_bg_list[0][i][-1])
    #         if mode == 'average':
    #             average = 0
    #             average_list = []
    #             for i,j in enumerate(self.plato_bg_list[0]): 
    #                 for a,b in enumerate(j): 
    #                     average = average + getIfile(b)
    #                 average = average/len(j)
    #                 average_list.append(average)
    #                 average = 0

    #     print('Lenght of 1st Plato BG:  ' + str(len(self.plato_bg_list[0][0])),'    Lenght of 2nd Plato BG:  ' + str(len(self.plato_bg_list[0][1])), '    Lenght of 1st Ramping BG:  ' + str(len(self.ramp_bg_list[0][0])))
    #     print('Lenght of 1st Plato Measurement:  ' + str(len(self.plato_list[0][0])),'    Lenght of 2nd Plato Measurement:  ' + str(len(self.plato_list[0][1])), '    Lenght of 1st Ramping Measurement:  ' + str(len(self.ramp_list[0][0])))
        
    #     self.bg_matrix = self.plato_bg_list[0][0] + self.ramp_bg_list[0][0] + self.plato_bg_list[0][1]
    #     if self.glassGr == True:
    #         self.bg_matrix_Gr = self.plato_bg_list[1][0] + self.ramp_bg_list[1][0] + self.plato_bg_list[1][1]
    #     outputDatFile(rootdir + '/../../processed/' + self.sampleName + '/Dat_files/bg_Dat/',self.q,self.bg_matrix,self.average,bg_sampleName)

    #     if single_BG_file != None:
    #         self.I_bg = getIfile(single_BG_file)
    #         for i,j in enumerate(self.I):  
    #             I_bg_frac = j[:int(len(j)*0.9)] / self.I_bg[:int(len(self.I_bg)*0.9)]
    #             I_bg_frac = savgol_filter(I_bg_frac,1000,2) 
    #             self.factor = min(I_bg_frac)
    #             if FactorMax != None:
    #                 if self.factor > FactorMax:
    #                     self.factor = FactorMax
    #             # self.factor = 1
    #             print(self.factor)

    #             self.I[i] = j - self.factor*self.I_bg
    #             if min(self.I[i]) < 0:
    #                 self.I[i] = j - self.factor*self.I_bg - min(self.I[i])
    #     else:
    #         I = 0
    #         count = 0
    #         self.bg_offset = 0
    #         if len(self.plato_bg_list[0][0]) > (round_to_multiple(heat_ranges["ramp1_start"],self.average)/self.average):
    #             self.bg_offset = int(len(self.plato_bg_list[0][0]) - (round_to_multiple(heat_ranges["ramp1_start"],self.average)/self.average) - 1)
    #             print('BG is offset by:  ' + str(self.bg_offset) + '  number of averaged scans')
    #             count = self.bg_offset
    #         step = 1
    #         for i,j in enumerate(self.I):  
    #             if i > round_to_multiple(heat_ranges["ramp1_start"],self.average)/self.average and step < 2:
    #                 step = 2
    #                 count = 0

    #             if i-1 == round_to_multiple(heat_ranges["ramp1_end"],self.average)/self.average:
    #                 step = 3
    #                 count = 0

    #             if i-1 == round_to_multiple(heat_ranges["ramp2_start"],self.average)/self.average:
    #                 step = 4
    #                 count = 0

    #             if i-1 == round_to_multiple(heat_ranges["ramp2_end"],self.average)/self.average:
    #                 step = 5
    #                 count = 0


    #             if step == 2:
    #                 try:
    #                     self.I_bg =  self.ramp_bg_list[0][0][count]
    #                     if self.glassGr == True:
    #                         self.Gr_bg = self.ramp_bg_list[1][0][count]
    #                     self.I_bg_name = self.ramp_bg_list[0][0][count]
    #                     print('Scan:  ' + str(count),'    in Ramping with lenght: ' + str(len(self.ramp_bg_list[0][0])-1), '   step in procedure:  ' + str(step))
    #                 except:
    #                     print('optimize step 2')

    #             if step == 4:
    #                 try:
    #                     self.I_bg = self.ramp_bg_list[0][1][count]
    #                     if self.glassGr == True:
    #                         self.Gr_bg = self.ramp_bg_list[1][1][count]
    #                     self.I_bg_name = self.ramp_bg_list[0][1][count]  
    #                     print('Scan:  ' + str(count),'    in Ramping with lenght: ' + str(len(self.ramp_bg_list[0][1])-1), '   step in procedure:  ' + str(step))
    #                 except:
    #                     print('optimize step 4')

    #             if mode == 'last':
    #                 if step == 1:
    #                     self.I_bg =  self.plato_bg_list[0][0][count]
    #                     if self.glassGr == True:
    #                         self.Gr_bg =  self.plato_bg_list[1][0][count]
    #                     self.I_bg_name = self.plato_bg_list[0][0][count]
    #                     print('Scan:  ' + str(count),'    in Plato with lenght: ' + str(len(self.plato_bg_list[0][0])-1), '   step in procedure:  ' + str(step))

    #                 if step == 3:
    #                     try:
    #                         self.I_bg = self.plato_bg_list[0][1][count]
    #                         if self.glassGr == True:
    #                             self.Gr_bg = self.plato_bg_list[1][1][count]
    #                         self.I_bg_name = self.plato_bg_list[0][1][count]
    #                         print('Scan:  ' + str(count),'    in Plato with lenght: ' + str(len(self.plato_bg_list[0][1])-1), '   step in procedure:  ' + str(step))
    #                     except:
    #                         print('optimize step 3')

    #                 if step == 5:
    #                     try:
    #                         self.I_bg = self.plato_bg_list[0][2][count]
    #                         if self.glassGr == True:
    #                             self.Gr_bg = self.plato_bg_list[1][2][count]
    #                         self.I_bg_name = self.plato_bg_list[0][2][count]
    #                         print('Scan:  ' + str(count),'    in Plato with lenght: ' + str(len(self.plato_bg_list[0][2])-1), '   step in procedure:  ' + str(step))
    #                     except:
    #                         print('optimize step 5')


    #             if mode == 'average':
    #                 if step == 1:
    #                     self.I_bg = average_list[0]
                        

    #                 if step == 3:
    #                     self.I_bg = average_list[1]
                        

    #                 if step == 5:
    #                     self.I_bg = average_list[2]
                
    #             if empty_glass_int != 0 and empty_glass_int != None:  
    #                 self.I_empty_glass = self.I_empty_glass[:len(j)]  
    #                 j = j - self.I_empty_glass*max(j)*empty_glass_int
    #             if self.glassGr == True:
    #                 self.factor = max(self.Gr[i]) / max(self.Gr_bg)
    #                 print(self.factor)
    #                 print('max Gr:  ' + str(max(self.Gr[i])))
    #                 print('max Gr bg:  ' + str(max(self.Gr_bg)))                    
    #             else:
    #                 I_bg_frac = j[:int(len(j)*0.9)] / self.I_bg[:int(len(self.I_bg)*0.9)] 
    #                 # I_bg_frac = savgol_filter(I_bg_frac,11,2) 
    #                 self.factor = min(I_bg_frac-deviation)
    #             if FactorMax != None and step < 3:
    #                 if self.factor > FactorMax:
    #                     self.factor = FactorMax
    #             subtracted_I = j - self.factor*self.I_bg#
    #             # import matplotlib.pyplot as plt
    #             # plt.plot(self.q,subtracted_I)
    #             # plt.show()
    #             if min(subtracted_I) < 0:
    #                 print("Mi subtracted_I:  " + str(min(subtracted_I)))
    #                 self.subtracted_I.append(subtracted_I - min(subtracted_I))
    #                 min_index = np.where(min(subtracted_I)==subtracted_I)[0][0]
    #                 print("Min index:  " + str(min_index))
    #                 self.factor = (j[min_index] / self.I_bg[min_index]) 
    #                 # self.factor = self.factor + self.factor*(min(subtracted_I) / (max(j[:int(len(j)*0.9)]) - min(subtracted_I)) ) 
    #                 #self.factor = self.factor     #- 1 + (max(subtracted_I)/ (max(subtracted_I) + min(subtracted_I)))
    #             else:
    #                 self.subtracted_I.append(subtracted_I)
    #             print('BG_scale:  ' + str(self.factor))
    #             self.factor_list.append(self.factor)
    #             count = count + 1

    #     # Return self varibles to main class
    #     return self.logger, self.factor, self.factor_list, self.I_bg, self.I_bg_name, self.bg_matrix, self.subtracted_I, self.plato_bg_list[0], self.ramp_bg_list[0], self.q_empty_glass, self.I_empty_glass, self.bg_offset
    
    def bg_glassSubtraction(self, bg_sampleName, empty_glass: str=None, empty_glass_int: float=0.0):
        """
        Arguments
        ----------
        bg_sampleName:          Name of the BG sample for .dat output
        empty_glass:            choose "last" to repeat the last background scan or "average" to use an everage of the plato 
        empty_glass_int:        Intesnsity of the glass file
        """
        # Log current event
        self.logger['Glass was subtracted from Background for XRD analyses'] = {
                                                                            'Name of the background': bg_sampleName,
                                                                            'Glass subtration scaling': empty_glass,
                                                                            }
        
        if empty_glass != None: 
            with open(empty_glass) as f:
                lines = [line for line in f if line.startswith('#')]
            skiprows_of_files = len(lines)
            self.q_empty_glass, self.I_empty_glass = np.loadtxt(empty_glass,skiprows=1).T
        for i,j in enumerate(self.I): 
            print(i)
            self.I[i] = j - self.I_empty_glass[:len(j)]*empty_glass_int 
            
            
    #----------------------------------------------------------------------------------------#
    #                                   Correct for Baseline                                 #
    #----------------------------------------------------------------------------------------#
    def baselineCorrection(self,max_half_window):
        """
        Arguments
        ----------
        max_half_window:    Window size of the applied SNIP baseline correction
        """
        # Log current event
        self.logger['Baseline was corrected for'] = {'Max of the half window': max_half_window,}
        
        baseline_fitter = Baseline(x_data=self.q)
        self.baseline = []
        for i, j in enumerate(self.I):
            self.I[i] = self.I[i]- baseline_fitter.snip(j, max_half_window=max_half_window, decreasing=True, smooth_half_window=0)[0]
            self.baseline.append(baseline_fitter.snip(j, max_half_window=max_half_window, decreasing=True, smooth_half_window=0)[0])
            if min(self.I[i]) < 0:
                self.I[i] = self.I[i].clip(min=0)
                
        # Return self varibles to main class
        return self.logger, self.I, self.baseline

    #----------------------------------------------------------------------------------------#
    #                                          Filter                                        #
    #----------------------------------------------------------------------------------------#
    def Filter(self,Filter,strength,order: int=2):
        """
        Arguments
        ----------
        Filter:     Type of Filter to use (lFilter or savgol)
        strength:   Strength with what the filter is applied 
        order:      Order of polinominal magnitude 
        """
        self.logger['Data was filtered'] = {   'Filtertype': Filter,
                                                'Used window lenght': strength,
                                                'Order of polinominal magnitude':order}
        count = 0
        for i in self.I:
            #applying a filter if set
            if Filter == "savgol":
                    self.I[count] = savgol_filter(i,strength,order)
            elif Filter == "whittaker":
                self.I[count] = whittaker_smooth(i,strength,d=order)
            count = count + 1
            
        # Return self varibles to main class
        return self.logger, self.I



    #----------------------------------------------------------------------------------------#
    #                                      Interpolation                                     #
    #----------------------------------------------------------------------------------------#
    def smooth_interpolation(self, output_multiplier):
        """
        Arguments
        ----------
        output_multiplier:  number by which the data points are increased
        """
        # Log current event
        self.logger['Data was interpolated'] = {'Points were multiplied by': output_multiplier,}
        
        # Determine the original number of points in the array
        for i,j in enumerate(self.I):
            n_points = len(j)
            
            # Create a new array of x-values for the output array
            x_new = np.linspace(self.q[0], self.q[-1], n_points*output_multiplier)
            
            # Use cubic spline interpolation to interpolate the data
            self.I[i] = np.interp(x_new, self.q, j, period=n_points)
        self.q = x_new
        
        # Return self varibles to main class
        return self.logger, self.q, self.I

    #----------------------------------------------------------------------------------------#
    #                              generate the PDF with PDFgetX3                            #
    #----------------------------------------------------------------------------------------#
    def generateBatchPDF(self, dataPath, bgPath, outputPath, Composition, Qmaxinst, Qmax, Qmin, Rmax, Rpoly):
        """
        Arguments
        ----------
        dataPath:       path to .dat files of the meas signal    
        bgPath:         path to .dat files of the background signal, if already subtracted set to None
        outoutPath:     path to the output directory 
        Composition:    composition of the material of interest
        Qmaxinst:       maximum of q which correlates with background 
        Qmax:           maximum of q to be used for the Fourier Transform
        Qmin:           minimum of q to be used for the Fourier Transform
        Rmax:           maximum of r after the Fourier Transform
        Rpoly:          background correction factor from PDFgetX3
        """
        self.logger['PDF was batch processed'] = {'Path to the data files': dataPath,
                                            'Path to the background files': bgPath,
                                            'Path to output directory': outputPath,
                                            'Compostion': Composition,
                                            'Qmaxinst': Qmaxinst,
                                            'Qmax':Qmax,
                                            'Qmin': Qmin,
                                            'Rmax': Rmax,
                                            'Rpoly': Rpoly}
        if os.path.exists(outputPath):
            del_folder_content(outputPath)
        ensure_dir(outputPath)
        i = 0
        bg_file_list = []
        if bgPath != None:
            for bg_file in sorted(pathlib.Path(bgPath).iterdir()):
                bg_file_list.append(bg_file) 
        for file in sorted(pathlib.Path(dataPath).iterdir()):
            if bgPath != None:
                try:
                    if 'meas_type1_uncorrected_Dat' in dataPath: 
                        process = subprocess.Popen(['pdfgetx3',str(file), '--force=true', '--format=QA', '--mode=xray', '--composition='+str(Composition), '--qmaxinst='+str(Qmaxinst), '--qmin='+str(Qmin), '--qmax='+str(Qmax), '--rpoly='+str(Rpoly), '--rmax='+str(Rmax),'-t=gr', '-b='+str(bg_file_list[int(i+self.bg_offset)]), '--bgscale='+str(self.raw_factor_list[i]), '-o='+str(outputPath)+'/@b'], stdout=subprocess.PIPE, text=True)
                    if  'meas_type2_corrected_prior_Dat' in dataPath: 
                        process = subprocess.Popen(['pdfgetx3',str(file), '--force=true', '--format=QA', '--mode=xray', '--composition='+str(Composition), '--qmaxinst='+str(Qmaxinst), '--qmin='+str(Qmin), '--qmax='+str(Qmax), '--rpoly='+str(Rpoly), '--rmax='+str(Rmax),'-t=gr', '-b='+str(bg_file_list[int(i+self.bg_offset)]), '--bgscale='+str(self.factor_list[i]), '-o='+str(outputPath)+'/@b'], stdout=subprocess.PIPE, text=True)
                    else:
                        process = subprocess.Popen(['pdfgetx3',str(file), '--force=true', '--format=QA', '--mode=xray', '--composition='+str(Composition), '--qmaxinst='+str(Qmaxinst), '--qmin='+str(Qmin), '--qmax='+str(Qmax), '--rpoly='+str(Rpoly), '--rmax='+str(Rmax),'-t=gr', '-b='+str(bg_file_list[int(i)]), '--bgscale='+str(self.factor_list[i]), '-o='+str(outputPath)+'/@b'], stdout=subprocess.PIPE, text=True)
                except:
                    print('unoptimized BG')
            else:   
                process = subprocess.Popen(['pdfgetx3',str(file), '--force=true', '--format=QA', '--mode=xray', '--composition='+str(Composition), '--qmaxinst='+str(Qmaxinst), '--qmin='+str(Qmin), '--qmax='+str(Qmax), '--rpoly='+str(Rpoly), '--rmax='+str(Rmax),'-t=gr', '-o='+str(outputPath)+'/@b'], stdout=subprocess.PIPE, text=True)
            while True:
                output = process.stdout.readline()
                if output:
                    print(output.strip())
                result = process.poll()
                if result is not None:
                    break
            i = i + 1
        return self.logger


    #----------------------------------------------------------------------------------------#
    #                              generate the PDF with PDFgetX3                            #
    #----------------------------------------------------------------------------------------#
    def generateGlassyPDF(self, dataPath, outputPath, Qmaxinst, Qmax, Qmin):
        """
        Arguments
        ----------
        dataPath:       path to .dat files of the meas signal    
        bgPath:         path to .dat files of the background signal, if already subtracted set to None
        outoutPath:     path to the output directory 
        Composition:    composition of the material of interest
        Qmaxinst:       maximum of q which correlates with background 
        Qmax:           maximum of q to be used for the Fourier Transform
        Qmin:           minimum of q to be used for the Fourier Transform
        Rmax:           maximum of r after the Fourier Transform
        Rpoly:          background correction factor from PDFgetX3
        """
        # Log current event
        self.logger['Glassy PDF was generated'] = {'Path to the data files': dataPath,
                                            'Path to output directory': outputPath,
                                            'Compostion': 'SiO2',
                                            'Qmaxinst': Qmaxinst,
                                            'Qmax':Qmax,
                                            'Qmin': Qmin,
                                            'Rmin': 1.5,
                                            'Rmax': 1.7,
                                            'Rstep': 0.001,
                                            }
        
        self.glassGr = True
        
        ensure_dir(outputPath)
        i = 0
        for file in sorted(pathlib.Path(dataPath).iterdir()):
            process = subprocess.Popen(['pdfgetx3',str(file), '--force=true', '--format=QA', '--mode=xray', '--composition=SiO2', '--qmaxinst='+str(Qmaxinst), '--qmin='+str(Qmin), '--qmax='+str(Qmax), '--rpoly=0.9', '--rstep=0.001', '--rmin=1.5', '--rmax=1.7','-t=gr', '--bgscale=0','-o='+str(outputPath)+'/@b'], stdout=subprocess.PIPE, text=True)
            while True:
                output = process.stdout.readline()
                if output:
                    print(output.strip())
                result = process.poll()
                if result is not None:
                    break
            i = i + 1
            
        # Return self varibles to main class
        return self.logger, self.glassGr




    #----------------------------------------------------------------------------------------#
    #                              generate the PDF with PDFgetX3                            #
    #----------------------------------------------------------------------------------------#
    def generatePDF(self, dataPath, meas_scan_number, bgPath, bg_scan_number, outputPath, Composition, Qmaxinst, Qmax, Qmin, Rmax, Rpoly, scale, scatter_supression: float=0.0):
        """ Using PDFgetX3 as a subprocess we can integrate the programm into our PDF generation.
            !!! Caution in some consoles this gives an error (in Visual Studio Code I can not run this, but in the powershell or bash I can) !!!

        Args:
            dataPath (str): Path to .dat files of the meas signal.
            meas_scan_number (int): Index of the scan to be processed
            bgPath (str):  Path to .dat files of the background signal, if already subtracted set to None.
            bg_scan_number (int): Index of the background scan to be processed
            outputPath (str): Path to the output directory.
            Composition (_type_): Molecular formula of the material of interest.
           Qmin (float): Minuimum Q to include in the PDF generation. Defaults to 0.1.
            Qmax (float): Maximum Q to include in the PDF generation. Defaults to 30.
            Rpoly (float): Polynomal degree of data correction applied (best case: rpoly < Qmin). Defaults to 0.9.
            Rstep (float): Step with of the PDF (Smaller steps result in longer calculation times). Defaults to 0.01.
            Rmin (float): Minimum r the PDF is calculated from. Defaults to 0.5.
            Rmax (float): Maximum r the PDF is calculated to. Defaults to 30.
            scale (float): constant manual choosen scale to subtract background
            scatter_supression (float, optional): Reduce the BG factor by a constant ammount to counter outliers. Defaults to 0.0.

        References:
            PDFgetX3:
                Code available from https://www.diffpy.org/products/pdfgetx.html
                Related Paper:  P. Juhás and T. Davis, C. L. Farrow, S. J. L. Billinge,https://journals.iucr.org/j/issues/2013/02/00/nb5046/index.html
                                J. Appl. Crystallogr. 46, 560-566 (2013)
    
        Returns:
            self.looger: updates the log file
        """
        
        if scale == None:
            scale = self.factor
            
        
        # Logging the event
        self.logger['PDF was generated'] = {'Path to the data files': dataPath,
                                            'Path to the background files': bgPath,
                                            "Meas scan number": meas_scan_number,
                                            "Background scan number": bg_scan_number,
                                            'Path to output directory': outputPath,
                                            'Compostion': Composition,
                                            'Qmaxinst': Qmaxinst,
                                            'Qmax':Qmax,
                                            'Qmin': Qmin,
                                            'Rmax': Rmax,
                                            'Rpoly': Rpoly}
        
        # Delete older folder contains (thus different averaging doesn´t effect storing new data in the same directory)
        if os.path.exists(outputPath):
            del_folder_content(outputPath)
        
        # Create directory if not exsiting
        ensure_dir(outputPath)
        
        file = sorted(pathlib.Path(bgPath).iterdir())[0]
        print(str(file))
        
        # Get the names of the BG files in the given directory
        if bgPath != None:
            bg_file = sorted(pathlib.Path(bgPath).iterdir())[0]
            process = subprocess.Popen(['pdfgetx3',str(file), '--force=true', '--format=QA', '--mode=xray', '--composition='+str(Composition), '--qmaxinst='+str(Qmaxinst), '--qmin='+str(Qmin), '--qmax='+str(Qmax), '--rpoly='+str(Rpoly), '--rmax='+str(Rmax),'-t=gr', '-b='+str(bg_file), '--bgscale='+str(scale), '-o='+str(outputPath)+'/@b'], stdout=subprocess.PIPE, text=True)
                
        # If no BG is given we generate the PDF here
        else:   
            process = subprocess.Popen(['pdfgetx3',str(file), '--force=true', '--format=QA', '--mode=xray', '--composition='+str(Composition), '--qmaxinst='+str(Qmaxinst), '--qmin='+str(Qmin), '--qmax='+str(Qmax), '--rpoly='+str(Rpoly), '--rmax='+str(Rmax),'-t=gr', '-o='+str(outputPath)+'/@b'], stdout=subprocess.PIPE, text=True)
            
            # Get the consol outputs from PDFgetX3 and print them to the current consol
            while True:
                output = process.stdout.readline()
                if output:
                    print(output.strip())
                result = process.poll()
                if result is not None:
                    break
            
        return self.logger



















