try:
    from backbone.functions.utils import ensure_dir, del_folder_content
except:
    from python_scripts.backbone.functions.utils import ensure_dir, del_folder_content
from time import time
from scipy import io as sp_io 
import numpy as np
import os



class Export_PDF_Plot:
    
    #----------------------------------------------------------------------------------------#
    #                              save data for reloading                                   #
    #----------------------------------------------------------------------------------------#
    def saveReload(self,outputPath):
        """
        Arguments
        ----------
        outputPath: Path to output directory
        """    
        if outputPath != None:
            ensure_dir(outputPath)
        sp_io.savemat(outputPath + "/" + self.sampleName +'.mat', {'G_of_r': self.G, 'r': self.r,'time': self.time_array ,'time_averaged': self.time_averaged_array,'temp': self.temp_array, 'temp_averaged':self.temp_averaged_array})

    #----------------------------------------------------------------------------------------#
    #                                  export temp as xy                                     #
    #----------------------------------------------------------------------------------------#
    def outputTemp(self,outputPath):
        """
        Arguments
        ----------
        outputPath: Path to output directory
        """
        if outputPath != None:
            ensure_dir(outputPath)
        #Data output file
        data = np.array([self.time_array, self.temp_array])
        outfname = outputPath + "/" + self.path.split("/")[-1].split(".")[0]+ '.xy' #output directory and filename
        np.savetxt(outfname, data.T,header = 'Time(s) Temperature(°C)')



    
class Export_PDF_Gen:
    
            

    #----------------------------------------------------------------------------------------#
    #                              save data for reloading                                   #
    #----------------------------------------------------------------------------------------#
    def saveReload(self,outputPath):
        """
        Arguments
        ----------
        outputPath: Path to output directory
        """    
        self.logger['Reload .mat was created'] = {   'Output Path': outputPath,}
        if outputPath != None:
            ensure_dir(outputPath)
        sp_io.savemat(outputPath + "/" + self.sampleName +'.mat', {'I_matrix': self.I, 'q_axis': self.q,'time': self.time_array ,'time_averaged': self.time_average_array,'temp': self.temp_array, 'temp_averaged':self.temp_average_array})
        return self.logger

    #----------------------------------------------------------------------------------------#
    #                                  Export .mat for MCR-ALS                                #
    #----------------------------------------------------------------------------------------#
    def outputMatFile(self,outputPath):
        """
        Arguments
        ----------
        outputPath: Path to output directory
        """    
        self.logger['Data was stored in .mat'] = {   'Output Path': outputPath,}
        self.logger['Mat file saved'] = {'Output path': outputPath + "/" + self.sampleName +'.mat'}
        if outputPath != None:
            ensure_dir(outputPath)
        sp_io.savemat(outputPath + "/" + self.sampleName +'.mat', {'I_matrix': self.I_original, 'q_axis': self.q,'I_processed':self.I ,'time': self.time_array ,'time_averaged': self.time_averaged_array,'temp': self.temp_array, 'temp_averaged':self.temp_averaged_array})
        return self.logger

    #----------------------------------------------------------------------------------------#
    #                                   Export .dat to export                                #
    #----------------------------------------------------------------------------------------#
    def outputDatFile(self,outputPath,average, data_slice: int=None):
        """
        Arguments
        ----------
        outputPath: Path to output directory
        """
        self.logger['Data was saved in .dat for ever scan'] = {   'Output Path': outputPath,}
        t0 = time()
        self.logger['Dat files saved'] = {'Output path': outputPath}
        if os.path.exists(outputPath):
            list_files = [f for f in os.listdir(outputPath) if f.endswith('.txt')]
            list_files.sort()
            if len(list_files) > 2:
                old_average = abs(int(list_files[2][-9:-4]) - int(list_files[1][-9:-4]))
            else: 
                old_average = 0
        if outputPath != None:
            if os.path.exists(outputPath) and average != old_average:
                del_folder_content(outputPath)
                ensure_dir(outputPath)
            else:
                ensure_dir(outputPath)
        #Data output file
        if data_slice == None:
            for i, j in enumerate(self.I):
                data = np.array([self.q, j])
                outfname = outputPath + "/" + self.sampleName + "_" + str((i+1)*self.average + self.startScan).zfill(5) + '.txt' #output directory and filename
                np.savetxt(outfname, data.T,header = 'q (Angstrom), Intensity (a.u.)')
        else:
            data = np.array([self.q, self.I[data_slice]])
            outfname = outputPath + "/" + self.sampleName + "_" + str((data_slice+1)*self.average + self.startScan).zfill(5) + '.txt' #output directory and filename
            np.savetxt(outfname, data.T,header = 'q (Angstrom), Intensity (a.u.)')
        print("\n Time for generating Dat")
        print("------------------------------------------------------------------")
        print("Time: " + str((time() - t0)/60) + "\n") 
        return self.logger

    #----------------------------------------------------------------------------------------#
    #                                Output the loged actions                                #
    #----------------------------------------------------------------------------------------#
    def PrintLogFile(self,outputPath):
        """
        Arguments
        ----------
        outputPath: Destination of the .log file
        """
        if outputPath != None:
            ensure_dir(outputPath)
        log_file = open(outputPath + '/' + self.sampleName + ".log", "w")
        for x in self.logger:
            log_file.write('\n' + str(x) + '\n')
            for y in self.logger[x]:
                log_file.write(str(y) + '  :  ' + str(self.logger[x][y]) +'\n')
        log_file.close()

    
#----------------------------------------------------------------------------------------#
#                             Output Dat from BG for PDFgetX3                            #
#----------------------------------------------------------------------------------------#
def outputDatFile(outputPath,x,matrix,average,sampleName,clear_folder: bool=False):
    """
    Arguments
    ----------
    outputPath: Path to output directory
    """
    t0 = time()
    if outputPath != None:
        if os.path.exists(outputPath) and clear_folder==True:
            del_folder_content(outputPath)
        ensure_dir(outputPath)
    #Data output file
    for i, j in enumerate(matrix):
        data = np.array([x, j])
        outfname = outputPath + "/" + sampleName + "_" + str((i+1)*average).zfill(5) + '.txt' #output directory and filename
        np.savetxt(outfname, data.T,header = 'q (Angstrom), Intensity (a.u.)')
