from datetime import datetime
import numpy as np
from time import sleep
#from numba import jit, njit, cuda
import os

class Import_PDF_Plot:
    #@jit(target_backend='cuda',nopython=True) 
    def import_data(self):
        sleep(0.5)
        list_files = [f for f in os.listdir(self.path) if f.endswith('.gr') if not "dark" in f]
        list_files.sort()
        

        data = []
        skiprows_of_files = 0
        for i,frame in enumerate(list_files):
            if i == 0:
                with open(self.path+frame) as f:
                    lines = [line for line in f if line.startswith('#')]
                skiprows_of_files = 27#len(lines)
            data.append(np.genfromtxt(self.path+frame, skip_header=skiprows_of_files))
        data = np.array(data)

        np.shape(data)
        
        
        
        self.r = data[0,:,0]
        self.G = data[:,:,1] 
        return self.r, self.G
        

    #----------------------------------------------------------------------------------------#
    #                                get temp and time infos                                 #
    #----------------------------------------------------------------------------------------#
    def getTempTime(self,path):
        """Collects and averages the time and temperature points from the .metadata files.

        Args:
            path (str): Path to the metadata directory.
            self.counter_PDF (dict): names the comment on which time and temp follow in the metadata.

        Returns:
            self: Forwards updated self varibles to the main class.
        """
        start_time = 0
        self.time_array = [] 
        self.time_average_array = []
        self.temp_array = []
        self.temp_average_array = []
        count = 1
        if self.without_dark == True:
            list_files = [f for f in os.listdir(path) if f.endswith('.metadata') if not "dark" in f]
        else:
            list_files = [f for f in os.listdir(path) if f.endswith('.metadata')]
        list_files.sort(key=str.lower)
        for file in list_files:
            file = path + file  
            f = open(file, "r")
            for line in f:
                line = line.rstrip()  # remove '\n' at end of line
                if  self.counter_PDF['time'] in line:
                    if start_time == 0:
                        #if count % 60 == 0:
                        start_time = datetime.strptime(line[int(len(self.counter_PDF['time'])):int(len(self.counter_PDF['time'])+10)] + " " + line[int(len(self.counter_PDF['time'])+13):], "%Y.%m.%d %H:%M:%S.%f")
                        self.time_array.append(0+self.timeOfset)
                    else:
                        delta_t = datetime.strptime(line[int(len(self.counter_PDF['time'])):int(len(self.counter_PDF['time'])+10)] + " " + line[int(len(self.counter_PDF['time'])+13):], "%Y.%m.%d %H:%M:%S.%f") - start_time
                        delta_t = delta_t.total_seconds() / 60
                        self.time_array.append(delta_t+self.timeOfset)
                        if count % self.average == 0:
                            self.time_average_array.append(delta_t+self.timeOfset)
                if self.counter_PDF['temperature'] in line:
                    temp_string = line[int(len(self.counter_PDF['temperature'])):int(len(self.counter_PDF['temperature'])+8)]
                    if '"' in temp_string or ',' in temp_string:
                        temp_string = temp_string.replace('"','')
                        temp_string = temp_string.replace(',','')
                    self.temp_array.append(float(temp_string))
                    if count % self.average == 0:
                        temp_string = line[int(len(self.counter_PDF['temperature'])):int(len(self.counter_PDF['temperature'])+8)]
                        if '"' in temp_string or ',' in temp_string:
                            temp_string = temp_string.replace('"','')
                            temp_string = temp_string.replace(',','')
                        self.temp_average_array.append(float(temp_string))
            f.close()
            count = count + 1
        self.time_average_array = self.time_average_array[self.startScan:self.endScan]
        self.temp_average_array = self.temp_average_array[self.startScan:self.endScan]
        return self.time_array, self.temp_array, self.temp_average_array, self.temp_average_array






    
class Import_PDF_Gen:
    # @jit(target_backend='cuda',nopython=False) 
    def import_data(self): #connection
        if self.without_dark == True:
            list_files = [f for f in os.listdir(self.path) if f.endswith(self.ends_with) if not "dark" in f]
        else:
            list_files = [f for f in os.listdir(self.path) if f.endswith(self.ends_with)]
        list_files.sort(key=str.lower)
        if list_files == []:
            raise AttributeError('No files with: ' + self.ends_with + '  found in directory: ' + self.path)
        if len(list_files) <= self.endScan:
            self.number_of_files = int(len(list_files)/self.number_of_steps)
        else:
            self.number_of_files = int(self.endScan/self.number_of_steps)

        self.q = []
        self.I = []
        
        
        skiprows_of_files = 0
        for i,j in enumerate(list_files):
            if i == 0:
                with open(self.path+j) as f:
                    lines = [line for line in f if line.startswith('#')]
                skiprows_of_files = len(lines)
            if i >= self.startScan and i <= self.endScan :
                # if i % self.number_of_steps == 0 and i > 0:
                try:
                    x, matrix, dI = np.loadtxt(self.path+j, skiprows=skiprows_of_files).T
                except:
                    x, matrix = np.loadtxt(self.path+j, skiprows=skiprows_of_files).T
                
                self.q.append(x)
                self.I.append(matrix)
        
        
        self.q = np.array(self.q[0])
        self.I = np.array(self.I)
        # datablock = [self.q, self.I]
        # connection.send(datablock)
        # connection.close()
        return self.q, self.I
    
    #----------------------------------------------------------------------------------------#
    #                                get temp and time infos                                 #
    #----------------------------------------------------------------------------------------#
    # @jit(target_backend='cuda',nopython=False) 
    def getTempTime(self,path): #connection
        """Collects and averages the time and temperature points from the .metadata files.

        Args:
            path (str): Path to the metadata directory.
            self.counter_PDF (dict): names the comment on which time and temp follow in the metadata.

        Returns:
            self: Forwards updated self varibles to the main class.
        """
        self.logger['Temperature and Time loaded'] = {   'Path to .metadata': path,}
        start_time = 0
        self.time_array = [] 
        self.time_average_array = []
        self.temp_array = []
        self.temp_average_array = []
        count = 1
        if self.without_dark == True:
            list_files = [f for f in os.listdir(path) if f.endswith('.metadata') if not "dark" in f]
        else:
            list_files = [f for f in os.listdir(path) if f.endswith('.metadata')]
        list_files.sort(key=str.lower)
        for file in list_files:
            file = path + file  
            if count >= self.startScan and count <= self.endScan :
                if count % self.number_of_steps == 0:      
                    f = open(file, "r")
                    for line in f:
                        line = line.rstrip()  # remove '\n' at end of line
                        if self.counter_PDF['time'] in line:
                            if start_time == 0:
                                #if count % 60 == 0:
                                start_time = datetime.strptime(line[int(len(self.counter_PDF['time'])):int(len(self.counter_PDF['time'])+10)] + " " + line[int(len(self.counter_PDF['time'])+13):], "%Y.%m.%d %H:%M:%S.%f")
                                self.time_array.append(0+self.timeOfset)
                            else:
                                delta_t = datetime.strptime(line[int(len(self.counter_PDF['time'])):int(len(self.counter_PDF['time'])+10)] + " " + line[int(len(self.counter_PDF['time'])+13):], "%Y.%m.%d %H:%M:%S.%f") - start_time
                                delta_t = delta_t.total_seconds() / 60
                                self.time_array.append(delta_t+self.timeOfset)
                                if count % self.average == 0:
                                    self.time_average_array.append(delta_t+self.timeOfset)
                        if self.counter_PDF['temperature'] in line:
                            temp_string = line[int(len(self.counter_PDF['temperature'])):int(len(self.counter_PDF['temperature'])+8)]
                            if '"' in temp_string or ',' in temp_string:
                                temp_string = temp_string.replace('"','')
                                temp_string = temp_string.replace(',','')
                            self.temp_array.append(float(temp_string))
                            if count % self.average == 0:
                                temp_string = line[int(len(self.counter_PDF['temperature'])):int(len(self.counter_PDF['temperature'])+8)]
                                if '"' in temp_string or ',' in temp_string:
                                    temp_string = temp_string.replace('"','')
                                    temp_string = temp_string.replace(',','')
                                self.temp_average_array.append(float(temp_string))
                    f.close()
            count = count + 1
        # datablock = [self.time_array, self.temp_array, self.time_average_array, self.temp_average_array, self.logger]
        # connection.send(datablock)
        # connection.close()
        return self.time_array, self.temp_array, self.time_average_array, self.temp_average_array, self.logger
    
    #----------------------------------------------------------------------------------------#
    #                                     Glass PDF loading                                  #
    #----------------------------------------------------------------------------------------#
    def GlassPDFload(self, input_dir):
        self.logger['BG scale determined by Glass Peak Intensity'] = {' ': 0,}
        
        self.glassGr = True
        list_files_glassy_Gr = [f for f in os.listdir(input_dir) if f.endswith('.gr')]
        
        #The list of data files is the sorted to confirm to the order 00001-99999 
        list_files_glassy_Gr.sort(key=str.lower)

        self.Gr = []
        skiprows_of_files = 0
        for i,j in enumerate(list_files_glassy_Gr):
            if i == 0:
                with open(input_dir + '/' + j) as f:
                    lines = [line for line in f if line.startswith('#')]
                skiprows_of_files = len(lines)
            r, Gr = np.loadtxt(input_dir + '/' + j,skiprows=skiprows_of_files).T
            self.Gr.append(Gr)
        return self.Gr, self.glassGr   
