"""

:platform: Unix, Windows 
:last changed: 2023-01-19

.. moduleauthor:: Tjark Leon Raphael Gröne <tgroene@physnet.uni-hamburg.de>


This Code is used to import integrated .chi data and processes it to to the PDF .gr 
"""
import sys, getopt
from backbone.PDF_generation_class import PDF_generation_class
from backbone.functions.export import outputDatFile
import pathlib
rootdir = str(pathlib.Path(__file__).parent.resolve())

from backbone.settings.routines.Gen_ZnS_routine import *

# BG file name (directories for integrated data and metadata should be keept constant throughout the beamtime)
bg_sampleName = 'fc_bg_zns_10cm_water' #filename
bg_path = '/asap3/petra3/gpfs/p21.1/2023/data/11017732/processed/pe/ac_cell/' + bg_sampleName + "/" #integrated data path
bg_tempPath = '/asap3/petra3/gpfs/p21.1/2023/data/11017732/raw/pe/ac_cell/' + bg_sampleName + "/" #metadat path

# Measurment file name (directories for integrated data and metadata should be keept constant throughout the beamtime)
meas_sampleName = 'fc_zns_10cm_water'  #filename
meas_path = '/asap3/petra3/gpfs/p21.1/2023/data/11017732/processed/pe/ac_cell/' + meas_sampleName + "/" #integrated data path
meas_tempPath = '/asap3/petra3/gpfs/p21.1/2023/data/11017732/raw/pe/ac_cell/' + meas_sampleName + "/" #metadat path


bg_Reload = rootdir + '/processed/' + meas_sampleName + '/Reload_files/bg_Reload/new_counter_' + bg_sampleName + '.mat' 
meas_Reload = rootdir + '/processed/' + meas_sampleName + '/Reload_files/meas_Reload/new_counter_' + meas_sampleName + '.mat'

empty_glass_int = 0 #0.014 #0.0210336
empty_glass = None #'/home/groenetj/Documents/Beamtime0423/empty_ac_cell_inlet_180_10_Av120.chi'


references = {  'ZnS Sph': {'path': rootdir + "/References/ZnS Sphalerite.int", #Change here references name (Copper) and path (/References/cu.int) to acces other references
                            'skiprows': 2,  #skiprows and Energy are all normed to be the same 
                            'energy': 100} , 
                'ZnS Wz':{'path': rootdir + "/References/ZnS Wurzit.int",
                            'skiprows': 2,
                            'energy': 100},
                 'ZnO cb': {'path': rootdir + "/References/ZnO Cubic.int",
                            'skiprows': 2,
                            'energy': 100} ,
                'ZnO hex':{'path': rootdir + "/References/ZnO Hexagonal.int",
                            'skiprows': 2,
                            'energy': 100},
                'Zn': {'path': rootdir + "/References/Zn.int",
                            'skiprows': 2,
                            'energy': 100} ,} #you an add more if nessicarry, just add another three lined block 


ramp_start_temp = 35 #set a little bit higher than baseline temperature from which the measurment or bg started
plato_start_temp = 156 #temperatur on which to stay in °C
average = 60  #number of scans to average over
Qmaxinst = 20  #Qmax at which the mask sets in 
FactorMax=1.3

check_phases_at_scan = 20 #choose last scan processed by the live integration devided by the number of averged scans

###### PDF params
Composition = 'ZnO' # material the PDF should focus on (NP material)
Qmaxinst_integrate = 10.2 #Q at which in the XRD peaks dominate over the noise (Qmaxinst from xPDF)
Qmax = 9 #Qmax that the PDF will process
Qmin = 1.5 #minimal Q normally allways around 0.1
Rmax = 30 #maximal R to process the PDF to
Rpoly = 0.9 #polynomal degree which affects getPDFx bg subtraction

#----------------------------------------------------------------------------------------#
#                                                                                        #
#                                    Process data set                                    #
#                                                                                        #
#----------------------------------------------------------------------------------------#

#Loading the background data
startScan = 0
endScan = 1e10
bg = PDF_generation_class(path=bg_path, tempPath=bg_tempPath, sampleName=bg_sampleName, startScan=startScan, endScan=endScan, ends_with=ends_with, reload=bg_Reload, average=average)
# bg.getTempPoints(ramp_start_temp,plato_start_temp)
bg_heat_ranges = {  "ramp1_start": bg.rampstart_index,   
                    "ramp1_end": int(bg.platostart_index), #Sometimes the ramps dont match due to rounding error, needs little correction by +- half average 
                    "ramp2_start": 100000,
                    "ramp2_end": 100000,}
endScan = bg.platoend_index
# bg = PDF_generation_class(path=bg_path, tempPath=bg_tempPath, sampleName=bg_sampleName, startScan=startScan, endScan=endScan, ends_with=ends_with, reload=bg_Reload)
# bg.saveReload(rootdir + '/processed/' + meas_sampleName + '/Reload_files/bg_Reload/')

# bg.norm(Qmaxinst=Qmaxinst)
# bg.outputDatFile(rootdir + '/processed/' + meas_sampleName + '/Dat_files/bg_Dat/')

# bg.getRampPlatoLists(bg_heat_ranges)
# # bg.generateGlassyPDF(rootdir + '/processed/' + meas_sampleName + '/Dat_files/bg_Dat/',rootdir + '/processed/' + meas_sampleName + '/PDF_files/Glassy_PDF/bg_PDF/' ,Qmaxinst, Qmax=15, Qmin=Qmin)
# # bg.GlassPDFload(rootdir + '/processed/' + meas_sampleName + '/PDF_files/Glassy_PDF/bg_PDF/')
# # bg.getGlassRampPlatoLists(bg_heat_ranges)
bg_plato_list = bg.plato_list
bg_ramp_list = bg.ramp_list
# bg.PrintLogFile(rootdir + '/processed/' + meas_sampleName + '/Log_files/bg_Log/')
#-----------------------------------------------------------------------------------------#

#Loading the measurment data
meas = PDF_generation_class(path=meas_path, tempPath=meas_tempPath, sampleName=meas_sampleName, startScan=startScan, endScan=1e10, ends_with=ends_with, reload=meas_Reload, without_dark=False, average=average)
meas.getTempPoints(ramp_start_temp,plato_start_temp)
meas_heat_ranges = {  "ramp1_start": meas.rampstart_index,
                    "ramp1_end": meas.platostart_index,
                    "ramp2_start": 100000,
                    "ramp2_end": 100000,}
endScan = meas.platoend_index
# meas = PDF_generation_class(path=meas_path, tempPath=meas_tempPath, sampleName=meas_sampleName, startScan=startScan, endScan=endScan, ends_with=ends_with, reload=meas_Reload, without_dark=False)
meas.saveReload(rootdir + '/processed/' + meas_sampleName + '/Reload_files/meas_Reload/')

print('\n ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~')
print('           Reload files created, fast loading possible')
print(' ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n')

meas.norm(Qmaxinst=Qmaxinst)
meas.outputDatFile(rootdir + '/processed/' + meas_sampleName + '/Dat_files/meas_type1_uncorrected_Dat/')
original = meas.I
#-----------------------------------------------------------------------------------------#

#Background subtraction, filtering and baseline correction
meas.getRampPlatoLists(meas_heat_ranges)
# meas.generateGlassyPDF(rootdir + '/processed/' + meas_sampleName + '/Dat_files/meas_type1_uncorrected_Dat/',rootdir + '/processed/' + meas_sampleName +  '/PDF_files/Glassy_PDF/meas_PDF/' ,Qmaxinst, Qmax=15, Qmin=Qmin)
# meas.GlassPDFload(rootdir + '/processed/' + meas_sampleName +  '/PDF_files/Glassy_PDF/meas_PDF/')
# meas.getGlassRampPlatoLists(meas_heat_ranges)
meas.bgSubtraction(bg_plato_list,bg_ramp_list,meas_heat_ranges,bg_sampleName,FactorMax=FactorMax, deviation=0.0, empty_glass_int=empty_glass_int, empty_glass=empty_glass)

meas.outputMatFile(rootdir + '/processed/' + meas_sampleName + '/Mat_files/meas_type1_uncorrected_Mat/')



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
# meas.I = meas.subtracted_I
# meas.baselineCorrection(40)
# meas.Filter('whittaker',20)
# meas.checkPhases(check_phases_at_scan,references=references,savefig='png',outputPath=rootdir + '/processed/' + meas_sampleName + '/XRD_Plots/meas_phase_check/', maxQ=30)
# meas.plot2dTemp(savefig='svg',outputPath=rootdir + '/processed/' + meas_sampleName + '/XRD_Plots/meas_2D_plot/')
# meas.plot1dvsTime(step=2,maxQ=15, max_index=30, plot_temp=True,savefig='svg',outputPath=rootdir + '/processed/' + meas_sampleName + '/XRD_Plots/meas_1D_plot/')
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#

Qmaxinst = Qmaxinst_integrate
outputPath = rootdir + '/processed/' + meas_sampleName + '/PDF_files/meas_type1_uncorrected_PDF/'
dataPath = rootdir + '/processed/' + meas_sampleName + '/Dat_files/meas_type1_uncorrected_Dat/'
bgPath = rootdir + '/processed/' + meas_sampleName + '/Dat_files/bg_Dat/'

# import matplotlib.pyplot as plt
# import numpy as np
# bgi = []
# for bg_file in sorted(pathlib.Path(bgPath).iterdir()):
#     q, I_bg = np.loadtxt(bg_file,skiprows=1).T 
#     bgi.append(I_bg)
# i = 0
# for file in sorted(pathlib.Path(dataPath).iterdir()):
#     q, I = np.loadtxt(file,skiprows=1).T 
#     # val = I- factor*bgi[i]
#     val = I- meas.factor_list[i]*bgi[i+3]
#     plt.plot(meas.q,val)
#     # plt.plot(meas.q,I_bg)
#     plt.show()
#     i = i +1

meas.generatePDF(dataPath, bgPath, outputPath, Composition, Qmaxinst, Qmax, Qmin, Rmax, Rpoly)
meas.PrintLogFile(rootdir + '/processed/' + meas_sampleName + '/Log_files/meas_type1_uncorrected_Log/')


#-----------------------------------------------------------------------------------------#
meas.I = meas.subtracted_I
meas.baselineCorrection(40)
# meas.Filter('whittaker',30)

meas.outputDatFile(rootdir + '/processed/' + meas_sampleName + '/Dat_files/meas_type2_corrected_prior_Dat/')

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
outputPath = rootdir + '/processed/' + meas_sampleName + '/PDF_files/meas_type2_corrected_prior_PDF'
dataPath = rootdir + '/processed/' + meas_sampleName + '/Dat_files/meas_type2_corrected_prior_Dat/'
bgPath = None
meas.generatePDF(dataPath, bgPath, outputPath, Composition, Qmaxinst, Qmax, Qmin, Rmax, Rpoly)
meas.PrintLogFile(rootdir + '/processed/' + meas_sampleName + '/Log_files/meas_type2_corrected_prior_Log/')
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#




#-----------------------------------------------------------------------------------------#
meas.I = original


meas_I = meas.I
bg_I = bg_plato_list[0][0] + bg_ramp_list[0][0] + bg_plato_list[0][1]

for i,j in enumerate(meas.I):
    meas.I[i] = meas.I[i] - meas.baseline[i] 
    bg_I[i] = bg_I[i]  -meas.baseline[i] #- meas.I_empty_glass*empty_glass_int

        
outputDatFile(rootdir + '/processed/' + meas_sampleName + '/Dat_files/bg_type3_corrected_after_Dat/', meas.q, bg_I, average,  bg_sampleName)


meas.outputMatFile(rootdir + '/processed/' + meas_sampleName + '/Mat_files/meas_type3_corrected_after_Mat/')
meas.outputDatFile(rootdir + '/processed/' + meas_sampleName + '/Dat_files/meas_type3_corrected_after_Dat/')

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
Qmaxinst = Qmaxinst_integrate
outputPath = rootdir + '/processed/' + meas_sampleName + '/PDF_files/meas_type3_corrected_after_PDF'
dataPath = rootdir + '/processed/' + meas_sampleName + '/Dat_files/meas_type3_corrected_after_Dat/'
bgPath = rootdir + '/processed/' + meas_sampleName + '/Dat_files/bg_type3_corrected_after_Dat/'
meas.generatePDF(dataPath, bgPath, outputPath, Composition, Qmaxinst, Qmax, Qmin, Rmax, Rpoly)
meas.PrintLogFile(rootdir + '/processed/' + meas_sampleName + '/Log_files/meas_type3_corrected_after_Log/')




#$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
def create_pdf(project_dir):
    meas.outputDatFile(project_dir + '/processed/' + meas_sampleName + '/Dat_files/meas_type1_uncorrected_Dat/')
    original = meas.I
    #-----------------------------------------------------------------------------------------#

    #Background subtraction, filtering and baseline correction
    meas.getRampPlatoLists(meas_heat_ranges)
    
    # meas.generateGlassyPDF(rootdir + '/processed/' + meas_sampleName + '/Dat_files/meas_type1_uncorrected_Dat/',rootdir + '/processed/' + meas_sampleName +  '/PDF_files/Glassy_PDF/meas_PDF/' ,Qmaxinst, Qmax=15, Qmin=Qmin)
    # meas.GlassPDFload(rootdir + '/processed/' + meas_sampleName +  '/PDF_files/Glassy_PDF/meas_PDF/')
    # meas.getGlassRampPlatoLists(meas_heat_ranges)
    
    meas.bgSubtraction(bg_plato_list,bg_ramp_list,meas_heat_ranges,bg_sampleName,FactorMax=FactorMax, deviation=0.0, empty_glass_int=empty_glass_int, empty_glass=empty_glass)


    meas.outputMatFile(project_dir + '/processed/' + meas_sampleName + '/Mat_files/meas_type1_uncorrected_Mat/')

    Qmaxinst = Qmaxinst_integrate
    outputPath = project_dir + '/processed/' + meas_sampleName + '/PDF_files/meas_type1_uncorrected_PDF/'
    dataPath = project_dir + '/processed/' + meas_sampleName + '/Dat_files/meas_type1_uncorrected_Dat/'
    bgPath = project_dir + '/processed/' + meas_sampleName + '/Dat_files/bg_Dat/'
    meas.generatePDF(dataPath, bgPath, outputPath, Composition, Qmaxinst, Qmax, Qmin, Rmax, Rpoly)
    meas.PrintLogFile(project_dir + '/processed/' + meas_sampleName + '/Log_files/meas_type1_uncorrected_Log/')


    #-----------------------------------------------------------------------------------------#
    meas.I = meas.subtracted_I
    meas.baselineCorrection(40)
    # meas.Filter('whittaker',30)

    meas.outputDatFile(project_dir + '/processed/' + meas_sampleName + '/Dat_files/meas_type2_corrected_prior_Dat/')

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
    outputPath = project_dir + '/processed/' + meas_sampleName + '/PDF_files/meas_type2_corrected_prior_PDF'
    dataPath = project_dir + '/processed/' + meas_sampleName + '/Dat_files/meas_type2_corrected_prior_Dat/'
    bgPath = None
    meas.generatePDF(dataPath, bgPath, outputPath, Composition, Qmaxinst, Qmax, Qmin, Rmax, Rpoly)
    meas.PrintLogFile(project_dir + '/processed/' + meas_sampleName + '/Log_files/meas_type2_corrected_prior_Log/')
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#


    #-----------------------------------------------------------------------------------------#
    meas.I = original


    meas_I = meas.I
    bg_I = bg_plato_list[0][0] + bg_ramp_list[0][0] + bg_plato_list[0][1]

    for i,j in enumerate(meas.I):
        meas.I[i] = meas.I[i] - meas.baseline[i] 
        bg_I[i] = bg_I[i]  -meas.baseline[i] #- meas.I_empty_glass*empty_glass_int

            
    outputDatFile(project_dir + '/processed/' + meas_sampleName + '/Dat_files/bg_type3_corrected_after_Dat/', meas.q, bg_I, average,  bg_sampleName)


    meas.outputMatFile(project_dir + '/processed/' + meas_sampleName + '/Mat_files/meas_type3_corrected_after_Mat/')
    meas.outputDatFile(project_dir + '/processed/' + meas_sampleName + '/Dat_files/meas_type3_corrected_after_Dat/')

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
    Qmaxinst = Qmaxinst_integrate
    outputPath = project_dir + '/processed/' + meas_sampleName + '/PDF_files/meas_type3_corrected_after_PDF'
    dataPath = project_dir + '/processed/' + meas_sampleName + '/Dat_files/meas_type3_corrected_after_Dat/'
    bgPath = project_dir + '/processed/' + meas_sampleName + '/Dat_files/bg_type3_corrected_after_Dat/'
    meas.generatePDF(dataPath, bgPath, outputPath, Composition, Qmaxinst, Qmax, Qmin, Rmax, Rpoly)
    meas.PrintLogFile(project_dir + '/processed/' + meas_sampleName + '/Log_files/meas_type3_corrected_after_Log/')


def load_measurment(project_dir,meas_path,counter_PDF,bg_tempPath,bg_sampleName,meas_Reload,ramp_start_temp,plato_start_temp,average,Qmaxinst,bg_glass):
    ends_with = '.dat' # this can differe for differnet live integrations used, but should be the same for one beamtime

    #Loading the background data
    startScan = 0
    endScan = 1e10
    
    try:
        if bg_Reload != None:
            print('Reload accepted')
            meas = PDF_generation_class(path=meas_path, tempPath=meas_tempPath, sampleName=meas_sampleName, startScan=startScan, endScan=1e10, counter_PDF=counter_PDF, ends_with=ends_with, reload=meas_Reload, without_dark=False, average=average)
            meas.start()
            meas.join()
    except:
        print('No valid reload present')
        print(bg_Reload)
        meas = PDF_generation_class(path=meas_path, tempPath=meas_tempPath, sampleName=meas_sampleName, startScan=startScan, endScan=1e10, counter_PDF=counter_PDF, ends_with=ends_with, reload=meas_Reload, without_dark=False, average=average)
        meas.start()
        meas.join()
    #Loading the measurment data
    meas.saveReload(project_dir + '/processed/' + meas_sampleName + '/Reload_files/meas_Reload/')
    meas.getTempPoints(ramp_start_temp,plato_start_temp)
    meas_heat_ranges = {  "ramp1_start": meas.rampstart_index,
                        "ramp1_end": meas.platostart_index,
                        "ramp2_start": 100000,
                        "ramp2_end": 100000,}
    endScan = meas.platoend_index
    meas.saveReload(project_dir + '/processed/' + meas_sampleName + '/Reload_files/meas_Reload/')

    print('\n ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~')
    print('           Reload files created, fast loading possible')
    print(' ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n')

    meas.norm(Qmaxinst=Qmaxinst)
    meas.PrintLogFile(project_dir + '/processed/' + meas_sampleName + '/Log_files/meas_Log/')
    return meas_heat_ranges, endScan
    
def load_background(project_dir,bg_path,counter_PDF,bg_tempPath,bg_sampleName, meas_sampleName,bg_Reload,ramp_start_temp,plato_start_temp,average,Qmaxinst,bg_glass):
    ends_with = '.dat' # this can differe for differnet live integrations used, but should be the same for one beamtime

    #Loading the background data
    startScan = 0
    endScan = 1e10
    try:
        if bg_Reload != None:
            print('Reload accepted')
        bg = PDF_generation_class(path=bg_path, tempPath=bg_tempPath, sampleName=bg_sampleName, startScan=startScan, endScan=endScan, counter_PDF=counter_PDF, ends_with=ends_with, reload=bg_Reload, average=average)
        bg.start()
        bg.join()
    except:
        print('No valid bg reload present')
    
    bg.getTempPoints(ramp_start_temp,plato_start_temp)
    bg_heat_ranges = {  "ramp1_start": bg.rampstart_index,   
                    "ramp1_end": int(bg.platostart_index), #Sometimes the ramps dont match due to rounding error, needs little correction by +- half average 
                    "ramp2_start": 100000,
                    "ramp2_end": 100000,}
    endScan = bg.platoend_index

    bg.norm(Qmaxinst=Qmaxinst)
    bg_plato_list = bg.plato_list
    bg_ramp_list = bg.ramp_list
    bg.PrintLogFile(project_dir + '/processed/' + meas_sampleName + '/Log_files/bg_Log/')
    
    return bg_plato_list, bg_ramp_list, bg_heat_ranges


def plot_1D_XRD_Slice(project_dir,bg_sampleName,slice,average):
    try:
        print('Reload accepted')
        startScan = 0
        endScan = 1e10
        bg_Reload = project_dir + '/processed/' + bg_sampleName + '/Reload_files/bg_Reload/' + bg_sampleName + '.mat'
        meas = PDF_generation_class(path=None, tempPath=None, sampleName=meas_sampleName, startScan=startScan, endScan=endScan, counter_PDF=None, ends_with=None, reload=bg_Reload, average=average)
        bg.start()
        bg.join()
    except:
        print('Error')
    # Just a Dummy as reference to trigger the last XRD scan to plot
    references = {  'Dummy': {'path': rootdir + "/References/Dummy.int",
                                'skiprows': 2,
                                'energy': 100} ,}
    check_phases_at_scan = slice #choose last scan processed by the live integration devided by the number of averged scans
    meas.I = meas.subtracted_I
    meas.baselineCorrection(40)
    meas.Filter('whittaker',20)
    meas.checkPhases(check_phases_at_scan,references=references,savefig='png',outputPath=project_dir + '/processed/' + meas_sampleName + '/XRD_Plots/meas_phase_check/', maxQ=30)
    meas.PrintLogFile(project_dir + '/processed/' + bg_sampleName + '/Log_files/bg_Log/')
    
def plot_1D_XRD_vs_time(project_dir,bg_sampleName,slice,average):
    try:
        print('Reload accepted')
        startScan = 0
        endScan = 1e10
        bg_Reload = project_dir + '/processed/' + bg_sampleName + '/Reload_files/bg_Reload/' + bg_sampleName + '.mat'
        bg = PDF_generation_class(path=None, tempPath=None, sampleName=bg_sampleName, startScan=startScan, endScan=endScan, counter_PDF=None, ends_with=None, reload=bg_Reload, average=average)
        bg.start()
        bg.join()
    except:
        print('Error')
    
    bg.plot1dvsTime(step=2,maxQ=15, max_index=slice, plot_temp=True,savefig='svg',outputPath=project_dir + '/processed/' + bg_sampleName + '/XRD_Plots/bg_1D_plot/')
    bg.PrintLogFile(project_dir + '/processed/' + bg_sampleName + '/Log_files/bg_Log/')

def plot_2D_XRD_vs_time(project_dir,bg_sampleName, average):
    try:
        print('Reload accepted')
        startScan = 0
        endScan = 1e10
        bg_Reload = project_dir + '/processed/' + bg_sampleName + '/Reload_files/bg_Reload/' + bg_sampleName + '.mat'
        bg = PDF_generation_class(path=None, tempPath=None, sampleName=bg_sampleName, startScan=startScan, endScan=endScan, counter_PDF=None, ends_with=None, reload=bg_Reload, average=average)
        bg.start()
        bg.join()
    except:
        print('Error')
    bg.plot2dTemp(savefig='svg',outputPath=project_dir + '/processed/' + bg_sampleName + '/XRD_Plots/bg_2D_plot/')
    bg.PrintLogFile(project_dir + '/processed/' + bg_sampleName + '/Log_files/bg_Log/')


def main(argv):
    opts, args = getopt.getopt(argv,'m:f:p:a:s:r:n:',['mode=','bgfile=','project=','starttempbg=','reactempbg=','average=','qmaxinst=','slice=','tempdir=',"reload=","bgglass=","samplename="])
    for opt, arg in opts:
        # if opt == '-h':
        #     print('bla')
        #     sys.exit()
        if opt in ('-m','--mode'):
            mode = arg
        if mode == "init":
            if opt in ('-f','--bgfile'):
                bg_path = arg + "/"
            elif opt in ('-p','--project'):
                project_dir = arg + "/"
            elif opt in ('-t1','--starttempbg'):
                ramp_start_temp = float(arg)
            elif opt in ('-t2','--reactempbg'):
                plato_start_temp = float(arg)
            elif opt in ('-a','--average'):
                average = float(arg)
            elif opt in ('-qmax','--qmaxinst'):
                Qmaxinst = float(arg)
            elif opt in ('-temp','--tempdir'):
                temp_dir = arg + "/"
            elif opt in ('-r','--reload'):
                reload_dir = arg + "/"
            elif opt in ('-bgglass','--bgglass'):
                bg_glass = arg + "/"
                
        elif mode == "1D_slice":
            if opt in ('-s','--slice'):
                slice = int(arg)
            elif opt in ('-n','--samplename'):
                bg_sampleName = arg
            elif opt in ('-p','--project'):
                project_dir = arg + "/"
            elif opt in ('-a','--average'):
                average = float(arg)
        elif mode == "1D_vs_time":
            if opt in ('-s','--slice'):
                slice = int(arg)
            elif opt in ('-n','--samplename'):
                bg_sampleName = arg
            elif opt in ('-p','--project'):
                project_dir = arg + "/"
            elif opt in ('-a','--average'):
                average = float(arg)
        elif mode == "2D_vs_time":
            if opt in ('-n','--samplename'):
                bg_sampleName = arg
            elif opt in ('-p','--project'):
                project_dir = arg + "/"
            elif opt in ('-a','--average'):
                average = float(arg)
                    
    if mode == 'init':
        # BG file name (directories for integrated data and metadata should be keept constant throughout the beamtime)
        bg_sampleName = bg_path.split("/")[-2] 
        if temp_dir == 'None':
            bg_tempPath = '/asap3/petra3/gpfs/p21.1/2023/data/11017732/raw/pe/ac_cell/' + bg_sampleName + "/"  #metadat path
        else:
            bg_tempPath = temp_dir
            
        if reload_dir == 'None':
            bg_Reload = project_dir + '/processed/' + bg_sampleName + '/Reload_files/bg_Reload/' + bg_sampleName + '.mat'
        else:
            reload = pathlib.Path(project_dir + '/processed/' + bg_sampleName + '/Reload_files/bg_Reload/' + bg_sampleName + '.mat')
            if reload.is_file():
                print('Starting from Reload')
                bg_Reload = reload
            else: 
                bg_Reload = None
            
        counter_PDF = {
            "time": "dateString=",
            #"temperature": 'userComment1=HeaterSensor(A): ',
            "temperature": 'userComment1="Temp: ',
        }
        bg = load_background(project_dir,bg_path,counter_PDF,bg_tempPath,bg_sampleName,bg_Reload,ramp_start_temp,plato_start_temp,average,Qmaxinst,bg_glass)
    
    elif mode == "1D_slice":
        # try: 
        plot_1D_XRD_Slice(project_dir,bg_sampleName,slice,average)
        # except:
        #     None
    elif mode == "1D_vs_time":
        try: 
            plot_1D_XRD_vs_time(project_dir,bg_sampleName,slice,average)
        except:
            None
    elif mode == "2D_vs_time":
        # try: 
            plot_2D_XRD_vs_time(project_dir,bg_sampleName,average)
        # except:
            # None        


if __name__ == '__main__':
    main(sys.argv[1:])