"""

:platform: Unix, Windows 
:last changed: 2023-01-19

.. moduleauthor:: Tjark Leon Raphael Gröne <tgroene@physnet.uni-hamburg.de>


This Code is used to import integrated .chi data and processes it to to the PDF .gr 
"""
import pathlib
rootdir = str(pathlib.Path(__file__).parent.resolve())

try:
    from python_scripts.backbone.PDF_generation_class import PDF_generation_class
    from python_scripts.backbone.functions.export import outputDatFile
    from python_scripts.backbone.settings.reference_sets import *
except:
    from backbone.PDF_generation_class import PDF_generation_class
    from backbone.functions.export import outputDatFile
    from backbone.settings.reference_sets import *



class LOADMEAS():
    def __init__(self,projectPath: str,path: str,ramp_start_temp: float, plato_start_temp: float, references: str="FeS",
                 reload: str = None, average: int=1,tempPath: str=None,sampleName: str=None, bg_sampleName: str=None, FactorMax: float=1.3,
                 glass: str=None, glass_intens: str=None, Qmaxinst_XRD: float=30, counter_PDF: dict=None, mode: str=None,
                 temp_signatur: str='userComment1="Temperature=',time_signatur: str="dateString=", endswith: str=".dat",metadata_dir: str='/asap3/petra3/gpfs/p21.1/2023/data/11017687/raw/pe/insitu/insitu_pdf/'): #slice, bg_glass
        self.projectPath = projectPath
        if sampleName == None:
            self.sampleName = path.split("/")[-1] 
        if tempPath == None:
            self.tempPath = metadata_dir + sampleName + "/"  #metadat path
        else:
            self.tempPath = tempPath + "/"
        if path != None:
                self.path = path + "/"
        else: 
            self.path = path
        self.sampleName = sampleName
        self.bg_sampleName = bg_sampleName
        self.reload = reload
        self.average = average
        self.Qmaxinst_XRD = Qmaxinst_XRD
        self.empty_glass_int = glass_intens 
        self.empty_glass = glass
        self.ramp_start_temp = ramp_start_temp
        self.plato_start_temp = plato_start_temp
        self.endswith = endswith
        if mode == None:
            self.mode = ''
        else:
            self.mode = mode
        self.FactorMax = FactorMax

        if references != None and references != '' and references != 'Reference Pack':
            self.references = references_sets[str(references)]
        else:
            self.references = "FeS"

        if counter_PDF == None:
            self.counter_PDF = {
                "time": time_signatur,
                "temperature": temp_signatur,
            }
        
        if self.reload == None:
            print(self.projectPath + '/processed/Reload_files/meas_Reload/' + self.sampleName + '.mat')
            self.reload = pathlib.Path(self.projectPath + '/processed/Reload_files/meas_Reload/' + self.sampleName + '.mat')
            if self.reload.is_file():
                print('Starting from Reload')
                self.reload = self.reload
            else: 
                self.reload = None
        else:
            self.reload = pathlib.Path(reload)
            if self.reload.is_file():
                print('Starting from Reload')
                self.reload = self.reload
            else: 
                self.reload = None
                

        self.bg_reload = pathlib.Path(self.projectPath + '/processed/Reload_files/bg_Reload/' + self.bg_sampleName + '.mat')
        if self.bg_reload.is_file():
            print('Starting from Reload')
            self.bg_reload = self.bg_reload
        else: 
            self.bg_reload = None
        
        self.load_background()
        self.load_measurment()
        if 'MEAS_' in self.mode:
            self.bg_subtract()
        
    def load_measurment(self):
        #Loading the background data
        startScan = 0
        endScan = 1e10
        
        try:
            if self.reload != None:
                print('Reload accepted')
            self.meas = PDF_generation_class(projectPath=self.projectPath, path=self.path, tempPath=self.tempPath, sampleName=self.sampleName, startScan=startScan, endScan=1e10, counter_PDF=self.counter_PDF, reload=self.reload, without_dark=False, average=self.average, ends_with=self.endswith)
            self.meas.start()
            self.meas.join()
        except:
            print('No valid reload present')
            print(self.reload)
            self.meas = PDF_generation_class(projectPath=self.projectPath,path=self.path, tempPath=self.tempPath, sampleName=self.sampleName, startScan=startScan, endScan=1e10, counter_PDF=self.counter_PDF, without_dark=False, average=self.average, ends_with=self.endswith)
            self.meas.start()
            self.meas.join()
        #Loading the measurment data
        self.meas.join()
        print(self.mode, self.reload)
        if (self.mode==None or self.mode=="") and self.reload == None:
            self.meas.saveReload(self.projectPath + '/processed/Reload_files/meas_Reload/')
        self.meas.getTempPoints(self.ramp_start_temp,self.plato_start_temp)
        self.meas_heat_ranges = {  "ramp1_start": self.meas.rampstart_index,
                            "ramp1_end": self.meas.platostart_index,
                            "ramp2_start": 100000,
                            "ramp2_end": 100000,}
        self.endScan = self.meas.platoend_index
        if self.mode==None and self.reload == None:
            self.meas.saveReload(self.projectPath + '/processed/Reload_files/meas_Reload/')

        print('\n ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~')
        print('           Reload files created, fast loading possible')
        print(' ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n')

        self.meas.norm(Qmaxinst=self.Qmaxinst_XRD)
        self.meas.PrintLogFile(self.projectPath + '/processed/Log_files/meas_Log/')

        
    def load_background(self):
        ends_with = '.dat' # this can differe for differnet live integrations used, but should be the same for one beamtime

        #Loading the background data
        startScan = 0
        endScan = 1e10
        try:
            if self.bg_reload != None:
                print('Reload accepted')
            self.bg = PDF_generation_class(projectPath=self.projectPath,path=None, tempPath=None, sampleName=self.bg_sampleName, startScan=startScan, endScan=endScan, counter_PDF=self.counter_PDF, reload=self.bg_reload, average=self.average, ends_with=self.endswith)
            self.bg.start()
            self.bg.join()
        except:
            print('No valid bg reload present')
        
        self.bg.getTempPoints(self.ramp_start_temp,self.plato_start_temp)
        self.bg_heat_ranges = {  "ramp1_start": self.bg.rampstart_index,   
                        "ramp1_end": int(self.bg.platostart_index), #Sometimes the ramps dont match due to rounding error, needs little correction by +- half average 
                        "ramp2_start": 100000,
                        "ramp2_end": 100000,}
        self.endScan = self.bg.platoend_index

        self.bg.norm(Qmaxinst=self.Qmaxinst_XRD)
        self.bg.getRampPlatoLists(self.bg_heat_ranges)
        self.bg_plato_list = self.bg.plato_list
        self.bg_ramp_list = self.bg.ramp_list
        self.bg.PrintLogFile(self.projectPath + '/processed/Log_files/bg_Log/')
    
    def filter_and_baseline(self,filter,filter_strenght,filter_degree,baseline_strength,baseline_type,xrd_save_type):
        if filter != None:
            self.meas.Filter(filter,filter_strenght,filter_degree)
        if baseline_type != None:
            self.meas.baselineCorrection(baseline_strength)
        if xrd_save_type == "mat":
            self.meas.outputMatFile(self.projectPath + '/processed/XRD_files/XRD_Mat/')
        if xrd_save_type == "dat":
            self.meas.outputDatFile(self.projectPath + '/processed/XRD_files/XRD_Dat',average=self.average)
    
    def bg_subtract(self):
        self.meas.getRampPlatoLists(self.meas_heat_ranges)
        self.meas.bgSubtraction(self.bg_plato_list,self.bg_ramp_list,self.meas_heat_ranges,self.bg_sampleName,FactorMax=self.FactorMax, deviation=0.0, empty_glass_int=self.empty_glass_int, empty_glass=self.empty_glass)
        self.meas.I = self.meas.subtracted_I

    def create_pdf(self,Composition, Qmaxinst, Qmax, Qmin, Rmax, Rpoly):
        
        
        self.bg.outputDatFile(self.projectPath + '/processed/Dat_files/bg_Dat/',average=self.average)
        self.bg.PrintLogFile(self.projectPath + '/processed/Log_files/bg_Log/')
        self.meas.outputDatFile(self.projectPath + '/processed/Dat_files/meas_type1_uncorrected_Dat',average=self.average)
        self.original = self.meas.I
        #-----------------------------------------------------------------------------------------#

        #Background subtraction, filtering and baseline correction
        self.meas.getRampPlatoLists(self.meas_heat_ranges)
        
        # meas.generateGlassyPDF(rootdir + '/processed/Dat_files/meas_type1_uncorrected_Dat/',rootdir + '/processed/' + self.sampleName +  '/PDF_files/Glassy_PDF/meas_PDF/' ,Qmaxinst, Qmax=15, Qmin=Qmin)
        # meas.GlassPDFload(rootdir + '/processed/' + self.sampleName +  '/PDF_files/Glassy_PDF/meas_PDF/')
        # meas.getGlassRampPlatoLists(meas_heat_ranges)
        
        self.meas.bgSubtraction(self.bg_plato_list,self.bg_ramp_list,self.meas_heat_ranges,self.bg_sampleName,FactorMax=self.FactorMax, deviation=0.0, empty_glass_int=self.empty_glass_int, empty_glass=self.empty_glass)


        self.meas.outputMatFile(self.projectPath + '/processed/Mat_files/meas_type1_uncorrected_Mat/')

        outputPath = self.projectPath + '/processed/PDF_files/meas_type1_uncorrected_PDF/'
        dataPath = self.projectPath + '/processed/Dat_files/meas_type1_uncorrected_Dat/'
        bgPath = self.projectPath + '/processed/Dat_files/bg_Dat/'
        self.meas.generatePDF(dataPath, bgPath, outputPath, Composition, Qmaxinst, Qmax, Qmin, Rmax, Rpoly)
        self.meas.PrintLogFile(self.projectPath + '/processed/Log_files/meas_type1_uncorrected_Log/')


        #-----------------------------------------------------------------------------------------#
        self.meas.I = self.meas.subtracted_I
        self.meas.baselineCorrection(40)
        # meas.Filter('whittaker',30)

        self.meas.outputDatFile(self.projectPath + '/processed/Dat_files/meas_type2_corrected_prior_Dat/',average=self.average)

        #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
        outputPath = self.projectPath + '/processed/PDF_files/meas_type2_corrected_prior_PDF'
        dataPath = self.projectPath + '/processed/Dat_files/meas_type2_corrected_prior_Dat/'
        bgPath = None
        self.meas.generatePDF(dataPath, bgPath, outputPath, Composition, Qmaxinst, Qmax, Qmin, Rmax, Rpoly)
        self.meas.PrintLogFile(self.projectPath + '/processed/Log_files/meas_type2_corrected_prior_Log/')
        #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#


        #-----------------------------------------------------------------------------------------#
        self.meas.I = self.original


        self.meas_I = self.meas.I
        self.bg_I = self.bg_plato_list[0][0] + self.bg_ramp_list[0][0] + self.bg_plato_list[0][1]

        for i,j in enumerate(self.meas.I):
            self.meas.I[i] = self.meas.I[i] - self.meas.baseline[i] 
            self.bg_I[i] = self.bg_I[i]  -self.meas.baseline[i] #- meas.I_empty_glass*empty_glass_int

                
        outputDatFile(self.projectPath + '/processed/Dat_files/bg_type3_corrected_after_Dat/', self.meas.q, self.bg_I, self.average,  self.bg_sampleName)


        self.meas.outputMatFile(self.projectPath + '/processed/Mat_files/meas_type3_corrected_after_Mat/')
        self.meas.outputDatFile(self.projectPath + '/processed/Dat_files/meas_type3_corrected_after_Dat/',average=self.average)

        #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
        outputPath = self.projectPath + '/processed/PDF_files/meas_type3_corrected_after_PDF'
        dataPath = self.projectPath + '/processed/Dat_files/meas_type3_corrected_after_Dat/'
        bgPath = self.projectPath + '/processed/Dat_files/bg_type3_corrected_after_Dat/'
        self.meas.generatePDF(dataPath, bgPath, outputPath, Composition, Qmaxinst, Qmax, Qmin, Rmax, Rpoly)
        self.meas.PrintLogFile(self.projectPath + '/processed/Log_files/meas_type3_corrected_after_Log/')


    def plot_1D_XRD_Slice(self,slice):
        self.meas.checkPhases(slice,references=self.references,savefig='png',outputPath=self.projectPath + '/processed/XRD_Plots/bg_phase_check/', maxQ=9, energy=Experiment_Energy)
        self.meas.PrintLogFile(self.projectPath + '/processed/Log_files/bg_Log/')
        
    def plot_1D_XRD_vs_time(self):
        self.meas.plot1dvsTime(step=2,maxQ=15, plot_temp=True,savefig='svg',outputPath=self.projectPath + '/processed/XRD_Plots/bg_1D_plot/')
        self.meas.PrintLogFile(self.projectPath + '/processed/Log_files/bg_Log/')

    def plot_2D_XRD_vs_time(self):
        self.meas.plot2dTemp(savefig='svg',outputPath=self.projectPath + '/processed/XRD_Plots/bg_2D_plot/')
        self.meas.PrintLogFile(self.projectPath + '/processed/Log_files/bg_Log/')
        
                        
       
                
