"""

:platform: Unix, Windows 
:last changed: 2023-01-19

.. moduleauthor:: Tjark Leon Raphael Gröne <tgroene@physnet.uni-hamburg.de>


This Code is used to import integrated .chi data and processes it to to the PDF .gr 
"""
import sys, getopt
from backbone.PDF_generation_class import PDF_generation_class
import pathlib
rootdir = str(pathlib.Path(__file__).parent.resolve())

def load_background(project_dir,bg_path,counter_PDF,bg_tempPath,bg_sampleName,bg_Reload,ramp_start_temp,plato_start_temp,average,Qmaxinst,bg_glass):

    ends_with = '.dat' # this can differe for differnet live integrations used, but should be the same for one beamtime

    #Loading the background data
    startScan = 0
    endScan = 1e10
    try:
        if bg_Reload != None:
            print('Reload accepted')
        bg = PDF_generation_class(path=bg_path, tempPath=bg_tempPath, sampleName=bg_sampleName, startScan=startScan, endScan=endScan, counter_PDF=counter_PDF, ends_with=ends_with, reload=bg_Reload)
        bg.start()
        bg.join()
    except:
        print('No valid reload present')
        print(bg_Reload)
        bg = PDF_generation_class(path=bg_path, tempPath=bg_tempPath, sampleName=bg_sampleName, startScan=startScan, endScan=endScan, counter_PDF=counter_PDF, ends_with=ends_with)
        bg.start()
        bg.join()
        
    bg.saveReload(project_dir + '/processed/' + bg_sampleName + '/Reload_files/bg_Reload/')
    bg.PrintLogFile(project_dir + '/processed/' + bg_sampleName + '/Log_files/bg_Log/')
    
    
    bg.getTempPoints(ramp_start_temp,plato_start_temp)
    bg_heat_ranges = {  "ramp1_start": bg.rampstart_index,
                        "ramp1_end": int(bg.platostart_index+100),
                        "ramp2_start": 100000,
                        "ramp2_end": 100000,}
    endScan = bg.platoend_index
    # bg = PDF_generation_class(path=bg_path, tempPath=bg_tempPath, sampleName=bg_sampleName, startScan=startScan, endScan=endScan, ends_with=ends_with, reload=bg_Reload)
    bg.saveReload(project_dir + '/processed/' + bg_sampleName + '/Reload_files/bg_Reload/')
    bg.PrintLogFile(project_dir + '/processed/' + bg_sampleName + '/Log_files/bg_Log/')

    bg.norm(Qmaxinst=Qmaxinst)
    # bg.outputDatFile(project_dir + '/processed/' + bg_sampleName + '/Dat_files/bg_Dat/')

    bg.getRampPlatoLists(bg_heat_ranges)
    bg_plato_list = bg.plato_list
    bg_ramp_list = bg.ramp_list
    
    # if bg_glass != 'None':
        ####### Just glass sutraction
        #bg.bgSubtraction(bg_plato_list,bg_ramp_list,meas_heat_ranges,bg_sampleName,FactorMax=FactorMax, deviation=0.0, empty_glass_int=empty_glass_int, empty_glass=empty_glass)

    bg.PrintLogFile(project_dir + '/processed/' + bg_sampleName + '/Log_files/bg_Log/')
    
    return bg

def plot_1D_XRD_Slice(project_dir,bg_sampleName,slice,average):
    try:
        print('Reload accepted')
        startScan = 0
        endScan = 1e10
        bg_Reload = project_dir + '/processed/' + bg_sampleName + '/Reload_files/bg_Reload/' + bg_sampleName + '.mat'
        bg = PDF_generation_class(path=None, tempPath=None, sampleName=bg_sampleName, startScan=startScan, endScan=endScan, counter_PDF=None, ends_with=None, reload=bg_Reload, average=average)
        bg.start()
        bg.join()
    except:
        print('Error')
    # Just a Dummy as reference to trigger the last XRD scan to plot
    references = {  'Dummy': {'path': rootdir + "/References/Dummy.int",
                                'skiprows': 2,
                                'energy': 100} ,}
    check_phases_at_scan = slice #choose last scan processed by the live integration devided by the number of averged scans
    bg.baselineCorrection(40)
    bg.Filter('whittaker',20)
    bg.checkPhases(check_phases_at_scan,references=references,savefig='png',outputPath=project_dir + '/processed/' + bg_sampleName + '/XRD_Plots/bg_phase_check/', maxQ=9)
    bg.PrintLogFile(project_dir + '/processed/' + bg_sampleName + '/Log_files/bg_Log/')
    
def plot_1D_XRD_vs_time(project_dir,bg_sampleName,slice,average):
    try:
        print('Reload accepted')
        startScan = 0
        endScan = 1e10
        bg_Reload = project_dir + '/processed/' + bg_sampleName + '/Reload_files/bg_Reload/' + bg_sampleName + '.mat'
        bg = PDF_generation_class(path=None, tempPath=None, sampleName=bg_sampleName, startScan=startScan, endScan=endScan, counter_PDF=None, ends_with=None, reload=bg_Reload, average=average)
        bg.start()
        bg.join()
    except:
        print('Error')
    
    bg.plot1dvsTime(step=2,maxQ=15, max_index=slice, plot_temp=True,savefig='svg',outputPath=project_dir + '/processed/' + bg_sampleName + '/XRD_Plots/bg_1D_plot/')
    bg.PrintLogFile(project_dir + '/processed/' + bg_sampleName + '/Log_files/bg_Log/')

def plot_2D_XRD_vs_time(project_dir,bg_sampleName, average):
    try:
        print('Reload accepted')
        startScan = 0
        endScan = 1e10
        bg_Reload = project_dir + '/processed/' + bg_sampleName + '/Reload_files/bg_Reload/' + bg_sampleName + '.mat'
        bg = PDF_generation_class(path=None, tempPath=None, sampleName=bg_sampleName, startScan=startScan, endScan=endScan, counter_PDF=None, ends_with=None, reload=bg_Reload, average=average)
        bg.start()
        bg.join()
    except:
        print('Error')
    bg.plot2dTemp(savefig='svg',outputPath=project_dir + '/processed/' + bg_sampleName + '/XRD_Plots/bg_2D_plot/')
    bg.PrintLogFile(project_dir + '/processed/' + bg_sampleName + '/Log_files/bg_Log/')
    
    
def main(argv):
    opts, args = getopt.getopt(argv,'m:f:p:a:s:r:n:',['mode=','bgfile=','project=','starttempbg=','reactempbg=','average=','qmaxinst=','slice=','tempdir=',"reload=","bgglass=","samplename="])
    for opt, arg in opts:
        # if opt == '-h':
        #     print('bla')
        #     sys.exit()
        if opt in ('-m','--mode'):
            mode = arg
        if mode == "init":
            if opt in ('-f','--bgfile'):
                bg_path = arg + "/"
            elif opt in ('-p','--project'):
                project_dir = arg + "/"
            elif opt in ('-t1','--starttempbg'):
                ramp_start_temp = float(arg)
            elif opt in ('-t2','--reactempbg'):
                plato_start_temp = float(arg)
            elif opt in ('-a','--average'):
                average = float(arg)
            elif opt in ('-qmax','--qmaxinst'):
                Qmaxinst = float(arg)
            elif opt in ('-temp','--tempdir'):
                temp_dir = arg + "/"
            elif opt in ('-r','--reload'):
                reload_dir = arg + "/"
            elif opt in ('-bgglass','--bgglass'):
                bg_glass = arg + "/"
                
        elif mode == "1D_slice":
            if opt in ('-s','--slice'):
                slice = int(arg)
            elif opt in ('-n','--samplename'):
                bg_sampleName = arg
            elif opt in ('-p','--project'):
                project_dir = arg + "/"
            elif opt in ('-a','--average'):
                average = float(arg)
        elif mode == "1D_vs_time":
            if opt in ('-s','--slice'):
                slice = int(arg)
            elif opt in ('-n','--samplename'):
                bg_sampleName = arg
            elif opt in ('-p','--project'):
                project_dir = arg + "/"
            elif opt in ('-a','--average'):
                average = float(arg)
        elif mode == "2D_vs_time":
            if opt in ('-n','--samplename'):
                bg_sampleName = arg
            elif opt in ('-p','--project'):
                project_dir = arg + "/"
            elif opt in ('-a','--average'):
                average = float(arg)
                    
    if mode == 'init':
        # BG file name (directories for integrated data and metadata should be keept constant throughout the beamtime)
        bg_sampleName = bg_path.split("/")[-2] 
        if temp_dir == 'None':
            bg_tempPath = '/asap3/petra3/gpfs/p21.1/2023/data/11017732/raw/pe/ac_cell/' + bg_sampleName + "/"  #metadat path
        else:
            bg_tempPath = temp_dir
            
        if reload_dir == 'None':
            bg_Reload = project_dir + '/processed/' + bg_sampleName + '/Reload_files/bg_Reload/' + bg_sampleName + '.mat'
        else:
            reload = pathlib.Path(project_dir + '/processed/' + bg_sampleName + '/Reload_files/bg_Reload/' + bg_sampleName + '.mat')
            if reload.is_file():
                print('Starting from Reload')
                bg_Reload = reload
            else: 
                bg_Reload = None
            
        counter_PDF = {
            "time": "dateString=",
            #"temperature": 'userComment1=HeaterSensor(A): ',
            "temperature": 'userComment1="Temp: ',
        }
        bg = load_background(project_dir,bg_path,counter_PDF,bg_tempPath,bg_sampleName,bg_Reload,ramp_start_temp,plato_start_temp,average,Qmaxinst,bg_glass)
    
    elif mode == "1D_slice":
        # try: 
        plot_1D_XRD_Slice(project_dir,bg_sampleName,slice,average)
        # except:
        #     None
    elif mode == "1D_vs_time":
        try: 
            plot_1D_XRD_vs_time(project_dir,bg_sampleName,slice,average)
        except:
            None
    elif mode == "2D_vs_time":
        # try: 
            plot_2D_XRD_vs_time(project_dir,bg_sampleName,average)
        # except:
            # None        
   
if __name__ == '__main__':
    main(sys.argv[1:])
    





