"""

:platform: Unix, Windows 
:last changed: 2022-05-14

.. moduleauthor:: Tjark Leon Raphael Gröne <tgroene@physnet.uni-hamburg.de>

This code is used to load and plot pdf data
-- There for exitu data we will load the class called exituPDF
-- To display insitu data the insituPDF class is suitable
-- Last there is the insituPDFfitted class you can use to display 
    the different properties that the diffpy-cmi will print in the result files

"""
try:
    import backbone.PDF_class_insitu as PDF_class
except:
    import scripts.backbone.PDF_class_insitu as PDF_class
import pathlib
rootdir = str(pathlib.Path(__file__).parent.resolve())
#----------------------------------------------------------------------------------------#
#                                                                                        #
#                                      Configurations                                    #
#                                                                                        #
#----------------------------------------------------------------------------------------#

startScan = 1       #Define the first fscan to process for the plot
endScan = 180        #Define the last fscan to process for the plot
average = 60       #Averaging as used in the PDF generation
step_lenght = 3     #To shortan the number of scans in a 1D plot step lenght can be increased


figratio1D =[4,3]   #Define the window size ratio for a 1D plot
figratio2D =[7,4]   #Define the window size ratio for a 2D plot
#----------------------------------------------------------------------------------------#
#                                                                                        #
#                                    Process data set                                    #
#                                           1                                            #
#----------------------------------------------------------------------------------------#


################ Insitu PDF only

sample_name = 'fc_zns_10cm_water' #filename
path_to_Temp = rootdir + "/processed/" + sample_name + "/Reload_files/meas_Reload/new_counter_" + sample_name + ".mat"


# sample_name = 'fc_zns_1cm_3' #filename
# path_to_Temp = '/asap3/petra3/gpfs/p21.1/2023/data/11017732/raw/pe/ac_cell/' + sample_name + "/"
# sample_name = 'zns_2p_10_2' #filename
# path_to_Temp = rootdir + "/processed/" + sample_name + "/Reload_files/meas_Reload/new_counter_zns_2p_10_2.mat"
path =  rootdir + "/processed/" + sample_name + "/PDF_files/meas_type1_uncorrected_PDF/" #There exsist 3 methods to generate the PDF 
# path =  rootdir + "/processed/" + sample_name + "/PDF_files/meas_type2_corrected_prior_PDF/"
# path =  rootdir + "/processed/" + sample_name + "/PDF_files/meas_type3_corrected_after_PDF/"
meas = PDF_class.InsituPDF(path=path, sampleName=sample_name, startScan=startScan, endScan=endScan,
                             timeOfset=0, tempPath=path_to_Temp, average=average,
                            figratio1D=figratio1D,figratio2D=figratio2D)
#meas.saveReload(rootdir + "processed/Reload")

# Waterfall like 1D plot
meas.plot1dvsTimeProjectedOntop(offsetTime=0,savefig="svg",outputPath=rootdir + "/processed/" + sample_name + "//PDF_Plots/PDF_1D_projected_plot/",jump=step_lenght, minR=1.65, maxR=10, intercept=0.2) #plot in 1D, insitu=True will define color change and hides ledgend
# 1D plot over time with temperature label
meas.plot1dvsTime(offsetTime=0,savefig="svg",outputPath=rootdir + "/processed/" + sample_name + "/PDF_Plots/PDF_1D_plot/",jump=step_lenght, minR=1, maxR=10) #plot in 1D, insitu=True will define color change and hides ledgend
# 2D plot over time with temperature curve
meas.plot2dTemp(vmin=-0.5,vmax=1.5,savefig="svg",outputPath= rootdir + '/processed/' + sample_name + '/PDF_Plots/PDF_2D_plot/', maxR=10) #rect=rectangle)
