import matplotlib.pyplot as plt
import matplotlib.patches as patches
from matplotlib.widgets import CheckButtons
import numpy as np
from scipy.stats import norm
import scipy.constants as const
import os


import pathlib
rootdir = str(pathlib.Path(__file__).parent.resolve())
plt.style.use(rootdir + '/../settings/style.mplstyle')

try:
    from backbone.functions.utils import ensure_dir
    import backbone.settings.Colormap as cmp
except:
    from python_scripts.backbone.functions.utils import ensure_dir
    import python_scripts.backbone.settings.Colormap as cmp



class Plottin_PDF:



    def plot1dvsTime(self, offsetTime = 0, title: str = None,savefig: str = None,outputPath=None, jump=1, maxR=None, minR=None, getEndtime=False, fix_ratios=False):
        """
        Arguments
        ----------
        startTime:  Time the first plot is take from  
        offsetTime: Difine specific color for plot
        title:      Give plot title
        savefig:    When True saves the plot as a png
        outoutPath: Path to output directory
        jump:       step width
        maxR:       Upper border of the plot in Angström
        minR:       Lower border of the plot in Angström
        """
        if outputPath != None:
            ensure_dir(outputPath)
        if title == None:
            title = self.sampleName

        fig, ax = plt.subplots()

        #define maxima and minima for energy and intensity for ploting purpose
        self.max_x = max(self.r)
        self.min_x = min(self.r)
        for i in self.G:
            if max(i) > self.max_all_int:
                self.max_all_int = max(i)
            if min(i) < self.min_all_int:
                self.min_all_int = min(i)

        
        colors = plt.cm.jet(np.linspace(0, 1, len(self.G)))
        for i,j in enumerate(self.G): #insitu plot
            if i % jump == 0:
                
                # colormapRB = cmp.cmRB_PDF(int(self.time_averaged_array[-1])-startTime)
                # newcmp = ListedColormap(colormapRB(np.linspace(0,1,int(self.time_averaged_array[-1])-startTime)))
                # colormapRB = colormapRB(np.linspace(0,1,int(self.time_averaged_array[-1])-startTime))
                

                # for g in range(0,startTime,1):
                #     colormapRB = np.vstack(([[0.0, 0.0, 1.0, 1.0]],colormapRB))


                if len(self.G) % 2 ==0:
                    ramp = np.linspace(3,3/2,int(len(self.G)/2)) 
                    thickness = np.concatenate((ramp,ramp[::-1])) 
                else:
                    ramp = np.linspace(3,3/2,int(len(self.G)/2)) 
                    thickness = np.concatenate((np.concatenate((ramp,np.array([3/2]))),ramp[::-1])) 


                if self.time_averaged_array[i] < offsetTime:
                    prelabel = "pre "
                else:
                    prelabel = ""

                ax.plot(self.r, j, linewidth=thickness[i], color=colors[i], zorder=-i, label=prelabel + str(int(self.temp_averaged_array[i])) + " °C", alpha=1) 
                plt.legend(prop={'size': 10}, ncol=3,loc='upper right').get_frame().set_linewidth(0.0)

        sm = plt.cm.ScalarMappable(cmap='jet', norm=plt.Normalize(vmin=int(self.time_averaged_array[jump]), vmax=int(self.time_averaged_array[-1])))
        if getEndtime == True:
            print(self.time_averaged_array[-1])
        cb = plt.colorbar(sm, ax=plt.gca())
        cb.set_label(label='time [min]')


        ax.set_title(title)    
        ax.set_xlabel("r [$\\rm{\AA}$]")
        ax.set_ylabel("G [$\\rm{\AA^{-2}}$]")

        if maxR != None and minR !=None:
            plt.xlim(minR,maxR)
        elif maxR != None and minR == None:
            plt.xlim(self.min_x,maxR)
        elif minR != None and maxR == None:
            plt.xlim(minR,self.max_x)
        elif maxR == None and minR == None:
            plt.xlim(self.min_x,self.max_x)
        plt.ylim()
        if fix_ratios == False:
            plt.tight_layout()
        if savefig == "png":
            plt.savefig(outputPath + "/1d_"+ self.sampleName +".png", dpi=600)
        if savefig == "svg":
            plt.savefig(outputPath + "/1d_"+ self.sampleName +".svg", dpi=600)
        plt.show()


    def plot1dvsTimeProjectedOntop(self, offsetTime = 0, title: str = None,savefig: str = None,outputPath=None, jump=1, maxR=None, minR=None, getEndtime=False, intercept: float=0.3, fix_ratios=False):
        """
        Arguments
        ----------
        startTime:  Time the first plot is take from  
        offsetTime: Difine specific color for plot
        title:      Give plot title
        savefig:    When True saves the plot as a png
        outoutPath: Path to output directory
        jump:       step width
        maxR:       Upper border of the plot in Angström
        minR:       Lower border of the plot in Angström
        """
        if outputPath != None:
            ensure_dir(outputPath)
        if title == None:
            title = self.sampleName

        fig, ax = plt.subplots()

        #define maxima and minima for energy and intensity for ploting purpose
        self.max_x = max(self.r)
        self.min_x = min(self.r)
        for i in self.G:
            if max(i) > self.max_all_int:
                self.max_all_int = max(i)
            if min(i) < self.min_all_int:
                self.min_all_int = min(i)

        height = 0
        colors = plt.cm.jet(np.linspace(0, 1, len(self.G)))
        for i,j in enumerate(self.G): #insitu plot
            if i % jump == 0:
                
                # colormapRB = cmp.cmRB_PDF(int(self.time_averaged_array[-1])-startTime)
                # newcmp = ListedColormap(colormapRB(np.linspace(0,1,int(self.time_averaged_array[-1])-startTime)))
                # colormapRB = colormapRB(np.linspace(0,1,int(self.time_averaged_array[-1])-startTime))
                

                # for g in range(0,startTime,1):
                #     colormapRB = np.vstack(([[0.0, 0.0, 1.0, 1.0]],colormapRB))


                if len(self.G) % 2 ==0:
                    ramp = np.linspace(3,3/2,int(len(self.G)/2)) 
                    thickness = np.concatenate((ramp,ramp[::-1])) 
                else:
                    ramp = np.linspace(3,3/2,int(len(self.G)/2)) 
                    thickness = np.concatenate((np.concatenate((ramp,np.array([3/2]))),ramp[::-1])) 
                if self.time_averaged_array[i] < offsetTime:
                    prelabel = "pre "
                else:
                    prelabel = ""

                ax.plot(self.r, j+height, linewidth=thickness[i], color=colors[i], zorder=-i, label=prelabel + str(int(self.temp_averaged_array[i])) + " °C", alpha=1) 
                plt.legend(prop={'size': 10}, ncol=3,loc='upper right').get_frame().set_linewidth(0.0)
                height = height + max(j) + intercept*abs(min(j))

        sm = plt.cm.ScalarMappable(cmap='jet', norm=plt.Normalize(vmin=int(self.time_averaged_array[jump]), vmax=int(self.time_averaged_array[-1])))
        if getEndtime == True:
            print(self.time_averaged_array[-1])
        cb = plt.colorbar(sm, ax=plt.gca())
        cb.set_label(label='time [min]')


        ax.set_title(title)    
        ax.set_xlabel("r [$\\rm{\AA}$]")
        ax.set_ylabel("G [$\\rm{\AA^{-2}}$]")

        if maxR != None and minR !=None:
            plt.xlim(minR,maxR)
        elif maxR != None and minR == None:
            plt.xlim(self.min_x,maxR)
        elif minR != None and maxR == None:
            plt.xlim(minR,self.max_x)
        elif maxR == None and minR == None:
            plt.xlim(self.min_x,self.max_x)
        plt.ylim(top=height+3)
        if fix_ratios == False:
            plt.tight_layout()
        if savefig == "png":
            plt.savefig(outputPath + "/1d_"+ self.sampleName +".png", dpi=600)
        if savefig == "svg":
            plt.savefig(outputPath + "/1d_"+ self.sampleName +".svg", dpi=600)
        plt.show()




    #----------------------------------------------------------------------------------------#
    #                             plot data in 2D with temp curve                            #
    #----------------------------------------------------------------------------------------#
    def plot2dTemp(self,vmin=None,vmax=None,savefig: str = None,outputPath=None, rect=None, minR = None, maxR = None, fix_ratios=False):
        """
        Arguments
        ----------
        vmin:       Intensity the colormap should start
        vmax:       Intensity the colormap should end
        savefig:    save the plot as a png or svg
        outoutPath: Path to output directory
        rect:       Dictionary of the rectangle properties that you can use to mark a certain area in the 2D plot
        """
        if outputPath != None:
            ensure_dir(outputPath)
        #set some plotting settings
        newcmp = cmp.cm2d()

        #fig = plt.figure(figsize=(32/5, 4)) 
        fig, (ax1, ax2) = plt.subplots(1, 2, gridspec_kw={'width_ratios': [1, 4]},figsize=(self.figratio2D[0], self.figratio2D[1]))
        #ax = plt.gca()
        plt.subplots_adjust(wspace = 0.075 )

        if self.reload == None:
            time = self.time_averaged_array[self.startScan:self.endScan]
            temp = self.temp_averaged_array[self.startScan:self.endScan]
        else:
            time = self.time_averaged_array
            temp= self.temp_averaged_array

        ax1.plot(temp,time, color="red")
        ax1.set(xlabel = 'T [°C]', ylabel="time [min]",ylim = (min(time),max(time)), xlim=(0,200))
        ax1.tick_params(axis='x',which="major", direction="out")
        ax1.set_xticks([0,100,200])

        x = self.r
        if self.reload == None:
            y = self.time_averaged_array[self.startScan:self.endScan]
        else:
            y = self.time_averaged_array
        z = self.G


        imap = ax2.imshow(z, origin = 'lower', aspect = 'auto', 
                extent = [min(x), max(x), min(y), max(y)], vmin = vmin, vmax = vmax, 
                cmap=newcmp, interpolation='gaussian')

        # imap = ax2.contourf(x,y,z, origin = 'lower', levels=np.linspace(vmin, vmax, 1000), vmin = vmin, vmax = vmax, cmap=newcmp,extend='both')

       
        if rect != None:
            rect = patches.Rectangle((rect["minR"], rect["mintime"]), (rect["maxR"]-rect["minR"]), (rect["maxtime"]-rect["mintime"]), linewidth=3/2, edgecolor='r', facecolor='none',zorder=100)
            ax2.add_patch(rect)

        ax2.set_xticks(np.arange(round(min(x),1),round(max(x)+1,2), 5))
        if maxR != None and minR !=None:
                ax2.set(xlim = (minR,maxR), xlabel = 'r [$\\rm{\AA}$]')
        elif minR != None and maxR == None:
            ax2.set(xlim = (minR,max(x)), xlabel = 'r [$\\rm{\AA}$]')
        elif maxR != None and minR == None:
            ax2.set(xlim = (min(x),maxR), xlabel = 'r [$\\rm{\AA}$]')
        elif maxR == None and minR == None:
            ax2.set(xlim = (min(x),max(x)), xlabel = 'r [$\\rm{\AA}$]')

        ax2.tick_params(axis="y",left=False,labelleft=False)
        ax2.tick_params(axis='both',which="major", direction='out')
        ax2.tick_params(axis='x',which="minor", width=1, length=3, labelsize=15, direction='out')
        ax2.set_title(self.sampleName)
        


        cb = plt.colorbar(imap, extendrect=True)
        cb.ax.tick_params(axis='both',which="both", width=1.7, length=3.5, labelsize=15, direction="in") 
        cb.ax.tick_params(axis='x',which="minor", width=1, length=2, labelsize=15,direction="in") 
        cb.set_label(label='G [$\\rm{\AA^{-2}}$]')

        if fix_ratios == False:
            plt.tight_layout()
        if savefig == "png":
            plt.savefig(outputPath + "/2d_"+ self.sampleName +".png", dpi=600)
        if savefig == "svg":
            plt.savefig(outputPath + "/2d_"+ self.sampleName +".svg", dpi=600)
        plt.show()        


    #----------------------------------------------------------------------------------------#
    #                                    plot data in 3D                                     #
    #----------------------------------------------------------------------------------------#
    def plot3d(self, vmin= None, vmax=None, elev=None, azim=None,savefig: str = None,outputPath=None, fix_ratios=False):  
        """
        Arguments
        ----------
        vmin:       Intensity the colormap should start
        vmax:       Intensity the colormap should end
        elev:       Elevation angle 0 equals ground or 1d side perspective
        azim:       Azimutal angel -90 equals 1d front perspective
        savefig:    save the plot as a png or svg
        outoutPath: Path to output directory
        """
        if outputPath != None:
            ensure_dir(outputPath)
        from matplotlib.ticker import LinearLocator, FormatStrFormatter  
        fig = plt.figure()
        ax = fig.gca(projection='3d')
        fig.subplots_adjust(top=1.1, bottom=-.1)


        newcmp = cmp.cm2d()
        # Plot the surface.
        x0, x1 = np.meshgrid(np.array(self.r), np.array(self.time_array))
        surf = ax.plot_surface(x0, x1, np.array(self.G), cmap=newcmp,
                            linewidth=-3, antialiased=False, rstride=1, cstride=1, alpha=1, vmin=vmin, vmax=vmax)

        # Customize the z axis.
        # ax.set_zlim(-1.01, 1.01)
        ax.zaxis.set_major_locator(LinearLocator(10))
        ax.zaxis.set_major_formatter(FormatStrFormatter('%.02f'))

        #ax.set_zticks([])
        ax.set_xlabel("Energy [keV]")
        ax.set_ylabel("time [min]")#"time [s]")
        ax.set_zlabel("Intensity [arb. units]")
        ax.view_init(elev=elev, azim=azim)

        # Add a color bar which maps values to colors.
        # fig.colorbar(surf, shrink=0.5, aspect=5)

        if fix_ratios == False:
            plt.tight_layout()
        if savefig == "png":
            plt.savefig(outputPath + "/3d_"+ self.sampleName +".png", dpi=600)
        if savefig == "svg":
            plt.savefig(outputPath + "/3d_"+ self.sampleName +".svg", dpi=600)
        plt.show()


class Plotting_XRD:
    
    #----------------------------------------------------------------------------------------#
    #                             plot data in 2D with temp curve                            #
    #----------------------------------------------------------------------------------------#
    def plot2dTemp(self,vmin=None,vmax=None,savefig: str = None,outputPath=None, rect=None, maxTemp: float =200, minTemp: float =0, fix_ratios=False):
        """
        Arguments
        ----------
        vmin:       Intensity the colormap should start
        vmax:       Intensity the colormap should end
        savefig:    save the plot as a png or svg
        outoutPath: Path to output directory
        rect:       Dictionary of the rectangle properties that you can use to mark a certain area in the 2D plot
        """
        if outputPath != None:
            ensure_dir(outputPath)
        else:
            outputPath = self.path +'../processed/Plot_2D_Temp/' 
            ensure_dir(outputPath)
        #set some plotting settings
        for i in self.I:
            if max(i) > self.max_all_int:
                self.max_all_int = max(i)
            if min(i) < self.min_all_int:
                self.min_all_int = min(i)
        if vmin == None:
            vmin = self.max_all_int
        if vmax == None:
            vmax = self.min_all_int
        newcmp = cmp.cm2d()

        #fig = plt.figure(figsize=(32/5, 4)) 
        fig, (ax1, ax2) = plt.subplots(1, 2, gridspec_kw={'width_ratios': [1, 4]},figsize=(self.figratio2D[0], self.figratio2D[1]))
        #ax = plt.gca()
        plt.subplots_adjust(wspace = 0.075 )


        time = self.time_array
        temp = self.temp_array

        ax1.plot(temp,time, color="red")
        ax1.set(xlabel = 'T [°C]', ylabel="Time [min]",ylim = (min(time),max(time)), xlim=(minTemp,maxTemp)) 
        ax1.tick_params(axis='x',which="major", direction="out")

        x = self.q
        y = self.time_array
        z = self.I


        imap = ax2.imshow(z, origin = 'lower', aspect = 'auto', 
                extent = [min(x), max(x), min(y), max(y)], vmin = vmin, vmax = vmax, 
                cmap=newcmp, interpolation='gaussian')

       
        if rect != None:
            rect = patches.Rectangle((rect["minR"], rect["mintime"]), (rect["maxR"]-rect["minR"]), (rect["maxtime"]-rect["mintime"]), linewidth=self.linewidth/2, edgecolor='r', facecolor='none',zorder=100)
            ax2.add_patch(rect)

        ax2.set(xlim = (min(x),max(x)), xlabel = 'q [$\\rm{\AA^{-1}}$]')
        ax2.tick_params(axis='x',which="major", direction="out")
        ax2.tick_params(axis="y",left=False,labelleft=False)
        ax2.set_title(self.sampleName)


        cb = plt.colorbar(imap)
        cb.set_label(label='I [arb. un.]')


        if fix_ratios == False:
            plt.tight_layout()
        if savefig == "png":
            plt.savefig(outputPath + "/2d_"+ self.sampleName +".png", dpi=600)
        if savefig == "svg":
            plt.savefig(outputPath + "/2d_"+ self.sampleName +".svg", dpi=600)
        plt.show()        


    #----------------------------------------------------------------------------------------#
    #                              Plot 1d Data vs Temp and Time                             #
    #----------------------------------------------------------------------------------------#
    def plot1dvsTime(self,startTime = -1, offsetTime = 0, title: str = None,savefig: str = None,outputPath=None, step=1, maxQ=None, minQ=None, plot_temp: bool=False,min_index: int=0, max_index: int=-1, fix_ratios=False):
        """
        Arguments
        ----------
        startTime:  Time the first plot is take from  
        offsetTime: Difine specific color for plot
        title:      Give plot title
        savefig:    When True saves the plot as a png
        outoutPath: Path to output directory
        jump:       step width
        maxR:       Upper border of the plot in Angström
        minR:       Lower border of the plot in Angström
        """
        if outputPath != None:
            ensure_dir(outputPath)
        else:
            ensure_dir(outputPath)
            outputPath = self.path +'/plot/' 
        if title == None:
            title = self.sampleName

        plt.figure() #
        ax = plt.gca()     

        #define maxima and minima for energy and intensity for ploting purpose
        self.max_x = max(self.q)
        self.min_x = min(self.q)
        for i in self.I:
            if max(i) > self.max_all_int:
                self.max_all_int = max(i)
            if min(i) < self.min_all_int:
                self.min_all_int = min(i)


        
        colormapRB = cmp.cmRB_Time(10000)
        colors = colormapRB(np.linspace(0,1,len(self.I[min_index:max_index])))
        for i,j in enumerate(self.I[min_index:max_index]): #insitu plot
            if i % step == 0:


                if len(self.I) % 2 ==0:
                    ramp = np.linspace(3,3,int(len(self.I)/2)) 
                    thickness = np.concatenate((ramp,ramp[::-1])) 
                else:
                    ramp = np.linspace(3,3/2,int(len(self.I)/2)) 
                    thickness = np.concatenate((np.concatenate((ramp,np.array([3/2]))),ramp[::-1])) 

                if self.time_array[i] < offsetTime:
                    prelabel = "pre "
                else:
                    prelabel = ""

                # ax.loglog(self.q,j)

                ax.plot(self.q, j, linewidth=thickness[i], color=colors[i], zorder=len(self.I)-i, label=prelabel + str(int(self.temp_array[int(i*self.average)])) + " °C", alpha=1) 
                if plot_temp == True:
                    plt.legend(prop={'size': 10}, ncol=3,loc='upper right', handlelength=1.6).get_frame().set_linewidth(0.0)
       
        if max_index != -1:
            max_index_time = int(max_index*self.average)
        else:
            max_index_time = -1
        min_index_time = int(min_index*self.average)
        #sm = plt.cm.ScalarMappable(cmap=colormapRB, norm=plt.Normalize(vmin=int(self.time_array[min_index_time]), vmax=int(self.time_array[max_index_time])-startTime))
        sm = plt.cm.ScalarMappable(cmap='jet', norm=plt.Normalize(vmin=int(self.time_array[min_index_time]), vmax=int(self.time_array[max_index_time])-startTime))
        cb = plt.colorbar(sm, ax=plt.gca())
        cb.set_label(label='Time [min]')



        ax.set_title(title)    
        ax.set_xlabel("q [$\\rm{\AA^{-1}}$]")
        ax.set_ylabel("I [$\\rm{arb. un.}$]")

        if maxQ != None and minQ !=None:
            plt.xlim(minQ,maxQ)
        elif maxQ != None and minQ == None:
            plt.xlim(self.min_x,maxQ)
        elif minQ != None and maxQ == None:
            plt.xlim(minQ,self.max_x)
        elif maxQ == None and minQ == None:
            plt.xlim(self.min_x,self.max_x)
        plt.ylim(bottom=0)
        if fix_ratios == False:
            plt.tight_layout()
        if savefig == "png":
            plt.savefig(outputPath + "/1d_"+ self.sampleName +".png", dpi=600)
        if savefig == "svg":
            plt.savefig(outputPath + "/1d_"+ self.sampleName +".svg", dpi=600)
        plt.show()

    #----------------------------------------------------------------------------------------#
    #                                    Check the phases                                    #
    #----------------------------------------------------------------------------------------#
    def checkPhases(self,scan_number: int,references: dict, energy: float=100 , savefig: str = None, outputPath=None,  maxQ=None, minQ=None):
        """
        Arguments
        ----------
        outputPath: Destination of the .log file
        """
        q_ref_array = []
        I_ref_array = []
        Xray_energys = []

    
        max_I = 0

        def q_to_two_theta(q,Energy=100):
            """transforms a Q value into an two theta angle

            Args:
                q (float): cuttof of pattern in Q 
                Energy (float, optional): Energy used in keV. Defaults to 101.

            Returns:
                two theta: returns two theta <= 180
            """
            w = (const.h*const.c/(Energy*1e3*const.electron_volt))
            if w*q*1e10/(4*np.pi) >= 1:
                return 180
            else:
                return np.rad2deg(2*np.arcsin(w*q*1e10/(4*np.pi)))

        def bars_to_gaussian(x_data, y_data, sigma=0.1):
            """
            Convert bar plot data (x_data, y_data) to a plot with Gaussian functions at bar positions.

            Parameters:
            - x_data: X-axis data (list or array)
            - y_data: Y-axis data (list or array)
            - sigma: Standard deviation of the Gaussian functions (default is 0.1)

            Returns:
            - x: q array
            - y: Intensity(q) array
            
            """
            sum = 0
            x = np.linspace(0,max(x_data)+1,10000)
            # Iterate through bar positions and heights to plot Gaussian functions
            for position, height in zip(x_data, y_data):
                # x = np.linspace(position - 3 * sigma, position + 3 * sigma, 100)
                y = height * norm.pdf(x, position, sigma)
                sum += y
            sum = sum / max(sum)
            return x,sum
        

        import xrayutilities as xu


        for i in references:
            print(references[i]['path'])
            xu_crystal = xu.materials.Crystal.fromCIF(os.path.join('data', references[i]['path']))
            pst = xu.simpack.PowderDiffraction(xu_crystal, en=energy*1e3, tt_cutoff=q_to_two_theta(max(self.q),Energy=energy))

            qpos = []
            ang = []
            r = []

            for i in pst.data.values():
                qpos_i = i["qpos"]
                ang_i = i["ang"]
                r_i = i["r"]
                qpos.append(qpos_i)
                ang.append(ang_i)
                r.append(r_i)

            q_ref, I_ref = bars_to_gaussian(qpos,r, sigma=0.04)
            if max_I < max(I_ref):
                max_I = max(I_ref)
            q_ref_array.append(q_ref)
            I_ref_array.append(I_ref)



        # def teta_to_q(teta,Energy):
        #     wavelenght = (6.62607015e-34*299792458*1e10)  / (Energy*1e3*1.602176634e-19)
        #     return ((4*np.pi*np.sin(np.deg2rad(teta)/2))/wavelenght)

        # for i in references:
        #     q_ref, I_ref, unimp = np.loadtxt(references[i]['path'],skiprows=references[i]['skiprows']).T
        #     q_ref_array.append(teta_to_q(q_ref,references[i]['energy']))
        #     I_ref_array.append(I_ref)

        keys = [k for k, v in references.items()]
        fig, ax = plt.subplots(figsize=(self.figratio2D[0]*1.3, self.figratio2D[1]*1.3))
        
        lns = []
        for k in range(len(I_ref_array)):
            exec(f'l{k}, = ax.plot(q_ref_array[k],I_ref_array[k]/max_I*0.5-0.5, label=keys[k])')
            exec(f'lns.append(l{k})')
        try:
            ax.plot(self.q,self.I[scan_number]/max(self.I[scan_number]))
        except:
            raise Exception('scan_number higher than the number of scans after averageing; Please select a smaller scan_number than: ' + str(len(self.I)))
        ax.set_title('XRD phase check of ' + self.sampleName)    
        ax.set_xlabel("q [$\\rm{\AA^{-1}}$]")
        ax.set_ylabel("I [$\\rm{arb. un.}$]")
        if maxQ != None and minQ !=None:
            plt.xlim(minQ,maxQ)
        elif maxQ != None and minQ == None:
            plt.xlim(min(self.q),maxQ)
        elif minQ != None and maxQ == None:
            plt.xlim(minQ,max(self.q))
        elif maxQ == None and minQ == None:
            plt.xlim(min(self.q),max(self.q))
        plt.ylim(bottom=-0.5)
        plt.tight_layout()
        fig.subplots_adjust(right=0.7)

        lines_by_label = {l.get_label(): l for l in lns}
        line_colors = [l.get_color() for l in lines_by_label.values()]

        # Make checkbuttons with all plotted lines with correct visibility
        rax = fig.add_axes([0.71, 0.5, 0.28, 0.07*len(I_ref_array)])
        check = CheckButtons(
            ax=rax,
            labels=lines_by_label.keys(),
            actives=[l.get_visible() for l in lines_by_label.values()],
            label_props={'color': line_colors},
            frame_props={'edgecolor': line_colors},
            check_props={'facecolor': line_colors},
            )
        def callback(label):
            ln = lines_by_label[label]
            ln.set_visible(not ln.get_visible())
            ln.figure.canvas.draw_idle()

        check.on_clicked(callback)
        plt.show()
