import numpy as np
from time import time
import os, shutil

#----------------------------------------------------------------------------------------#
#                  Ensures a output directory will be created on demand                  #
#----------------------------------------------------------------------------------------#

def ensure_dir(dir_path):
    """
    Arguments
    ----------
    dir_path:   Path to directory   
    """ 
    import os
    if not os.path.exists(dir_path):
        os.makedirs(dir_path)

#----------------------------------------------------------------------------------------#
#              Find closest related elemt to a given element in array                    #
#----------------------------------------------------------------------------------------#
def find_nearest(array, value):
    """Find element in array nearest to input value.  

    Args:
        array (array): Array in which to search.
        value (float): Value to search for in array.

    Returns:
        float: Value in array which is nearest to the input value.
    """
    array = np.asarray(array)
    idx = (np.abs(array - value)).argmin()
    return array[idx]

#----------------------------------------------------------------------------------------#
#                                   Get processing time                                  #
#----------------------------------------------------------------------------------------#
def processTime(t0):
    """
    Arguments
    ----------
    t0: Start time
    """
    print("\n Time for processing")
    print("------------------------------------------------------------------")
    print("Time: " + str((time() - t0)/60))


#----------------------------------------------------------------------------------------#
#                     round to a multiple, to a value higher then 1                      #
#----------------------------------------------------------------------------------------#
def round_to_multiple(number,multiple):
    """
    Arguments
    ----------
    number: unrounded number
    multiple: number to round to
    """
    return multiple * round(number/multiple)


#----------------------------------------------------------------------------------------#
#                                   get array from file                                   #
#----------------------------------------------------------------------------------------#
def getIfile(file):
    """
    Arguments
    ----------
    file: path to array file
    """
    q_file, j = np.loadtxt(file,skiprows=1).T
    return j


def averageArray(array,average: int, dimension: int =1):
    """_summary_

    Args:
        array (array): Input array (or matrix)
        average (int): Number of elements to average over
        dimension (int, optional): Dimension is 1 for vector like array and 2 for matrix like array. Defaults to 1.

    Returns:
        array: Averaged array (1 or 2 dimensional)
    """
    #Create new list for array
    new_array = []
    
    #Create a array for the average array
    if dimension == 1:
        array_averaged = 0
    if dimension == 2:
        array_averaged = np.zeros(len(array[0]))
    
    #Enumerate over all scans
    for i,j in enumerate(array):
        
        #Add up array of each scan 
        array_averaged = array_averaged + j
        
        #Store the averaged array when the averaging is reached and
        # normalize again to the average
        if (i+1) % average == 0 and i > 0:
            array_averaged = array_averaged/average  
            new_array.append(array_averaged)              
            if dimension == 1:
                array_averaged = 0
            if dimension == 2:
                array_averaged = np.zeros(len(array[0]))
            
    #Update array to the averaged data
    return new_array

def averageArrayError(array,average: int, dimension: int =1):
    """_summary_

    Args:
        array (array): Input array (or matrix)
        average (int): Number of elements to average over
        dimension (int, optional): Dimension is 1 for vector like array and 2 for matrix like array. Defaults to 1.

    Returns:
        array: Averaged array (1 or 2 dimensional)
    """
    #Create new list for array
    new_array = []
    
    #Create a array for the average array
    if dimension == 1:
        array_averaged = 0
    if dimension == 2:
        array_averaged = np.zeros(len(array[0]))
    
    #Enumerate over all scans
    for i,j in enumerate(array):
        
        #Add up the squared error in each array of each scan 
        array_averaged = array_averaged + j**2
        
        #Store the averaged array when the averaging is reached and
        # normalize again to the root of the average minus 1
        if (i+1) % average == 0 and i > 0:
            array_averaged = np.sqrt(array_averaged/ average)  
            new_array.append(array_averaged)              
            if dimension == 1:
                array_averaged = 0
            if dimension == 2:
                array_averaged = np.zeros(len(array[0]))
            
    #Update array to the averaged data
    return new_array

def del_folder_content(dir_path):
    for filename in os.listdir(dir_path):
        file_path = os.path.join(dir_path, filename)
        try:
            if os.path.isfile(file_path) or os.path.islink(file_path):
                os.unlink(file_path)
            elif os.path.isdir(file_path):
                shutil.rmtree(file_path)
        except Exception as e:
            print('Failed to delete %s. Reason: %s' % (file_path, e))